@@ -1,5 +1,4 @@
 # Copyright (C) 2006 CSC-Scientific Computing Ltd.
-
 # Please see the accompanying LICENSE file for further information.
 from __future__ import print_function
 import os
@@ -8,66 +7,12 @@ import sys
 import re
 import distutils.util
 from distutils.sysconfig import get_config_var, get_config_vars
+from distutils.version import LooseVersion
 from glob import glob
 from os.path import join
 from stat import ST_MTIME
 
 
-def check_packages(packages, msg, include_ase, import_numpy):
-    """Check the python version and required extra packages
-
-    If ASE is not installed, the `packages` list is extended with the
-    ASE modules if they are found."""
-
-    if sys.version_info < (2, 3, 0, 'final', 0):
-        raise SystemExit('Python 2.3.1 or later is required!')
-
-    if import_numpy:
-        try:
-            import numpy
-        except ImportError:
-            raise SystemExit('numpy is not installed!')
-    else:
-        msg += ['* numpy is not installed.',
-                '  "include_dirs" in your customize.py must point to '
-                '"numpy/core/include".']
-
-    if not include_ase:
-        if import_numpy:
-            try:
-                import ase
-            except ImportError:
-                import_ase = True
-            else:
-                import_ase = False
-        else:
-            import_ase = False
-
-    if include_ase or import_ase:
-        # Find ASE directories:
-        # include_ase works in case:
-        # cd gpaw # top-level gpaw source directory
-        # tar zxf ~/python-ase-3.1.0.846.tar.gz
-        # ln -s python-ase-3.1.0.846/ase .
-        ase_root = 'ase'
-        if include_ase:
-            assert os.path.isdir(ase_root), ase_root
-        ase = []
-        for root, dirs, files in os.walk(ase_root):
-            if 'CVS' in dirs:
-                dirs.remove('CVS')
-            if '.svn' in dirs:
-                dirs.remove('.svn')
-            if '__init__.py' in files:
-                ase.append(root.replace('/', '.'))
-
-        if len(ase) == 0:
-            msg += ['* ASE is not installed!  You may be able to install',
-                    "  gpaw, but you can't use it without ASE!"]
-        else:
-            packages += ase
-
-            
 def find_file(arg, dir, files):
     # looks if the first element of the list arg is contained in the list files
     # and if so, appends dir to to arg. To be used with the os.path.walk
@@ -78,7 +23,7 @@ def find_file(arg, dir, files):
 def get_system_config(define_macros, undef_macros,
                       include_dirs, libraries, library_dirs, extra_link_args,
                       extra_compile_args, runtime_library_dirs, extra_objects,
-                      msg, import_numpy):
+                      import_numpy):
 
     undef_macros += ['NDEBUG']
     if import_numpy:
@@ -112,14 +57,12 @@ def get_system_config(define_macros, undef_macros,
         # http://forum.java.sun.com/thread.jspa?threadID=5072537&messageID=9265782
         extra_link_args += ['-Bstatic', '-lsunperf', '-lfsu', '-Bdynamic']
         cc_version = os.popen3('cc -V')[2].readline().split()[3]
-        if cc_version > '5.6':
+        if LooseVersion(cc_version) > '5.6':
             libraries.append('mtsk')
         else:
             extra_link_args.append('-lmtsk')
         # define_macros.append(('NO_C99_COMPLEX', '1'))
 
-        msg += ['* Using SUN high performance library']
-
     elif sys.platform.startswith('win'):
 
         # We compile with mingw coming from pythonyx (32-bit)
@@ -140,8 +83,6 @@ def get_system_config(define_macros, undef_macros,
                 break
         if lib == 'openblas':
             libraries += [lib, 'gfortran']
-        if lib:
-            msg += ['* Using %s library from %s' % (lib, directory)]
 
     elif sys.platform in ['aix5', 'aix6']:
 
@@ -159,14 +100,30 @@ def get_system_config(define_macros, undef_macros,
         libraries += ['f', 'lapack', 'essl']
         define_macros.append(('GPAW_AIX', '1'))
 
-    elif machine in ['x86_64', 'ppc64']:
+    elif sys.platform == 'darwin':
+
+        extra_compile_args += ['-Wall', '-Wno-unknown-pragmas', '-std=c99']
+        include_dirs += ['/usr/include/malloc']
+        library_dirs += ['/usr/local/lib']   # libxc installed with Homebrew
+        extra_link_args += ['-framework', 'Accelerate']  # BLAS
+
+        # If the user uses python 3 from homebrew, then installation would
+        # fail due to a broken LINKFORSHARED configuration variable
+        cfgDict = get_config_vars()
+        if ('LINKFORSHARED' in cfgDict and
+            'Python.framework/Versions/3.6' in cfgDict['LINKFORSHARED']):
+            cfgDict['LINKFORSHARED'] = '-l python3.6'
+            
+        # We should probably add something to allow for user-installed BLAS?
+
+    elif machine in ['x86_64', 'ppc64', 'ppc64le', 'aarch64', 's390x']:
 
         #    _
         # \/|_||_    |_ |_|
         # /\|_||_| _ |_|  |
         #
 
-        extra_compile_args += ['-Wall', '-std=c99']
+        extra_compile_args += ['-Wall', '-Wno-unknown-pragmas', '-std=c99']
 
         # Look for ACML libraries:
         acml = glob('/opt/acml*/g*64/lib')
@@ -178,7 +135,6 @@ def get_system_config(define_macros, undef_macros,
             if acml[-1].find('gnu') != -1:
                 libraries.append('g2c')
             extra_link_args += ['-Wl,-rpath=' + acml[-1]]
-            msg += ['* Using ACML library']
         else:
             atlas = False
             for dir in ['/usr/lib', '/usr/local/lib', '/usr/lib64/atlas']:
@@ -199,9 +155,8 @@ def get_system_config(define_macros, undef_macros,
                     libdir = dir
                     break
             if openblas:  # prefer openblas
-                libraries += ['openblas']
+                libraries += ['openblas', 'lapack']
                 library_dirs += [libdir]
-                msg += ['* Using OpenBLAS library']
             else:
                 if atlas:  # then atlas
                     # http://math-atlas.sourceforge.net/errata.html#LINK
@@ -210,14 +165,11 @@ def get_system_config(define_macros, undef_macros,
                     # http://math-atlas.sourceforge.net/faq.html#tsafe
                     libraries += ['lapack', 'f77blas', 'cblas', 'atlas']
                     library_dirs += [libdir]
-                    msg += ['* Using ATLAS library']
                 elif satlas:  # then atlas >= 3.10 Fedora/RHEL
                     libraries += ['satlas']
                     library_dirs += [libdir]
-                    msg += ['* Using ATLAS library']
                 else:
                     libraries += ['blas', 'lapack']
-                    msg += ['* Using standard lapack']
 
     elif machine == 'ia64':
 
@@ -230,8 +182,8 @@ def get_system_config(define_macros, undef_macros,
         libraries += ['mkl', 'mkl_lapack64']
 
     elif platform.machine().startswith('arm'):
-        
-        extra_compile_args += ['-Wall', '-std=c99']
+
+        extra_compile_args += ['-Wall', '-Wno-unknown-pragmas', '-std=c99']
 
         atlas = False
         for dir in ['/usr/lib', '/usr/local/lib', '/usr/lib/atlas']:
@@ -254,7 +206,6 @@ def get_system_config(define_macros, undef_macros,
         if openblas:  # prefer openblas
             libraries += ['openblas']
             library_dirs += [libdir]
-            msg += ['* Using OpenBLAS library']
         else:
             if atlas:  # then atlas
                 # http://math-atlas.sourceforge.net/errata.html#LINK
@@ -262,14 +213,11 @@ def get_system_config(define_macros, undef_macros,
                 # http://math-atlas.sourceforge.net/faq.html#tsafe
                 libraries += ['lapack', 'f77blas', 'cblas', 'atlas']
                 library_dirs += [libdir]
-                msg += ['* Using ATLAS library']
             elif satlas:  # then atlas >= 3.10 Fedora/RHEL
                 libraries += ['satlas']
                 library_dirs += [libdir]
-                msg += ['* Using ATLAS library']
             else:
                 libraries += ['blas', 'lapack']
-                msg += ['* Using standard lapack']
 
     elif machine == 'i686':
 
@@ -278,7 +226,7 @@ def get_system_config(define_macros, undef_macros,
         # ||_||_||_|
         #
 
-        extra_compile_args += ['-Wall', '-std=c99']
+        extra_compile_args += ['-Wall', '-Wno-unknown-pragmas', '-std=c99']
 
         if 'MKL_ROOT' in os.environ:
             mklbasedir = [os.environ['MKL_ROOT']]
@@ -293,7 +241,6 @@ def get_system_config(define_macros, undef_macros,
             libraries += ['mkl_lapack',
                           'mkl_ia32', 'guide', 'pthread', 'mkl']
             library_dirs += libs
-            msg += ['* Using MKL library: %s' % library_dirs[-1]]
             # extra_link_args += ['-Wl,-rpath=' + library_dirs[-1]]
         else:
             atlas = False
@@ -317,22 +264,19 @@ def get_system_config(define_macros, undef_macros,
             if openblas:  # prefer openblas
                 libraries += ['openblas']
                 library_dirs += [libdir]
-                msg += ['* Using OpenBLAS library']
             else:
                 if atlas:  # then atlas
                     # http://math-atlas.sourceforge.net/errata.html#LINK
-                    # atlas does not respect OMP_NUM_THREADS - build single-thread
+                    # atlas does not respect OMP_NUM_THREADS - build
+                    # single-thread
                     # http://math-atlas.sourceforge.net/faq.html#tsafe
                     libraries += ['lapack', 'f77blas', 'cblas', 'atlas']
                     library_dirs += [libdir]
-                    msg += ['* Using ATLAS library']
                 elif satlas:  # then atlas >= 3.10 Fedora/RHEL
                     libraries += ['satlas']
                     library_dirs += [libdir]
-                    msg += ['* Using ATLAS library']
                 else:
                     libraries += ['blas', 'lapack']
-                    msg += ['* Using standard lapack']
 
             # add libg2c if available
             g2c = False
@@ -346,25 +290,48 @@ def get_system_config(define_macros, undef_macros,
             if g2c:
                 libraries += ['g2c']
 
-    elif sys.platform == 'darwin':
+    else:
+        extra_compile_args += ['-Wall', '-Wno-unknown-pragmas', '-std=c99']
 
-        extra_compile_args += ['-Wall', '-std=c99']
-        include_dirs += ['/usr/include/malloc']
-
-        if glob('/System/Library/Frameworks/vecLib.framework') != []:
-            extra_link_args += ['-framework vecLib']
-            msg += ['* Using vecLib']
+        atlas = False
+        for dir in ['/usr/lib', '/usr/local/lib', '/usr/lib/atlas']:
+            if glob(join(dir, 'libatlas.so')) != []:
+                atlas = True
+                libdir = dir
+                break
+        satlas = False
+        for dir in ['/usr/lib', '/usr/local/lib', '/usr/lib/atlas']:
+            if glob(join(dir, 'libsatlas.so')) != []:
+                satlas = True
+                libdir = dir
+                break
+        openblas = False
+        for dir in ['/usr/lib', '/usr/local/lib']:
+            if glob(join(dir, 'libopenblas.so')) != []:
+                openblas = True
+                libdir = dir
+                break
+        if openblas:  # prefer openblas
+            libraries += ['openblas']
+            library_dirs += [libdir]
         else:
-            libraries += ['blas', 'lapack']
-            msg += ['* Using standard lapack']
+            if atlas:  # then atlas
+                # http://math-atlas.sourceforge.net/errata.html#LINK
+                # atlas does not respect OMP_NUM_THREADS - build single-thread
+                # http://math-atlas.sourceforge.net/faq.html#tsafe
+                libraries += ['lapack', 'f77blas', 'cblas', 'atlas']
+                library_dirs += [libdir]
+            elif satlas:  # then atlas >= 3.10 Fedora/RHEL
+                libraries += ['satlas']
+                library_dirs += [libdir]
+            else:
+                libraries += ['blas', 'lapack']
 
     # https://listserv.fysik.dtu.dk/pipermail/gpaw-users/2012-May/001473.html
     p = platform.dist()
     if p[0].lower() in ['redhat', 'centos'] and p[1].startswith('6.'):
         define_macros.append(('_GNU_SOURCE', '1'))
 
-    return msg
-
 
 def get_parallel_config(mpi_libraries, mpi_library_dirs, mpi_include_dirs,
                         mpi_runtime_library_dirs, mpi_define_macros):
@@ -392,16 +359,6 @@ def get_parallel_config(mpi_libraries, mpi_library_dirs, mpi_include_dirs,
 
     return mpicompiler
 
-    
-def get_scalapack_config(define_macros):
-    # check ScaLapack settings
-    define_macros.append(('GPAW_WITH_SL', '1'))
-
-    
-def get_hdf5_config(define_macros):
-    # check ScaLapack settings
-    define_macros.append(('GPAW_WITH_HDF5', '1'))
-
 
 def mtime(path, name, mtimes):
     """Return modification time.
@@ -423,7 +380,7 @@ def mtime(path, name, mtimes):
     mtimes[name] = t
     return t
 
-    
+
 def check_dependencies(sources):
     # Distutils does not do deep dependencies correctly.  We take care of
     # that here so that "python setup.py build_ext" always does the right
@@ -448,7 +405,7 @@ def check_dependencies(sources):
         # print 'removing', so
         os.remove(so)
 
-        
+
 def test_configuration():
     raise NotImplementedError
 
@@ -500,10 +457,13 @@ def build_interpreter(define_macros, include_dirs, libraries, library_dirs,
 
     cfiles = glob('c/[a-zA-Z_]*.c') + ['c/bmgs/bmgs.c']
     cfiles += glob('c/xc/*.c')
+    # Make build process deterministic (for "reproducible build" in debian)
+    # XXX some of this is duplicated in setup.py!  Why do the same thing twice?
+    cfiles.sort()
 
     sources = ['c/bc.c', 'c/localized_functions.c', 'c/mpi.c', 'c/_gpaw.c',
                'c/operators.c', 'c/woperators.c', 'c/transformers.c',
-               'c/blacs.c', 'c/utilities.c', 'c/hdf5.c']
+               'c/blacs.c', 'c/utilities.c', 'c/xc/libvdwxc.c']
     objects = ' '.join(['build/temp.%s/' % plat + x[:-1] + 'o'
                         for x in cfiles])
 
@@ -534,7 +494,7 @@ def build_interpreter(define_macros, include_dirs, libraries, library_dirs,
     if glob(libpl + '/libpython*mpi*'):
         libs += ' -lpython%s_mpi' % cfgDict['VERSION']
     else:
-        libs += ' -lpython%s' % cfgDict['VERSION']
+        libs += ' ' + cfgDict.get('BLDLIBRARY', '-lpython%s' % cfgDict['VERSION'])
     libs = ' '.join([libs, cfgDict['LIBS'], cfgDict['LIBM']])
 
     # Hack taken from distutils to determine option for runtime_libary_dirs
@@ -554,16 +514,20 @@ def build_interpreter(define_macros, include_dirs, libraries, library_dirs,
                              for lib in runtime_library_dirs])
 
     extra_link_args.append(cfgDict['LDFLAGS'])
+
     if sys.platform in ['aix5', 'aix6']:
         extra_link_args.append(cfgDict['LINKFORSHARED'].replace(
             'Modules', cfgDict['LIBPL']))
     elif sys.platform == 'darwin':
-        pass
+        # On a Mac, it is important to preserve the original compile args.
+        # This should probably always be done ?!?
+        extra_compile_args.append(cfgDict['CFLAGS'])
+        extra_link_args.append(cfgDict['LINKFORSHARED'])
     else:
         extra_link_args.append(cfgDict['LINKFORSHARED'])
 
     extra_compile_args.append('-fPIC')
-    
+
     # Compile the parallel sources
     for src in sources:
         obj = 'build/temp.%s/' % plat + src[:-1] + 'o'
@@ -575,12 +539,9 @@ def build_interpreter(define_macros, include_dirs, libraries, library_dirs,
                obj,
                src)
         print(cmd)
-        if '--dry-run' not in sys.argv:
-            error = os.system(cmd)
-            if error != 0:
-                msg = ['* compiling FAILED!  Only serial version of code '
-                       'will work.']
-                break
+        error = os.system(cmd)
+        if error != 0:
+            return error
 
     # Link the custom interpreter
     cmd = ('%s -o %s %s %s %s %s %s %s') % \
@@ -593,12 +554,6 @@ def build_interpreter(define_macros, include_dirs, libraries, library_dirs,
            runtime_libs,
            ' '.join(extra_link_args))
 
-    msg = ['* Building a custom interpreter']
     print(cmd)
-    if '--dry-run' not in sys.argv:
-        error = os.system(cmd)
-        if error != 0:
-            msg += ['* linking FAILED!  Only serial version of code will '
-                    'work.']
-
-    return error, msg
+    error = os.system(cmd)
+    return error
