@@ -2,26 +2,27 @@
 from __future__ import print_function
 import sys
 from math import pi, exp, sqrt, log
+from distutils.version import LooseVersion
 
 import numpy as np
-from scipy.optimize import fsolve  # , root
+from scipy.optimize import fsolve
 from scipy import __version__ as scipy_version
 from ase.units import Hartree
 from ase.data import atomic_numbers
 
-from gpaw.mpi import world
-from gpaw.version import version
+from gpaw import __version__ as version
 from gpaw.basis_data import Basis, BasisFunction, BasisPlotter
 from gpaw.gaunt import gaunt
 from gpaw.utilities import erf, pack2
 from gpaw.utilities.lapack import general_diagonalize
-from gpaw.atom.aeatom import AllElectronAtom, Channel, parse_ld_str, colors, \
-    GaussianBasis
+from gpaw.atom.aeatom import (AllElectronAtom, Channel, parse_ld_str, colors,
+                              GaussianBasis)
 
 
 class DatasetGenerationError(Exception):
     pass
-    
+
+
 parameters = {
     # 1-2:
     'H1': ('1s,s,p', 0.9),
@@ -210,8 +211,8 @@ def get_number_of_electrons(symbol, name):
     if name == 'default':
         return default[Z]
     return semicore[Z]
-    
-        
+
+
 class PAWWaves:
     def __init__(self, rgd, l, rcut):
         self.rgd = rgd
@@ -234,56 +235,33 @@ class PAWWaves:
         self.e_n.append(e)
         self.f_n.append(f)
 
-    def pseudize(self, type, nderiv):
+    def pseudize(self, type, nderiv, vtr_g, vr_g, rcmax):
         rgd = self.rgd
-
-        if type == 'poly':
-            ps = rgd.pseudize
-        elif type == 'bessel':
-            ps = rgd.jpseudize
-        elif type == 'nc':
-            ps = rgd.pseudize_normalized
-
+        r_g = rgd.r_g
         phi_ng = self.phi_ng = np.array(self.phi_ng)
         N = len(phi_ng)
         phit_ng = self.phit_ng = rgd.empty(N)
-        gcut = rgd.ceil(self.rcut)
-
-        self.nt_g = 0
-        self.c_n = []
-        for n in range(N):
-            phit_ng[n], c = ps(phi_ng[n], gcut, self.l, nderiv)
-            self.c_n.append(c)
-            self.nt_g += self.f_n[n] / 4 / pi * phit_ng[n]**2
-
-        self.dS_nn = np.empty((N, N))
-        for n1 in range(N):
-            for n2 in range(N):
-                self.dS_nn[n1, n2] = rgd.integrate(
-                    phi_ng[n1] * phi_ng[n2] -
-                    phit_ng[n1] * phit_ng[n2]) / (4 * pi)
-        self.Q = np.dot(self.f_n, self.dS_nn.diagonal())
-
-    def construct_projectors(self, vtr_g, rcmax):
-        N = len(self)
-        if N == 0:
-            self.pt_ng = []
-            return
-
-        rgd = self.rgd
-        phit_ng = self.phit_ng
+        pt_ng = self.pt_ng = rgd.empty(N)
+        gc = rgd.ceil(self.rcut)
         gcmax = rgd.ceil(rcmax)
-        r_g = rgd.r_g
+
         l = self.l
 
         dgdr_g = 1 / rgd.dr_g
         d2gdr2_g = rgd.d2gdr2()
 
-        q_ng = rgd.zeros(N)
-        for n in range(N):
+        self.nt_g = rgd.zeros()
+        for n, phi_g in enumerate(phi_ng):
+            if type == 'poly':
+                phit_ng[n], c0 = rgd.pseudize(phi_g, gc, self.l, nderiv)
+            elif type == 'nc':
+                phit_ng[n], c0 = rgd.pseudize_normalized(phi_g, gc, self.l,
+                                                         nderiv)
+            else:
+                1 / 0
             a_g, dadg_g, d2adg2_g = rgd.zeros(3)
             a_g[1:] = self.phit_ng[n, 1:] / r_g[1:]**l
-            a_g[0] = self.c_n[n]
+            a_g[0] = c0
             dadg_g[1:-1] = 0.5 * (a_g[2:] - a_g[:-2])
             d2adg2_g[1:-1] = a_g[2:] - 2 * a_g[1:-1] + a_g[:-2]
             q_g = (vtr_g - self.e_n[n] * r_g) * self.phit_ng[n]
@@ -291,33 +269,25 @@ class PAWWaves:
                 (2 * (l + 1) * dgdr_g + r_g * d2gdr2_g) * dadg_g +
                 r_g * d2adg2_g * dgdr_g**2)
             q_g[gcmax:] = 0
+            rgd.cut(q_g, self.rcut)
             q_g[1:] /= r_g[1:]
             if l == 0:
                 q_g[0] = q_g[1]
-            q_ng[n] = q_g
+            pt_ng[n] = q_g
 
-        A_nn = rgd.integrate(phit_ng[:, None] * q_ng) / (4 * pi)
-        self.dH_nn = self.e_n * self.dS_nn - A_nn
+            self.nt_g += self.f_n[n] / 4 / pi * phit_ng[n]**2
 
-        L_nn = np.eye(N)
-        U_nn = A_nn.copy()
+        self.dS_nn = (rgd.integrate(phi_ng[:, None] * phi_ng) -
+                      rgd.integrate(phit_ng[:, None] * phit_ng)) / (4 * pi)
+        self.Q = np.dot(self.f_n, self.dS_nn.diagonal())
+        A_nn = rgd.integrate(phit_ng[:, None] * pt_ng) / (4 * pi)
+        self.dH_nn = np.dot(self.dS_nn, np.diag(self.e_n)) - A_nn
+        self.dH_nn *= 0.5
+        self.dH_nn += self.dH_nn.T.copy()
+        pt_ng[:] = np.dot(np.linalg.inv(A_nn.T), pt_ng)
 
-        if N - self.n_n.count(-1) == 1:
-            assert self.n_n[0] != -1
-            # We have a single bound-state projector.
-            for n1 in range(N):
-                for n2 in range(n1 + 1, N):
-                    L_nn[n2, n1] = U_nn[n2, n1] / U_nn[n1, n1]
-                    U_nn[n2] -= U_nn[n1] * L_nn[n2, n1]
-
-            iL_nn = np.linalg.inv(L_nn)
-            phit_ng[:] = np.dot(iL_nn, phit_ng)
-            self.phi_ng[:] = np.dot(iL_nn, self.phi_ng)
-
-            self.dS_nn = np.dot(np.dot(iL_nn, self.dS_nn), iL_nn.T)
-            self.dH_nn = np.dot(np.dot(iL_nn, self.dH_nn), iL_nn.T)
-
-        self.pt_ng = np.dot(np.linalg.inv(U_nn.T), q_ng)
+    def construct_projectors(self, vtr_g, rcmax):
+        pass
 
     def calculate_kinetic_energy_correction(self, vr_g, vtr_g):
         if len(self) == 0:
@@ -414,7 +384,7 @@ class PAWSetupGenerator:
             def f(alpha):
                 return log(spillage(alpha)) - log(eps)
 
-            if scipy_version < '0.8':
+            if LooseVersion(scipy_version) < '0.8':
                 self.alpha = fsolve(f, 7.0)
             else:
                 self.alpha = fsolve(f, 7.0)[0]
@@ -454,122 +424,6 @@ class PAWSetupGenerator:
         self.log('Core electrons:', self.ncore)
         self.log('Valence electrons:', self.nvalence)
 
-    def add_waves(self, rc):
-        if isinstance(rc, float):
-            radii = [rc]
-        else:
-            radii = rc
-
-        if self.lmax >= 0:
-            radii += [radii[-1]] * (self.lmax + 1 - len(radii))
-        del radii[self.lmax + 1:]  # remove unused radii
-            
-        self.rcmax = max(radii)
-
-        self.waves_l = []
-        for l in range(self.lmax + 1):
-            rcut = radii[l]
-            waves = PAWWaves(self.rgd, l, rcut)
-            e = -1.0
-            for n in self.states[l]:
-                if isinstance(n, int):
-                    # Bound state:
-                    ch = self.aea.channels[l]
-                    e = ch.e_n[n - l - 1]
-                    f = ch.f_n[n - l - 1]
-                    phi_g = ch.phi_ng[n - l - 1]
-                else:
-                    if n is None:
-                        e += 1.0
-                    else:
-                        e = n
-                    n = -1
-                    f = 0.0
-                    phi_g = self.rgd.zeros()
-                    gc = self.rgd.round(1.5 * rcut)
-                    ch = Channel(l)
-                    a = ch.integrate_outwards(phi_g, self.rgd,
-                                              self.aea.vr_sg[0], gc, e,
-                                              self.aea.scalar_relativistic,
-                                              self.aea.Z)[1]
-                    phi_g[1:gc + 1] /= self.rgd.r_g[1:gc + 1]
-                    phi_g[0] = a
-                    phi_g /= (self.rgd.integrate(phi_g**2) / (4 * pi))**0.5
-
-                waves.add(phi_g, n, e, f)
-            self.waves_l.append(waves)
-
-    def pseudize(self, type='poly', nderiv=6, rcore=None):
-        self.Q = -self.aea.Z + self.ncore
-
-        self.nt_g = self.rgd.zeros()
-        for waves in self.waves_l:
-            waves.pseudize(type, nderiv)
-            self.nt_g += waves.nt_g
-            self.Q += waves.Q
-
-        if rcore is None:
-            rcore = self.rcmax * 0.8
-        else:
-            assert rcore <= self.rcmax
-
-        if self.ncore == 0:
-            self.nct_g = self.rgd.zeros()
-            self.tauct_g = self.rgd.zeros()
-        else:
-            # Make sure pseudo density is monotonically decreasing:
-            while True:
-                gcore = self.rgd.round(rcore)
-                self.nct_g = self.rgd.pseudize(self.nc_g, gcore)[0]
-                nt_g = self.nt_g + self.nct_g
-                dntdr_g = self.rgd.derivative(nt_g)[:gcore]
-                if dntdr_g.max() < 0.0:
-                    break
-                rcore -= 0.01
-
-            rcore *= 1.2
-            gcore = self.rgd.round(rcore)
-            self.nct_g = self.rgd.pseudize(self.nc_g, gcore)[0]
-            nt_g = self.nt_g + self.nct_g
-
-            self.log('Constructing smooth pseudo core density for r < %.3f' %
-                     rcore)
-            self.nt_g = nt_g
-
-            self.tauct_g = self.rgd.pseudize(self.tauc_g, gcore)[0]
-
-        self.npseudocore = self.rgd.integrate(self.nct_g)
-        self.log('Pseudo core electrons: %.6f' % self.npseudocore)
-        self.Q -= self.npseudocore
-
-        self.rhot_g = self.nt_g + self.Q * self.ghat_g
-        self.vHtr_g = self.rgd.poisson(self.rhot_g)
-
-        self.vxct_g = self.rgd.zeros()
-        nt_sg = self.nt_g.reshape((1, -1))
-        self.exct = self.aea.xc.calculate_spherical(
-            self.rgd, nt_sg, self.vxct_g.reshape((1, -1)))
-
-        self.v0r_g = self.vtr_g - self.vHtr_g - self.vxct_g * self.rgd.r_g
-        self.v0r_g[self.rgd.round(self.rcmax):] = 0.0
-
-        self.log('\nProjectors:')
-        self.log(' state  occ         energy             norm        rcut')
-        self.log(' nl            [Hartree]  [eV]      [electrons]   [Bohr]')
-        self.log('----------------------------------------------------------')
-        for l, waves in enumerate(self.waves_l):
-            for n, e, f, ds in zip(waves.n_n, waves.e_n, waves.f_n,
-                                   waves.dS_nn.diagonal()):
-                if n == -1:
-                    self.log('  %s         %10.6f %10.5f   %19.2f' %
-                             ('spdf'[l], e, e * Hartree, waves.rcut))
-                else:
-                    self.log(
-                        ' %d%s   %5.2f %10.6f %10.5f      %5.3f  %9.2f' %
-                        (n, 'spdf'[l], f, e, e * Hartree, 1 - ds,
-                         waves.rcut))
-        self.log()
-
     def find_local_potential(self, r0, P):
         self.r0 = r0
         self.nderiv0 = P
@@ -621,9 +475,144 @@ class PAWSetupGenerator:
         self.vtr_g[0] = 0.0
         self.vtr_g[1:g0] = q_g[1:g0]
 
-    def construct_projectors(self):
+    def add_waves(self, rc):
+        if isinstance(rc, float):
+            radii = [rc]
+        else:
+            radii = rc
+
+        if self.lmax >= 0:
+            radii += [radii[-1]] * (self.lmax + 1 - len(radii))
+        del radii[self.lmax + 1:]  # remove unused radii
+
+        self.rcmax = max(radii)
+
+        self.waves_l = []
+        for l in range(self.lmax + 1):
+            rcut = radii[l]
+            waves = PAWWaves(self.rgd, l, rcut)
+            e = -1.0
+            for n in self.states[l]:
+                if isinstance(n, int):
+                    # Bound state:
+                    ch = self.aea.channels[l]
+                    e = ch.e_n[n - l - 1]
+                    f = ch.f_n[n - l - 1]
+                    phi_g = ch.phi_ng[n - l - 1]
+                else:
+                    if n is None:
+                        e += 1.0
+                    else:
+                        e = n
+                    n = -1
+                    f = 0.0
+                    phi_g = self.rgd.zeros()
+                    gc = self.rgd.round(2.5 * self.rcmax)
+                    ch = Channel(l)
+                    a = ch.integrate_outwards(phi_g, self.rgd,
+                                              self.aea.vr_sg[0], gc, e,
+                                              self.aea.scalar_relativistic,
+                                              self.aea.Z)[1]
+                    phi_g[1:gc + 1] /= self.rgd.r_g[1:gc + 1]
+                    phi_g[0] = a
+                    phi_g /= (self.rgd.integrate(phi_g**2) / (4 * pi))**0.5
+
+                waves.add(phi_g, n, e, f)
+            self.waves_l.append(waves)
+
+    def pseudize(self, type='poly', nderiv=6, rcore=None):
+        self.Q = -self.aea.Z + self.ncore
+
+        self.nt_g = self.rgd.zeros()
         for waves in self.waves_l:
-            waves.construct_projectors(self.vtr_g, self.rcmax)
+            waves.pseudize(type, nderiv, self.vtr_g, self.aea.vr_sg[0],
+                           2.0 * self.rcmax)
+            self.nt_g += waves.nt_g
+            self.Q += waves.Q
+
+        self.construct_pseudo_core_density(rcore)
+        self.calculate_potentials()
+        self.summarize()
+
+    def construct_pseudo_core_density(self, rcore):
+        if rcore is None:
+            rcore = self.rcmax * 0.8
+        else:
+            assert abs(rcore) <= self.rcmax
+
+        if self.ncore == 0:
+            self.nct_g = self.rgd.zeros()
+            self.tauct_g = self.rgd.zeros()
+        elif rcore > 0.0:
+            # Make sure pseudo density is monotonically decreasing:
+            while True:
+                gcore = self.rgd.round(rcore)
+                self.nct_g = self.rgd.pseudize(self.nc_g, gcore)[0]
+                nt_g = self.nt_g + self.nct_g
+                dntdr_g = self.rgd.derivative(nt_g)[:gcore]
+                if dntdr_g.max() < 0.0:
+                    break
+                rcore -= 0.01
+
+            rcore *= 1.2
+            gcore = self.rgd.round(rcore)
+            self.nct_g = self.rgd.pseudize(self.nc_g, gcore)[0]
+            nt_g = self.nt_g + self.nct_g
+
+            self.log('Constructing smooth pseudo core density for r < %.3f' %
+                     rcore)
+            self.nt_g = nt_g
+
+            self.tauct_g = self.rgd.pseudize(self.tauc_g, gcore)[0]
+        else:
+            rcore *= -1
+            gcore = self.rgd.round(rcore)
+            nt_g = self.rgd.pseudize(self.aea.n_sg[0], gcore)[0]
+            self.nct_g = nt_g - self.nt_g
+            self.nt_g = nt_g
+
+            self.log('Constructing NLCC-style smooth pseudo core density for '
+                     'r < %.3f' % rcore)
+
+            self.tauct_g = self.rgd.pseudize(self.tauc_g, gcore)[0]
+
+        self.npseudocore = self.rgd.integrate(self.nct_g)
+        self.log('Pseudo core electrons: %.6f' % self.npseudocore)
+        self.Q -= self.npseudocore
+
+    def calculate_potentials(self):
+        self.rhot_g = self.nt_g + self.Q * self.ghat_g
+        self.vHtr_g = self.rgd.poisson(self.rhot_g)
+
+        self.vxct_g = self.rgd.zeros()
+        nt_sg = self.nt_g.reshape((1, -1))
+        self.exct = self.aea.xc.calculate_spherical(
+            self.rgd, nt_sg, self.vxct_g.reshape((1, -1)))
+
+        self.v0r_g = self.vtr_g - self.vHtr_g - self.vxct_g * self.rgd.r_g
+        self.v0r_g[self.rgd.round(self.rcmax):] = 0.0
+
+    def summarize(self):
+        self.log('\nProjectors:')
+        self.log(' state  occ         energy             norm        rcut')
+        self.log(' nl            [Hartree]  [eV]      [electrons]   [Bohr]')
+        self.log('----------------------------------------------------------')
+        for l, waves in enumerate(self.waves_l):
+            for n, e, f, ds in zip(waves.n_n, waves.e_n, waves.f_n,
+                                   waves.dS_nn.diagonal()):
+                if n == -1:
+                    self.log('  %s         %10.6f %10.5f   %19.2f' %
+                             ('spdf'[l], e, e * Hartree, waves.rcut))
+                else:
+                    self.log(
+                        ' %d%s   %5.2f %10.6f %10.5f      %5.3f  %9.2f' %
+                        (n, 'spdf'[l], f, e, e * Hartree, 1 - ds,
+                         waves.rcut))
+        self.log()
+
+    def construct_projectors(self, rcore):
+        for waves in self.waves_l:
+            waves.construct_projectors(self.vtr_g, 2.45 * self.rcmax)
             waves.calculate_kinetic_energy_correction(self.aea.vr_sg[0],
                                                       self.vtr_g)
 
@@ -636,20 +625,20 @@ class PAWSetupGenerator:
 
         for l in range(4):
             try:
-                e_b, n0 = self.check(l)
+                e_b = self.check(l)
             except RuntimeError:
                 self.log('Singular overlap matrix!')
                 ok = False
-                # continue  # fail here
+                continue
 
-            nbound = (e_b < -0.05).sum()
+            n0 = self.number_of_core_states(l)
 
             if l < len(self.aea.channels):
                 e0_b = self.aea.channels[l].e_n
-                nbound0 = (e0_b < -0.05).sum()
                 extra = 6
-                for n in range(1 + l, nbound0 + 1 + l + extra):
-                    if n - 1 - l < len(self.aea.channels[l].f_n):
+                nae = len(self.aea.channels[l].f_n)
+                for n in range(1 + l, nae + 1 + l + extra):
+                    if n - 1 - l < nae:
                         f = self.aea.channels[l].f_n[n - 1 - l]
                         self.log('%2d%s  %2d' % (n, 'spdf'[l], f), end='')
                     else:
@@ -663,26 +652,31 @@ class PAWSetupGenerator:
                     else:
                         self.log()
 
-                if nbound != nbound0 - n0:
-                    self.log('Wrong number of %s-states!' % 'spdf'[l])
-                    ok = False
-                elif (nbound > 0 and
-                      abs(e_b[:nbound] - e0_b[n0:nbound0]).max() > 1e-3):
+                errors = abs(e_b[:nae - n0] - e0_b[n0:nae])
+                if (errors > 2e-3).any():
                     self.log('Error in bound %s-states!' % 'spdf'[l])
                     ok = False
-                elif (abs(e_b[nbound:nbound + extra] -
-                          e0_b[nbound0:nbound0 + extra]).max() > 2e-2):
+                errors = abs(e_b[nae - n0:nae - n0 + extra] -
+                             e0_b[nae:nae + extra])
+                if (not self.aea.scalar_relativistic and
+                    (errors > 2e-2).any()):
                     self.log('Error in %s-states!' % 'spdf'[l])
-                    if not self.aea.scalar_relativistic:
-                        ok = False
-                    else:
-                        ok = True
-            elif nbound > 0:
-                self.log('Wrong number of %s-states!' % 'spdf'[l])
-                ok = False
+                    ok = False
 
         return ok
 
+    def number_of_core_states(self, l):
+        n0 = 0
+        if l < len(self.waves_l):
+            waves = self.waves_l[l]
+            if len(waves) > 0:
+                n0 = waves.n_n[0] - l - 1
+                if n0 < 0 and l < len(self.aea.channels):
+                    n0 = (self.aea.channels[l].f_n > 0).sum()
+        elif l < len(self.aea.channels):
+            n0 = (self.aea.channels[l].f_n > 0).sum()
+        return n0
+
     def check(self, l):
         basis = self.aea.channels[0].basis
         eps = basis.eps
@@ -693,7 +687,6 @@ class PAWSetupGenerator:
         H_bb += basis.T_bb
         S_bb = np.eye(len(basis))
 
-        n0 = 0
         if l < len(self.waves_l):
             waves = self.waves_l[l]
             if len(waves) > 0:
@@ -701,15 +694,10 @@ class PAWSetupGenerator:
                                           waves.pt_ng) / (4 * pi)
                 H_bb += np.dot(np.dot(P_bn, waves.dH_nn), P_bn.T)
                 S_bb += np.dot(np.dot(P_bn, waves.dS_nn), P_bn.T)
-                n0 = waves.n_n[0] - l - 1
-                if n0 < 0 and l < len(self.aea.channels):
-                    n0 = (self.aea.channels[l].f_n > 0).sum()
-        elif l < len(self.aea.channels):
-            n0 = (self.aea.channels[l].f_n > 0).sum()
 
         e_b = np.empty(len(basis))
         general_diagonalize(H_bb, e_b, S_bb)
-        return e_b, n0
+        return e_b
 
     def test_convergence(self):
         rgd = self.rgd
@@ -814,7 +802,7 @@ class PAWSetupGenerator:
 
     def create_basis_set(self, tailnorm=0.0005, scale=200.0, splitnorm=0.16):
         rgd = self.rgd
-        self.basis = Basis(self.aea.symbol, readxml=False, rgd=rgd)
+        self.basis = Basis(self.aea.symbol, 'dzp', readxml=False, rgd=rgd)
 
         # We print text to sdtout and put it in the basis-set file
         txt = 'Basis functions:\n'
@@ -1185,160 +1173,41 @@ class PAWSetupGenerator:
                 i1 += 2 * l1 + 1
 
 
-def main(argv=None):
-    from optparse import OptionParser
-
-    parser = OptionParser(usage='gwap dataset [options] element')
-    add = parser.add_option
-    add('-f', '--xc-functional', type='string', default='LDA',
-        help='Exchange-Correlation functional (default value LDA)',
-        metavar='<XC>')
-    add('-C', '--configuration',
-        help='e.g. for Li: "[(1, 0, 2, -1.878564), (2, 0, 1, -0.10554),'
-        ' (2, 1, 0, 0.0)]"')
-    add('-P', '--projectors',
-        help='Projector functions - use comma-separated - ' +
-        'nl values, where n can be pricipal quantum number ' +
-        '(integer) or energy (floating point number). ' +
-        'Example: 2s,0.5s,2p,0.5p,0.0d.')
-    add('-r', '--radius',
-        help='1.2 or 1.2,1.1,1.1')
-    add('-0', '--zero-potential',
-        metavar='nderivs,radius',
-        help='Parameters for zero potential.')
-    add('-c', '--pseudo-core-density-radius', type=float,
-        metavar='radius',
-        help='Radius for pseudizing core density.')
-    add('-z', '--pseudize',
-        metavar='type,nderivs',
-        help='Parameters for pseudizing wave functions.')
-    add('-p', '--plot', action='store_true')
-    add('-l', '--logarithmic-derivatives',
-        metavar='spdfg,e1:e2:de,radius',
-        help='Plot logarithmic derivatives. ' +
-        'Example: -l spdf,-1:1:0.05,1.3. ' +
-        'Energy range and/or radius can be left out.')
-    add('-w', '--write', action='store_true')
-    add('-s', '--scalar-relativistic', action='store_true')
-    add('--no-check', action='store_true')
-    add('-t', '--tag', type='string')
-    add('-a', '--alpha', type=float)
-    add('-b', '--create-basis-set', action='store_true')
-    add('--core-hole')
-    add('-e', '--electrons', type=int)
-    add('-o', '--output')
-    
-    opt, symbols = parser.parse_args(argv)
-
-    for symbol in symbols:
-        kwargs = get_parameters(symbol, opt)
-        gen = _generate(**kwargs)
-
-        if not opt.no_check:
-            if not gen.check_all():
-                raise DatasetGenerationError
-
-        if opt.create_basis_set or opt.write:
-            basis = None  # gen.create_basis_set()
-            
-            if opt.create_basis_set:
-                basis.write_xml()
-
-            if opt.write:
-                gen.make_paw_setup(opt.tag).write_xml()
-
-        if opt.logarithmic_derivatives or opt.plot:
-            if opt.plot:
-                import matplotlib.pyplot as plt
-            if opt.logarithmic_derivatives:
-                r = 1.1 * gen.rcmax
-                emin = min(min(wave.e_n) for wave in gen.waves_l) - 0.8
-                emax = max(max(wave.e_n) for wave in gen.waves_l) + 0.8
-                lvalues, energies, r = parse_ld_str(
-                    opt.logarithmic_derivatives, (emin, emax, 0.05), r)
-                error = 0.0
-                for l in lvalues:
-                    ld1 = gen.aea.logarithmic_derivative(l, energies, r)
-                    if opt.plot:
-                        plt.plot(energies, ld1, colors[l], label='spdfg'[l])
-                    ld2 = gen.logarithmic_derivative(l, energies, r)
-                    if opt.plot:
-                        plt.plot(energies, ld2, '--' + colors[l])
-                    de = energies[1] - energies[0]
-                    error = abs(ld1 - ld2).sum() * de
-                    print('Logarithmic derivative error:', l, error)
-                    
-                    # Fixed points:
-                    if l < len(gen.waves_l):
-                        efix = gen.waves_l[l].e_n
-                        ldfix = gen.logarithmic_derivative(l, efix, r)
-                        if opt.plot:
-                            plt.plot(efix, ldfix, 'x' + colors[l])
-
-                    if l == gen.l0:
-                        efix = [0.0]
-                        ldfix = gen.logarithmic_derivative(l, efix, r)
-                        if opt.plot:
-                            plt.plot(efix, ldfix, 'x' + colors[l])
-
-                if opt.plot:
-                    plt.xlabel('energy [Ha]')
-                    plt.ylabel(r'$\arctan(d\log\phi_{\ell\epsilon}(r)/dr)/\pi'
-                               r'|_{r=r_c}$')
-                    plt.legend(loc='best')
-
-            if opt.plot:
-                gen.plot()
-
-                if opt.create_basis_set:
-                    gen.basis.generatordata = ''  # we already printed this
-                    BasisPlotter(show=True).plot(gen.basis)
-
-            if opt.plot:
-                try:
-                    plt.show()
-                except KeyboardInterrupt:
-                    pass
-
-    world.barrier()
-    return gen
-
-
-def get_parameters(symbol, opt):
-    if opt.electrons:
-        par = parameters[symbol + str(opt.electrons)]
+def get_parameters(symbol, args):
+    if args.electrons:
+        par = parameters[symbol + str(args.electrons)]
     else:
         Z = atomic_numbers[symbol]
         par = parameters[symbol + str(default[Z])]
-        
+
     projectors, radii = par[:2]
     if len(par) == 3:
         extra = par[2]
     else:
         extra = {}
 
-    if opt.configuration:
-        configuration = eval(opt.configuration)
+    if args.configuration:
+        configuration = eval(args.configuration)
     else:
         configuration = None
 
-    if opt.projectors:
-        projectors = opt.projectors
+    if args.projectors:
+        projectors = args.projectors
 
-    if opt.radius:
-        radii = [float(r) for r in opt.radius.split(',')]
+    if args.radius:
+        radii = [float(r) for r in args.radius.split(',')]
 
     if isinstance(radii, float):
         radii = [radii]
 
-    if opt.pseudize:
-        type, nderiv = opt.pseudize.split(',')
+    if args.pseudize:
+        type, nderiv = args.pseudize.split(',')
         pseudize = (type, int(nderiv))
     else:
         pseudize = ('poly', 4)
 
-    if opt.zero_potential:
-        x = opt.zero_potential.split(',')
+    if args.zero_potential:
+        x = args.zero_potential.split(',')
         nderiv0 = int(x[0])
         r0 = float(x[1])
     else:
@@ -1349,21 +1218,24 @@ def get_parameters(symbol, opt):
             nderiv0 = 2
             r0 = extra.get('r0', min(radii))
 
-    if opt.pseudo_core_density_radius:
-        rcore = opt.pseudo_core_density_radius
+    if args.pseudo_core_density_radius:
+        rcore = args.pseudo_core_density_radius
     else:
         rcore = extra.get('rcore')
-        
+
+    if args.nlcc:
+        rcore *= -1
+
     return dict(symbol=symbol,
-                xc=opt.xc_functional,
+                xc=args.xc_functional,
                 configuration=configuration,
                 projectors=projectors,
                 radii=radii,
-                scalar_relativistic=opt.scalar_relativistic, alpha=opt.alpha,
+                scalar_relativistic=args.scalar_relativistic, alpha=args.alpha,
                 r0=r0, nderiv0=nderiv0,
                 pseudize=pseudize, rcore=rcore,
-                core_hole=opt.core_hole,
-                output=opt.output)
+                core_hole=args.core_hole,
+                output=args.output)
 
 
 def _generate(symbol, xc, configuration, projectors, radii,
@@ -1381,7 +1253,7 @@ def _generate(symbol, xc, configuration, projectors, radii,
     gen.find_local_potential(r0, nderiv0)
     gen.add_waves(radii)
     gen.pseudize(pseudize[0], pseudize[1], rcore=rcore)
-    gen.construct_projectors()
+    gen.construct_projectors(rcore)
     return gen
 
 
@@ -1405,5 +1277,136 @@ def generate_all():
             main(argv)
 
 
-if __name__ == '__main__':
-    main()
+class CLICommand:
+    short_description = 'Create PAW dataset'
+
+    @staticmethod
+    def add_arguments(parser):
+        add = parser.add_argument
+        add('symbol')
+        add('-f', '--xc-functional', type=str, default='LDA',
+            help='Exchange-Correlation functional (default value LDA)',
+            metavar='<XC>')
+        add('-C', '--configuration',
+            help='e.g. for Li: "[(1, 0, 2, -1.878564), (2, 0, 1, -0.10554),'
+            ' (2, 1, 0, 0.0)]"')
+        add('-P', '--projectors',
+            help='Projector functions - use comma-separated - ' +
+            'nl values, where n can be principal quantum number ' +
+            '(integer) or energy (floating point number). ' +
+            'Example: 2s,0.5s,2p,0.5p,0.0d.')
+        add('-r', '--radius',
+            help='1.2 or 1.2,1.1,1.1')
+        add('-0', '--zero-potential',
+            metavar='nderivs,radius',
+            help='Parameters for zero potential.')
+        add('-c', '--pseudo-core-density-radius', type=float,
+            metavar='radius',
+            help='Radius for pseudizing core density.')
+        add('-z', '--pseudize',
+            metavar='type,nderivs',
+            help='Parameters for pseudizing wave functions.')
+        add('-p', '--plot', action='store_true')
+        add('-l', '--logarithmic-derivatives',
+            metavar='spdfg,e1:e2:de,radius',
+            help='Plot logarithmic derivatives. ' +
+            'Example: -l spdf,-1:1:0.05,1.3. ' +
+            'Energy range and/or radius can be left out.')
+        add('-w', '--write', action='store_true')
+        add('-s', '--scalar-relativistic', action='store_true')
+        add('-n', '--no-check', action='store_true')
+        add('-t', '--tag', type=str)
+        add('-a', '--alpha', type=float)
+        add('-b', '--create-basis-set', action='store_true')
+        add('--nlcc', action='store_true',
+            help='Use NLCC-style pseudo core density '
+            '(for vdW-DF functionals).')
+        add('--core-hole')
+        add('-e', '--electrons', type=int)
+        add('-o', '--output')
+
+    @staticmethod
+    def run(args):
+        main(args)
+
+
+def main(args):
+    kwargs = get_parameters(args.symbol, args)
+    gen = _generate(**kwargs)
+
+    if not args.no_check:
+        if not gen.check_all():
+            raise DatasetGenerationError
+
+    if args.create_basis_set or args.write:
+        basis = gen.create_basis_set()
+
+        if args.create_basis_set:
+            basis.write_xml()
+
+        if args.write:
+            setup = gen.make_paw_setup(args.tag)
+            parameters = []
+            for key, value in kwargs.items():
+                if value is not None:
+                    parameters.append('{0}={1!r}'.format(key, value))
+            setup.generatordata = ',\n    '.join(parameters)
+            setup.write_xml()
+
+    if args.logarithmic_derivatives or args.plot:
+        if args.plot:
+            import matplotlib.pyplot as plt
+        if args.logarithmic_derivatives:
+            r = 1.1 * gen.rcmax
+            emin = min(min(wave.e_n) for wave in gen.waves_l) - 0.8
+            emax = max(max(wave.e_n) for wave in gen.waves_l) + 0.8
+            lvalues, energies, r = parse_ld_str(
+                args.logarithmic_derivatives, (emin, emax, 0.05), r)
+            emin = energies[0]
+            de = energies[1] - emin
+
+            error = 0.0
+            for l in lvalues:
+                efix = []
+                # Fixed points:
+                if l < len(gen.waves_l):
+                    efix.extend(gen.waves_l[l].e_n)
+                if l == gen.l0:
+                    efix.append(0.0)
+
+                ld1 = gen.aea.logarithmic_derivative(l, energies, r)
+                ld2 = gen.logarithmic_derivative(l, energies, r)
+                for e in efix:
+                    i = int((e - emin) / de)
+                    if 0 <= i < len(energies):
+                        ld1 -= round(ld1[i] - ld2[i])
+                        if args.plot:
+                            ldfix = ld1[i]
+                            plt.plot([energies[i]], [ldfix],
+                                     'x' + colors[l])
+
+                if args.plot:
+                    plt.plot(energies, ld1, colors[l], label='spdfg'[l])
+                    plt.plot(energies, ld2, '--' + colors[l])
+
+                error = abs(ld1 - ld2).sum() * de
+                print('Logarithmic derivative error:', l, error)
+
+            if args.plot:
+                plt.xlabel('energy [Ha]')
+                plt.ylabel(r'$\arctan(d\log\phi_{\ell\epsilon}(r)/dr)/\pi'
+                           r'|_{r=r_c}$')
+                plt.legend(loc='best')
+
+        if args.plot:
+            gen.plot()
+
+            if args.create_basis_set:
+                gen.basis.generatordata = ''  # we already printed this
+                BasisPlotter(show=True).plot(gen.basis)
+
+        if args.plot:
+            try:
+                plt.show()
+            except KeyboardInterrupt:
+                pass
