@@ -121,7 +121,7 @@ class PhononCalculator:
 
         # Number of occupied bands
         nvalence = self.calc.wfs.nvalence
-        nbands = nvalence / 2 + nvalence % 2
+        nbands = nvalence // 2 + nvalence % 2
         assert nbands <= self.calc.wfs.bd.nbands
 
         # Extract other useful objects
@@ -164,13 +164,8 @@ class PhononCalculator:
 
     def initialize(self):
         """Initialize response calculator and perturbation."""
-
-        # Get scaled atomic positions
-        spos_ac = self.atoms.get_scaled_positions()
-
-        self.perturbation.initialize(spos_ac)
-        self.response_calc.initialize(spos_ac)
-
+        self.perturbation.initialize(self.calc.spos_ac)
+        self.response_calc.initialize(self.calc.spos_ac)
         self.initialized = True
 
     def __getstate__(self):
