@@ -17,7 +17,7 @@ import _gpaw
 import gpaw.mpi as mpi
 from gpaw.domain import Domain
 from gpaw.utilities import mlsqr
-from gpaw.utilities.blas import rk, r2k, gemv, gemm
+from gpaw.utilities.blas import rk, r2k, gemm
 
 
 # Remove this:  XXX
@@ -68,8 +68,8 @@ class GridDescriptor(Domain):
 
     ndim = 3  # dimension of ndarrays
 
-    def __init__(self, N_c, cell_cv=(1, 1, 1), pbc_c=True,
-                 comm=None, parsize=None):
+    def __init__(self, N_c, cell_cv=[1, 1, 1], pbc_c=True,
+                 comm=None, parsize_c=None):
         """Construct grid-descriptor object.
 
         parameters:
@@ -82,10 +82,10 @@ class GridDescriptor(Domain):
             Periodic boundary conditions flag(s).
         comm: MPI-communicator
             Communicator for domain-decomposition.
-        parsize: tuple of 3 ints, a single int or None
+        parsize_c: tuple of 3 ints, a single int or None
             Number of domains.
 
-        Note that if pbc_c[c] is True, then the actual number of gridpoints
+        Note that if pbc_c[c] is False, then the actual number of gridpoints
         along axis c is one less than N_c[c].
 
         Attributes:
@@ -111,32 +111,32 @@ class GridDescriptor(Domain):
         self.N_c = np.array(N_c, int)
         if (self.N_c != N_c).any():
             raise ValueError('Non-int number of grid points %s' % N_c)
-        
-        Domain.__init__(self, cell_cv, pbc_c, comm, parsize, self.N_c)
-        self.rank = self.comm.rank
 
-        parsize_c = self.parsize_c
-        n_c, remainder_c = divmod(self.N_c, parsize_c)
+        Domain.__init__(self, cell_cv, pbc_c, comm, parsize_c, self.N_c)
+        self.rank = self.comm.rank
 
         self.beg_c = np.empty(3, int)
         self.end_c = np.empty(3, int)
 
         self.n_cp = []
         for c in range(3):
-            n_p = (np.arange(parsize_c[c] + 1) * float(self.N_c[c]) /
-                   parsize_c[c])
+            n_p = (np.arange(self.parsize_c[c] + 1) * float(self.N_c[c]) /
+                   self.parsize_c[c])
             n_p = np.around(n_p + 0.4999).astype(int)
-            
+
             if not self.pbc_c[c]:
                 n_p[0] = 1
 
-            if not np.alltrue(n_p[1:] - n_p[:-1]):
-                raise ValueError('Grid too small!')
-                    
+            if not np.all(n_p[1:] - n_p[:-1] > 0):
+                raise ValueError('Grid {0} too small for {1} cores!'
+                                 .format('x'.join(str(n) for n in self.N_c),
+                                         'x'.join(str(n) for n
+                                                  in self.parsize_c)))
+
             self.beg_c[c] = n_p[self.parpos_c[c]]
             self.end_c[c] = n_p[self.parpos_c[c] + 1]
             self.n_cp.append(n_p)
-            
+
         self.n_c = self.end_c - self.beg_c
 
         self.h_cv = self.cell_cv / self.N_c[:, np.newaxis]
@@ -151,8 +151,21 @@ class GridDescriptor(Domain):
         if max(h_c) / min(h_c) > 1.3:
             raise ValueError('Very anisotropic grid spacings: %s' % h_c)
 
+    def __repr__(self):
+        if self.orthogonal:
+            cellstring = np.diag(self.cell_cv).tolist()
+        else:
+            cellstring = self.cell_cv.tolist()
+
+        pcoords = tuple(self.get_processor_position_from_rank())
+        return ('GridDescriptor(%s, cell_cv=%s, pbc_c=%s, comm=[%d/%d, '
+                'domain=%s], parsize=%s)'
+                % (self.N_c.tolist(), cellstring,
+                   np.array(self.pbc_c).astype(int).tolist(), self.comm.rank,
+                   self.comm.size, pcoords, self.parsize_c.tolist()))
+
     def new_descriptor(self, N_c=None, cell_cv=None, pbc_c=None,
-                       comm=None, parsize=None):
+                       comm=None, parsize_c=None):
         """Create new descriptor based on this one.
 
         The new descriptor will use the same class (possibly a subclass)
@@ -166,9 +179,25 @@ class GridDescriptor(Domain):
             pbc_c = self.pbc_c
         if comm is None:
             comm = self.comm
-        if parsize is None:
-            parsize = self.parsize_c
-        return self.__class__(N_c, cell_cv, pbc_c, comm, parsize)
+        if parsize_c is None and comm.size == self.comm.size:
+            parsize_c = self.parsize_c
+        return self.__class__(N_c, cell_cv, pbc_c, comm, parsize_c)
+
+    def coords(self, c, pad=True):
+        """Return coordinates along one of the three axes.
+
+        Useful for plotting::
+
+            import matplotlib.pyplot as plt
+            plt.plot(gd.coords(0), data[:, 0, 0])
+            plt.show()
+
+        """
+        L = np.linalg.norm(self.cell_cv[c])
+        N = self.N_c[c]
+        h = L / N
+        p = self.pbc_c[c] or pad
+        return np.linspace((1 - p) * h, L, N - 1 + p, False)
 
     def get_grid_spacings(self):
         L_c = (np.linalg.inv(self.cell_cv)**2).sum(0)**-0.5
@@ -183,7 +212,7 @@ class GridDescriptor(Domain):
     def flat_index(self, G_c):
         g1, g2, g3 = G_c - self.beg_c
         return g3 + self.n_c[2] * (g2 + g1 * self.n_c[1])
-    
+
     def get_slice(self):
         return [slice(b - 1 + p, e - 1 + p) for b, e, p in
                 zip(self.beg_c, self.end_c, self.pbc_c)]
@@ -197,7 +226,7 @@ class GridDescriptor(Domain):
         ``global_array=True``."""
 
         return self._new_array(n, dtype, True, global_array, pad)
-    
+
     def empty(self, n=(), dtype=float, global_array=False, pad=False):
         """Return new uninitialized 3D array for this domain.
 
@@ -207,14 +236,14 @@ class GridDescriptor(Domain):
         ``global_array=True``."""
 
         return self._new_array(n, dtype, False, global_array, pad)
-        
+
     def _new_array(self, n=(), dtype=float, zero=True,
                    global_array=False, pad=False):
         if global_array:
             shape = self.get_size_of_global_array(pad)
         else:
             shape = self.n_c
-            
+
         if isinstance(n, numbers.Integral):
             n = (n,)
 
@@ -224,7 +253,16 @@ class GridDescriptor(Domain):
             return np.zeros(shape, dtype)
         else:
             return np.empty(shape, dtype)
-        
+
+    def get_axial_communicator(self, axis):
+        peer_ranks = []
+        pos_c = self.parpos_c.copy()
+        for i in range(self.parsize_c[axis]):
+            pos_c[axis] = i
+            peer_ranks.append(self.get_rank_from_processor_position(pos_c))
+        peer_comm = self.comm.new_communicator(peer_ranks)
+        return peer_comm
+
     def integrate(self, a_xg, b_yg=None,
                   global_integral=True, hermitian=False,
                   _transposed_result=None):
@@ -243,9 +281,9 @@ class GridDescriptor(Domain):
         _transposed_result: ndarray
             Long story.  Don't use this unless you are a method of the
             MatrixOperator class ..."""
-        
+
         xshape = a_xg.shape[:-3]
-        
+
         if b_yg is None:
             # Only one array:
             result = a_xg.reshape(xshape + (-1,)).sum(axis=-1) * self.dv
@@ -271,31 +309,23 @@ class GridDescriptor(Domain):
             r2k(0.5 * self.dv, A_xg, B_yg, 0.0, result_yx)
         else:
             gemm(self.dv, A_xg, B_yg, 0.0, result_yx, 'c')
-        
+
         if global_integral:
             self.comm.sum(result_yx)
 
         yshape = b_yg.shape[:-3]
         result = result_yx.T.reshape(xshape + yshape)
-        
+
         if result.ndim == 0:
             return result.item()
         else:
             return result
 
-    def gemm(self, alpha, psit_nG, C_mn, beta, newpsit_mG):
-        """Helper function for MatrixOperator class."""
-        gemm(alpha, psit_nG, C_mn, beta, newpsit_mG)
-
-    def gemv(self, alpha, psit_nG, C_n, beta, newpsit_G, trans='t'):
-        """Helper function for CG eigensolver."""
-        gemv(alpha, psit_nG, C_n, beta, newpsit_G, trans)
-
     def coarsen(self):
         """Return coarsened `GridDescriptor` object.
 
         Reurned descriptor has 2x2x2 fewer grid points."""
-        
+
         if np.sometrue(self.N_c % 2):
             raise ValueError('Grid %s not divisible by 2!' % self.N_c)
 
@@ -306,7 +336,7 @@ class GridDescriptor(Domain):
 
         Returned descriptor has 2x2x2 more grid points."""
         return self.new_descriptor(self.N_c * 2)
-    
+
     def get_boxes(self, spos_c, rcut, cut=True):
         """Find boxes enclosing sphere."""
         N_c = self.N_c
@@ -330,25 +360,25 @@ class GridDescriptor(Domain):
                            'Beg. of box %s, end of box %s, max box size %s' %
                            (tuple(spos_c) + (beg_c, end_c, self.N_c)))
                     raise GridBoundsError(msg)
-                    
+
         range_c = ([], [], [])
-        
+
         for c in range(3):
             b = beg_c[c]
             e = b
-            
+
             while e < end_c[c]:
                 b0 = b % N_c[c]
-               
+
                 e = min(end_c[c], b + N_c[c] - b0)
 
                 if b0 < self.beg_c[c]:
                     b1 = b + self.beg_c[c] - b0
                 else:
                     b1 = b
-                    
+
                 e0 = b0 - b + e
-                              
+
                 if e0 > self.end_c[c]:
                     e1 = e - (e0 - self.end_c[c])
                 else:
@@ -356,7 +386,7 @@ class GridDescriptor(Domain):
                 if e1 > b1:
                     range_c[c].append((b1, e1))
                 b = e
-        
+
         boxes = []
 
         for b0, e0 in range_c[0]:
@@ -378,7 +408,7 @@ class GridDescriptor(Domain):
 
     def get_nearest_grid_point(self, spos_c, force_to_this_domain=False):
         """Return index of nearest grid point.
-        
+
         The nearest grid point can be on a different CPU than the one the
         nucleus belongs to (i.e. return can be negative, or larger than
         gd.end_c), in which case something clever should be done.
@@ -404,16 +434,18 @@ class GridDescriptor(Domain):
         where the wave vector is given by k_c (in units of reciprocal
         lattice vectors)."""
 
-        N_c = self.N_c
-        return np.exp(2j * pi * np.dot(np.indices(N_c).T, k_c / N_c).T)
+        index_Gc = np.indices(self.n_c).T + self.beg_c
+        return np.exp(2j * pi * np.dot(index_Gc, k_c / self.N_c).T)
 
     def symmetrize(self, a_g, op_scc, ft_sc=None):
+        # ft_sc: fractional translations
+        # XXXX documentation missing.  This is some kind of array then?
         if len(op_scc) == 1:
             return
-        
+
         if ft_sc is not None and not ft_sc.any():
             ft_sc = None
-            
+
         A_g = self.collect(a_g)
         if self.comm.rank == 0:
             B_g = np.zeros_like(A_g)
@@ -426,7 +458,7 @@ class GridDescriptor(Domain):
             B_g = None
         self.distribute(B_g, a_g)
         a_g /= len(op_scc)
-    
+
     def collect(self, a_xg, broadcast=False):
         """Collect distributed array to master-CPU or all CPU's."""
         if self.comm.size == 1:
@@ -444,7 +476,7 @@ class GridDescriptor(Domain):
                 self.comm.broadcast(A_xg, 0)
                 return A_xg
             else:
-                return None
+                return np.nan
 
         # Put the subdomains from the slaves into the big array
         # for the whole domain:
@@ -478,7 +510,7 @@ class GridDescriptor(Domain):
         if self.comm.size == 1:
             b_xg[:] = B_xg
             return
-        
+
         if self.rank != 0:
             self.comm.receive(b_xg, 0, 42)
             return
@@ -502,31 +534,49 @@ class GridDescriptor(Domain):
                         else:
                             b_xg[:] = B_xg[..., b0:e0, b1:e1, b2:e2]
                         r += 1
-                        
+
             for request, a_xg in requests:
                 self.comm.wait(request)
-        
-    def zero_pad(self, a_xg):
+
+    def zero_pad(self, a_xg, global_array=True):
         """Pad array with zeros as first element along non-periodic directions.
 
-        Should only be invoked on global arrays.
+        Array may either be local or in standard decomposition.
         """
-        assert np.all(a_xg.shape[-3:] == (self.N_c + self.pbc_c - 1))
+
+        # We could infer what global_array should be from a_xg.shape.
+        # But as it is now, there is a bit of redundancy to avoid
+        # confusing errors
+
+        gshape = a_xg.shape[-3:]
+        padding_c = 1 - self.pbc_c
+        if global_array:
+            assert (gshape == self.N_c - padding_c).all(), gshape
+            bshape = tuple(self.N_c)
+        else:
+            assert (gshape == self.n_c).all()
+            parpos_c = self.get_processor_position_from_rank()
+            # Only pad where domain is on edge:
+            padding_c *= (parpos_c == 0)
+            bshape = tuple(self.n_c + padding_c)
+
         if self.pbc_c.all():
             return a_xg
 
-        npbx, npby, npbz = 1 - self.pbc_c
-        b_xg = np.zeros(a_xg.shape[:-3] + tuple(self.N_c), dtype=a_xg.dtype)
+        npbx, npby, npbz = padding_c
+        b_xg = np.zeros(a_xg.shape[:-3] + tuple(bshape), dtype=a_xg.dtype)
         b_xg[..., npbx:, npby:, npbz:] = a_xg
         return b_xg
 
-    def calculate_dipole_moment(self, rho_g):
+    def calculate_dipole_moment(self, rho_g, center=False):
         """Calculate dipole moment of density."""
+        r_cz = [np.arange(self.beg_c[c], self.end_c[c]) for c in range(3)]
+        if center:
+            r_cz = [r_cz[c] - 0.5 * self.N_c[c] for c in range(3)]
         rho_01 = rho_g.sum(axis=2)
         rho_02 = rho_g.sum(axis=1)
-        rho_cg = [rho_01.sum(axis=1), rho_01.sum(axis=0), rho_02.sum(axis=0)]
-        rhog_c = [np.dot(np.arange(self.beg_c[c], self.end_c[c]), rho_cg[c])
-                  for c in range(3)]
+        rho_cz = [rho_01.sum(axis=1), rho_01.sum(axis=0), rho_02.sum(axis=0)]
+        rhog_c = [np.dot(r_cz[c], rho_cz[c]) for c in range(3)]
         d_c = -np.dot(rhog_c, self.h_cv) * self.dv
         self.comm.sum(d_c)
         return d_c
@@ -539,7 +589,7 @@ class GridDescriptor(Domain):
             ~       ~     -i G.r   ~
             Z   = <psi | e      |psi >
              nm       n             m
-                    
+
         psit_nG and psit_nG1 are the set of wave functions for the two
         different spin/kpoints in question.
 
@@ -568,7 +618,7 @@ class GridDescriptor(Domain):
             z = self.integrate(a_R, np.exp(2j * pi / L * r_R))
             center.append(np.angle(z) / (2 * pi) * L % L)
         return np.array(center)
-        
+
     def bytecount(self, dtype=float):
         """Get the number of bytes used by a grid of specified dtype."""
         return int(np.prod(self.n_c)) * np.array(1, dtype).itemsize
@@ -591,16 +641,19 @@ class GridDescriptor(Domain):
         """
         s_Gc = (np.indices(self.n_c, dtype).T + self.beg_c) / self.N_c
         cell_cv = self.N_c * self.h_cv
-        s_Gc -= np.linalg.solve(cell_cv.T, r_v)
-        
+        r_c =  np.linalg.solve(cell_cv.T, r_v)
+        # do the correction twice works better because of rounding errors
+        # e.g.: -1.56250000e-25 % 1.0 = 1.0,
+        #      but (-1.56250000e-25 % 1.0) % 1.0 = 0.0
+        r_c = np.where(self.pbc_c, r_c % 1.0, r_c)
+        s_Gc -= np.where(self.pbc_c, r_c % 1.0, r_c)
+
         if mic:
-            # XXX do the correction twice works better
-            s_Gc -= self.pbc_c * (2 * s_Gc).astype(int)
             s_Gc -= self.pbc_c * (2 * s_Gc).astype(int)
             # sanity check
             assert((s_Gc * self.pbc_c >= -0.5).all())
             assert((s_Gc * self.pbc_c <= 0.5).all())
-                
+
         return np.dot(s_Gc, cell_cv).T.copy()
 
     def interpolate_grid_points(self, spos_nc, vt_g, target_n, use_mlsqr=True):
@@ -611,11 +664,11 @@ class GridDescriptor(Domain):
 
         Uses moving least squares algorithm by default, or otherwise
         trilinear interpolation.
-        
+
         This doesn't work in parallel, since it would require
         communication between neighbouring grid.  """
 
-        assert mpi.world.size == 1
+        assert self.comm.size == 1
 
         if use_mlsqr:
             mlsqr(3, 2.3, spos_nc, self.N_c, self.beg_c, vt_g, target_n)
@@ -651,6 +704,9 @@ class GridDescriptor(Domain):
                     (0.0 + dg_c[0]) * (0.0 + dg_c[1]) * (0.0 + dg_c[2]))
 
     def __eq__(self, other):
+        # XXX Wait, should this not check the global distribution?  This
+        # could return True on some nodes and False on others because the
+        # check does not verify self.n_cp.
         return (self.dv == other.dv and
                 (self.h_cv == other.h_cv).all() and
                 (self.N_c == other.N_c).all() and
