@@ -1,7 +1,5 @@
 import numpy as np
 
-from gpaw.utilities.blas import gemm
-
 
 class DirectLCAO(object):
     """Eigensolver for LCAO-basis calculation"""
@@ -10,7 +8,7 @@ class DirectLCAO(object):
         self.diagonalizer = diagonalizer
         # ??? why should we be able to set
         # this diagonalizer in both constructor and initialize?
-        self.has_initialized = False # XXX
+        self.has_initialized = False  # XXX
 
     def initialize(self, gd, dtype, nao, diagonalizer=None):
         self.gd = gd
@@ -18,7 +16,7 @@ class DirectLCAO(object):
         if diagonalizer is not None:
             self.diagonalizer = diagonalizer
         assert self.diagonalizer is not None
-        self.has_initialized = True # XXX
+        self.has_initialized = True  # XXX
 
     def reset(self):
         pass
@@ -26,11 +24,11 @@ class DirectLCAO(object):
     @property
     def error(self):
         return 0.0
-        
+
     @error.setter
     def error(self, e):
         pass
-        
+
     def calculate_hamiltonian_matrix(self, hamiltonian, wfs, kpt, Vt_xMM=None,
                                      root=-1, add_kinetic=True):
         # XXX document parallel stuff, particularly root parameter
@@ -41,8 +39,10 @@ class DirectLCAO(object):
         # distributed_atomic_correction works with ScaLAPACK/BLACS in general.
         # If SL is not enabled, it will not work with band parallelization.
         # But no one would want that for a practical calculation anyway.
-        dH_asp = wfs.atomic_correction.redistribute(wfs, hamiltonian.dH_asp)
-        
+        # dH_asp = wfs.atomic_correction.redistribute(wfs, hamiltonian.dH_asp)
+        # XXXXX fix atomic corrections
+        dH_asp = hamiltonian.dH_asp
+
         if Vt_xMM is None:
             wfs.timer.start('Potential matrix')
             vt_G = hamiltonian.vt_sG[kpt.s]
@@ -68,7 +68,6 @@ class DirectLCAO(object):
         #  mu nu    --   mu i  ij nu j
         #           aij
         #
-
         name = wfs.atomic_correction.__class__.__name__
         wfs.timer.start(name)
         wfs.atomic_correction.calculate_hamiltonian(wfs, kpt, dH_asp, H_MM, yy)
@@ -108,7 +107,7 @@ class DirectLCAO(object):
 
         if kpt.eps_n is None:
             kpt.eps_n = np.empty(wfs.bd.mynbands)
-            
+
         diagonalization_string = repr(self.diagonalizer)
         wfs.timer.start(diagonalization_string)
         self.diagonalizer.diagonalize(H_MM, kpt.C_nM, kpt.eps_n, S_MM)
