@@ -1,15 +1,12 @@
 from __future__ import print_function
 from math import sqrt
-import numpy as np
-import gpaw.mpi as mpi
-MASTER = mpi.MASTER
 
+import numpy as np
 from ase.units import Hartree
+from ase.utils import convert_string_to_fd
 from ase.utils.timing import Timer
 
 import gpaw.mpi as mpi
-#from gpaw.poisson import PoissonSolver
-from gpaw.output import get_txt
 from gpaw.lrtddft.kssingle import KSSingles
 from gpaw.transformers import Transformer
 from gpaw.utilities import pack
@@ -42,15 +39,15 @@ class OmegaMatrix:
                  xc=None,
                  derivativeLevel=None,
                  numscale=0.001,
+                 poisson=None,
                  filehandle=None,
                  txt=None,
                  finegrid=2,
-                 eh_comm=None,
-                 ):
+                 eh_comm=None):
 
         if not txt and calculator:
-            txt = calculator.txt
-        self.txt = get_txt(txt, mpi.rank)
+            txt = calculator.log.fd
+        self.txt = convert_string_to_fd(txt, mpi.world)
 
         if eh_comm is None:
             eh_comm = mpi.serial_comm
@@ -74,21 +71,21 @@ class OmegaMatrix:
         # handle different grid possibilities
         self.restrict = None
         # self.poisson = PoissonSolver(nn=self.paw.hamiltonian.poisson.nn)
-        self.poisson = calculator.hamiltonian.poisson
+        if poisson is None:
+            self.poisson = calculator.hamiltonian.poisson
+        else:
+            self.poisson = poisson
         if finegrid:
             self.poisson.set_grid_descriptor(self.paw.density.finegd)
-            self.poisson.initialize()
 
             self.gd = self.paw.density.finegd
             if finegrid == 1:
                 self.gd = wfs.gd
         else:
             self.poisson.set_grid_descriptor(wfs.gd)
-            self.poisson.initialize()
             self.gd = wfs.gd
         self.restrict = Transformer(self.paw.density.finegd, wfs.gd,
-                                    self.paw.input_parameters.stencils[1]
-                                    ).apply
+                                    self.paw.density.stencil).apply
 
         if xc == 'RPA':
             xc = None  # enable RPA as keyword
@@ -493,10 +490,11 @@ class OmegaMatrix:
                 skss.append((ks + ks) / sqrt(2))
                 tkss.append((ks - ks) / sqrt(2))
                 map.append(ij)
-
+        skss.istart = tkss.istart = self.fullkss.istart
+        skss.jend = tkss.jend = self.fullkss.jend
         nkss = len(skss)
 
-        # define the singlet and the triplet omega-matrixes
+        # define the singlet and the triplet omega-matrices
         sOm = OmegaMatrix(kss=skss)
         sOm.full = np.empty((nkss, nkss))
         tOm = OmegaMatrix(kss=tkss)
@@ -581,13 +579,9 @@ class OmegaMatrix:
 
         return map, kss
 
-    def diagonalize(self, istart=None, jend=None, energy_range=None,
-                    TDA=False):
+    def diagonalize(self, istart=None, jend=None, energy_range=None):
         """Evaluate Eigenvectors and Eigenvalues:"""
 
-        if TDA:
-            raise NotImplementedError
-
         map, kss = self.get_map(istart, jend, energy_range)
         nij = len(kss)
         if map is None:
@@ -624,10 +618,9 @@ class OmegaMatrix:
         nij = int(f.readline())
         full = np.zeros((nij, nij))
         for ij in range(nij):
-            l = f.readline().split()
-            for kq in range(ij, nij):
-                full[ij, kq] = float(l[kq - ij])
-                full[kq, ij] = full[ij, kq]
+            l = [float(x) for x in f.readline().split()]
+            full[ij, ij:] = l
+            full[ij:, ij] = l
         self.full = full
 
         if fh is None:
@@ -635,7 +628,12 @@ class OmegaMatrix:
 
     def write(self, filename=None, fh=None):
         """Write current state to a file."""
-        if mpi.rank == mpi.MASTER:
+
+        try:
+            rank = self.paw.wfs.world.rank
+        except AttributeError:
+            rank = mpi.world.rank
+        if rank == 0:
             if fh is None:
                 f = open(filename, 'w')
             else:
