@@ -7,19 +7,22 @@ import time
 import traceback
 import atexit
 import pickle
-from fractions import gcd
 
 import numpy as np
 
 from gpaw import debug
 from gpaw import dry_run as dry_run_size
-from gpaw.utilities import is_contiguous
 
 import _gpaw
 
 MASTER = 0
 
 
+def is_contiguous(*args, **kwargs):
+    from gpaw.utilities import is_contiguous
+    return is_contiguous(*args, **kwargs)
+
+
 class _Communicator:
     def __init__(self, comm, parent=None):
         """Construct a wrapper of the C-object for any MPI-communicator.
@@ -231,16 +234,16 @@ class _Communicator:
         Parameters:
 
         sbuffer: ndarray
-            Source of the data to distribute, i.e. send buffers on all rank.
+            Source of the data to distribute, i.e., send buffers on all ranks
         scounts: ndarray
             Integer array equal to the group size specifying the number of
             elements to send to each processor
-        sbuffer: ndarray
+        sdispls: ndarray
             Integer array (of length group size). Entry j specifies the
             displacement (relative to sendbuf from which to take the
-            outgoing data destined for process j
+            outgoing data destined for process j)
         rbuffer: ndarray
-            Destination of the distributed data, i.e. local receive buffer.
+            Destination of the distributed data, i.e., local receive buffer.
         rcounts: ndarray
             Integer array equal to the group size specifying the maximum
             number of elements that can be received from each processor.
@@ -256,9 +259,17 @@ class _Communicator:
         assert rcounts.flags.contiguous
         assert rdispls.flags.contiguous
         assert sbuffer.dtype == rbuffer.dtype
-        # FIXME: more tests
-        self.comm.alltoallv(sbuffer, scounts, sdispls, rbuffer, rcounts,
-                            rdispls)
+
+        for arr in [scounts, sdispls, rcounts, rdispls]:
+            assert arr.dtype == np.int, arr.dtype
+            assert len(arr) == self.size
+
+        assert np.all(0 <= sdispls)
+        assert np.all(0 <= rdispls)
+        assert np.all(sdispls + scounts <= sbuffer.size)
+        assert np.all(rdispls + rcounts <= rbuffer.size)
+        self.comm.alltoallv(sbuffer, scounts, sdispls,
+                            rbuffer, rcounts, rdispls)
 
     def all_gather(self, a, b):
         """Gather data from all ranks onto all processes in a group.
@@ -495,6 +506,39 @@ class _Communicator:
         """Block execution until all process have reached this point."""
         self.comm.barrier()
 
+    def compare(self, othercomm):
+        """Compare communicator to other.
+
+        Returns 'ident' if they are identical, 'congruent' if they are
+        copies of each other, 'similar' if they are permutations of
+        each other, and otherwise 'unequal'.
+
+        This method corresponds to MPI_Comm_compare."""
+        if isinstance(self.comm, SerialCommunicator):
+            return self.comm.compare(othercomm.comm)  # argh!
+        result = self.comm.compare(othercomm.get_c_object())
+        assert result in ['ident', 'congruent', 'similar', 'unequal']
+        return result
+
+    def translate_ranks(self, other, ranks):
+        """"Translate ranks from communicator to other.
+
+        ranks must be valid on this communicator.  Returns ranks
+        on other communicator corresponding to the same processes.
+        Ranks that are not defined on the other communicator are
+        assigned values of -1.  (In contrast to MPI which would
+        assign MPI_UNDEFINED)."""
+        assert hasattr(other, 'translate_ranks'), \
+            'Excpected communicator, got %s' % other
+        assert all(0 <= rank for rank in ranks)
+        assert all(rank < self.size for rank in ranks)
+        if isinstance(self.comm, SerialCommunicator):
+            return self.comm.translate_ranks(other.comm, ranks)  # argh!
+        otherranks = self.comm.translate_ranks(other.get_c_object(), ranks)
+        assert all(-1 <= rank for rank in otherranks)
+        assert ranks.dtype == otherranks.dtype
+        return otherranks
+
     def get_members(self):
         """Return the subset of processes which are members of this MPI group
         in terms of the ranks they are assigned on the parent communicator.
@@ -568,9 +612,14 @@ class SerialCommunicator:
         b[:] = a
 
     def alltoallv(self, sbuffer, scounts, sdispls, rbuffer, rcounts, rdispls):
-        rbuffer[:] = sbuffer
-        rcounts[:] = scounts
-        rdispls[:] = sdispls
+        assert len(scounts) == 1
+        assert len(sdispls) == 1
+        assert len(rcounts) == 1
+        assert len(rdispls) == 1
+        assert len(sbuffer) == len(rbuffer)
+
+        rbuffer[rdispls[0]:rdispls[0] + rcounts[0]] = \
+            sbuffer[sdispls[0]:sdispls[0] + scounts[0]]
 
     def new_communicator(self, ranks):
         if self.rank not in ranks:
@@ -596,6 +645,21 @@ class SerialCommunicator:
     def get_members(self):
         return np.array([0])
 
+    def compare(self, other):
+        if self == other:
+            return 'ident'
+        elif isinstance(other, SerialCommunicator):
+            return 'congruent'
+        else:
+            raise NotImplementedError('Compare serial comm to other')
+
+    def translate_ranks(self, other, ranks):
+        if isinstance(other, SerialCommunicator):
+            assert all(rank == 0 for rank in ranks)
+            return np.zeros(len(ranks), dtype=int)
+        raise NotImplementedError(
+            'Translate non-trivial ranks with serial comm')
+
     def get_c_object(self):
         raise NotImplementedError('Should not get C-object for serial comm')
 
@@ -615,18 +679,22 @@ else:
     except AttributeError:
         world = serial_comm
 
-    
+
 class DryRunCommunicator(SerialCommunicator):
     def __init__(self, size=1, parent=None):
         self.size = size
         self.parent = parent
-    
+
     def new_communicator(self, ranks):
         return DryRunCommunicator(len(ranks), parent=self)
 
     def get_c_object(self):
         return None  # won't actually be passed to C
 
+    def translate_ranks(self, comm, ranks):
+        return np.zeros_like(ranks)
+
+
 if dry_run_size > 1:
     world = DryRunCommunicator(dry_run_size)
 
@@ -639,6 +707,12 @@ if debug:
 size = world.size
 rank = world.rank
 parallel = (size > 1)
+try:
+    world.get_c_object()
+except NotImplementedError:
+    have_mpi = False
+else:
+    have_mpi = True
 
 
 # XXXXXXXXXX for easier transition to Parallelization class
@@ -652,6 +726,7 @@ def distribute_cpus(parsize_domain, parsize_bands,
     else:
         # Plane wave mode:
         if parsize_bands is None:
+            from ase.utils import gcd
             parsize_bands = comm.size // gcd(nsk, comm.size)
 
     p = Parallelization(comm, nsk)
@@ -676,7 +751,7 @@ def broadcast(obj, root=0, comm=world):
 
 def synchronize_atoms(atoms, comm, tolerance=1e-8):
     """Synchronize atoms between multiple CPUs removing numerical noise.
-    
+
     If the atoms differ significantly, raise ValueError on all ranks.
     The error object contains the ranks where the check failed.
 
@@ -684,42 +759,57 @@ def synchronize_atoms(atoms, comm, tolerance=1e-8):
 
     if len(atoms) == 0:
         return
-    
-    if comm.rank == 0:
-        src_atoms = atoms
-    else:
-        src_atoms = None
 
-    newatoms = broadcast(src_atoms, root=0, comm=comm)
-    err = np.abs(newatoms.positions - atoms.positions).max()
-    # Now copy positions array so we can check for strict identity
-    atoms.positions[:, :] = newatoms.positions[:, :]
+    if comm.rank == 0:
+        src = (atoms.positions, atoms.cell, atoms.numbers, atoms.pbc)
+    else:
+        src = None
+
+    # XXX replace with ase.cell.same_cell in the future
+    # (if that functions gets to exist)
+    #def same_cell(cell1, cell2):
+    #    return ((cell1 is None) == (cell2 is None) and
+    #            (cell1 is None or (cell1 == cell2).all()))
+
+    # Cell vectors should be compared with a tolerance like positions?
+    def same_cell(cell1, cell2, tolerance=1e-8):
+        return ((cell1 is None) == (cell2 is None) and
+                (cell1 is None or (abs(cell1 - cell2).max() <= tolerance)))
+
+    
+    positions, cell, numbers, pbc = broadcast(src, root=0, comm=comm)
+    ok = (len(positions) == len(atoms.positions) and
+          (abs(positions - atoms.positions).max() <= tolerance) and
+          (numbers == atoms.numbers).all() and
+          same_cell(cell, atoms.cell) and
+          (pbc == atoms.pbc).all())
 
     # We need to fail equally on all ranks to avoid trouble.  Thus
     # we use an array to gather check results from everyone.
-    my_fail = np.array(err > tolerance or newatoms != atoms,
-                       dtype=bool)
-
+    my_fail = np.array(not ok, dtype=bool)
     all_fail = np.zeros(comm.size, dtype=bool)
     comm.all_gather(my_fail, all_fail)
 
     if all_fail.any():
-        err_ranks = np.arange(comm.size)[all_fail]
         if debug:
-            fd = open('synchronize_atoms_r%d.pckl' % comm.rank, 'wb')
-            pickle.dump((newatoms, atoms), fd)
-            fd.close()
+            with open('synchronize_atoms_r%d.pckl' % comm.rank, 'wb') as fd:
+                pickle.dump((atoms.positions, atoms.cell,
+                             atoms.numbers, atoms.pbc,
+                             positions, cell, numbers, pbc), fd)
+        err_ranks = np.arange(comm.size)[all_fail]
         raise ValueError('Mismatch of Atoms objects.  In debug '
                          'mode, atoms will be dumped to files.',
                          err_ranks)
 
-        
+    atoms.positions = positions
+    atoms.cell = cell
+
 def broadcast_string(string=None, root=0, comm=world):
     if comm.rank == root:
         string = string.encode()
     return broadcast_bytes(string, root, comm).decode()
-    
-    
+
+
 def broadcast_bytes(b=None, root=0, comm=world):
     """Broadcast a bytes across an MPI communicator and return it."""
     if comm.rank == root:
@@ -736,12 +826,12 @@ def broadcast_bytes(b=None, root=0, comm=world):
     comm.broadcast(b, root)
     return b.tostring()
 
-    
+
 def send_string(string, rank, comm=world):
     comm.send(np.array(len(string)), rank)
     comm.send(np.fromstring(string, np.int8), rank)
 
-    
+
 def receive_string(rank, comm=world):
     n = np.array(0)
     comm.receive(n, rank)
@@ -749,7 +839,7 @@ def receive_string(rank, comm=world):
     comm.receive(string, rank)
     return string.tostring().decode()
 
-    
+
 def alltoallv_string(send_dict, comm=world):
     scounts = np.zeros(comm.size, dtype=np.int)
     sdispls = np.zeros(comm.size, dtype=np.int)
@@ -788,7 +878,7 @@ def alltoallv_string(send_dict, comm=world):
 
     return rdict
 
-    
+
 def ibarrier(timeout=None, root=0, tag=123, comm=world):
     """Non-blocking barrier returning a list of requests to wait for.
     An optional time-out may be given, turning the call into a blocking
@@ -819,7 +909,7 @@ def ibarrier(timeout=None, root=0, tag=123, comm=world):
             raise RuntimeError('MPI barrier timeout.')
     return []
 
-    
+
 def run(iterators):
     """Run through list of iterators one step at a time."""
     if not isinstance(iterators, list):
@@ -837,17 +927,17 @@ def run(iterators):
         except StopIteration:
             return results
 
-            
+
 class Parallelization:
     def __init__(self, comm, nspinkpts):
         self.comm = comm
         self.size = comm.size
         self.nspinkpts = nspinkpts
-        
+
         self.kpt = None
         self.domain = None
         self.band = None
-        
+
         self.nclaimed = 1
         self.navail = comm.size
 
@@ -858,11 +948,13 @@ class Parallelization:
             self.domain = domain
         if band is not None:
             self.band = band
-        
+
         nclaimed = 1
         for group, name in zip([self.kpt, self.domain, self.band],
                                ['k-point', 'domain', 'band']):
             if group is not None:
+                assert group > 0, ('Bad: Only {} cores requested for '
+                                   '{} parallelization'.format(group, name))
                 if self.size % group != 0:
                     msg = ('Cannot parallelize as the '
                            'communicator size %d is not divisible by the '
@@ -871,7 +963,7 @@ class Parallelization:
                     raise ValueError(msg)
                 nclaimed *= group
         navail = self.size // nclaimed
-        
+
         assert self.size % nclaimed == 0
         assert self.size % navail == 0
 
@@ -899,7 +991,7 @@ class Parallelization:
         ranks."""
         self.set(kpt=kpt, domain=domain, band=band)
         self.autofinalize()
-        
+
         comm = self.comm
         rank = comm.rank
         communicators = {}
@@ -908,7 +1000,7 @@ class Parallelization:
 
         groups = dict(k=self.kpt, b=self.band, d=self.domain)
 
-        # Build communicators in hierachical manner
+        # Build communicators in hierarchical manner
         # The ranks in the first group have largest separation while
         # the ranks in the last group are next to each other
         for name in order:
@@ -928,7 +1020,7 @@ class Parallelization:
         # grid comm: a communicator uniting all cores with the same domain.
         c1, c2, c3 = [communicators[name] for name in order]
         allranks = [range(c1.size), range(c2.size), range(c3.size)]
-        
+
         def get_communicator_complement(name):
             relevant_ranks = list(allranks)
             relevant_ranks[order.find(name)] = [communicators[name].rank]
@@ -937,7 +1029,7 @@ class Parallelization:
                               for r2 in relevant_ranks[1]
                               for r3 in relevant_ranks[2]])
             return comm.new_communicator(ranks)
-        
+
         # The communicator of all processes that share a domain, i.e.
         # the combination of k-point and band dommunicators.
         communicators['D'] = get_communicator_complement('d')
@@ -946,7 +1038,7 @@ class Parallelization:
         # and LCAO orbital stuff.
         communicators['K'] = get_communicator_complement('k')
         return communicators
-    
+
     def autofinalize(self):
         if self.kpt is None:
             self.set(kpt=self.get_optimal_kpt_parallelization())
@@ -956,16 +1048,21 @@ class Parallelization:
             self.set(band=self.navail)
 
         if self.navail > 1:
-            raise RuntimeError('All the CPUs must be used')
-    
+            assignments = dict(kpt=self.kpt,
+                               domain=self.domain,
+                               band=self.band)
+            raise RuntimeError('All the CPUs must be used.  Have %s but '
+                               '%d times more are available'
+                               % (assignments, self.navail))
+
     def get_optimal_kpt_parallelization(self, kptprioritypower=1.4):
         if self.domain and self.band:
             # Try to use all the CPUs for k-point parallelization
             ncpus = min(self.nspinkpts, self.navail)
             return ncpus
         ncpuvalues, wastevalues = self.find_kpt_parallelizations()
-        scores = ((self.navail // ncpuvalues)
-                  * ncpuvalues**kptprioritypower)**(1.0 - wastevalues)
+        scores = ((self.navail // ncpuvalues) *
+                  ncpuvalues**kptprioritypower)**(1.0 - wastevalues)
         arg = np.argmax(scores)
         ncpus = ncpuvalues[arg]
         return ncpus
@@ -974,7 +1071,7 @@ class Parallelization:
         nspinkpts = self.nspinkpts
         ncpuvalues = []
         wastevalues = []
-        
+
         ncpus = nspinkpts
         while ncpus > 0:
             if self.navail % ncpus == 0:
@@ -1010,11 +1107,11 @@ def print_mpi_stack_trace(type, value, tb):
     still be printed from different ranks in random order, but
     one can grep for a rank or run 'sort' on the output to obtain
     readable data."""
-    
+
     exception_text = traceback.format_exception(type, value, tb)
     ndigits = len(str(world.size - 1))
     rankstring = ('%%0%dd' % ndigits) % world.rank
-    
+
     lines = []
     # The exception elements may contain newlines themselves
     for element in exception_text:
@@ -1026,10 +1123,11 @@ def print_mpi_stack_trace(type, value, tb):
         lineno = ('%%0%dd' % line_ndigits) % lineno
         sys.stderr.write('rank=%s L%s: %s\n' % (rankstring, lineno, line))
 
+
 if world.size > 1:  # Triggers for dry-run communicators too, but we care not.
     sys.excepthook = print_mpi_stack_trace
 
-            
+
 def exit(error='Manual exit'):
     # Note that exit must be called on *all* MPI tasks
     atexit._exithandlers = []  # not needed because we are intentially exiting
@@ -1043,4 +1141,5 @@ def exit(error='Manual exit'):
     world.barrier()  # sync up before exiting
     sys.exit()  # quit for serial case, return to _gpaw.c for parallel case
 
+
 atexit.register(cleanup)
