@@ -1,23 +1,26 @@
 # Copyright (C) 2003  CAMP
 # Please see the accompanying LICENSE file for further information.
 
-from math import pi
 import warnings
+from math import pi
 
 import numpy as np
 from numpy.fft import fftn, ifftn, fft2, ifft2
 
-from gpaw.transformers import Transformer
-from gpaw.fd_operators import Laplace, LaplaceA, LaplaceB
 from gpaw import PoissonConvergenceError
+from gpaw.dipole_correction import DipoleCorrection
+from gpaw.domain import decompose_domain
+from gpaw.fd_operators import Laplace, LaplaceA, LaplaceB
+from gpaw.transformers import Transformer
 from gpaw.utilities.blas import axpy
 from gpaw.utilities.gauss import Gaussian
+from gpaw.utilities.grid import grid2grid
 from gpaw.utilities.ewald import madelung
 from gpaw.utilities.tools import construct_reciprocal
 import _gpaw
 
 
-POISSON_GRID_WARNING="""Grid unsuitable for Poisson solver!
+POISSON_GRID_WARNING = """Grid unsuitable for Poisson solver!
 
 The Poisson solver does not have sufficient multigrid levels for good
 performance and will converge inefficiently if at all, or yield wrong
@@ -31,7 +34,7 @@ or 32 depending on system size; examples:
 
 or
 
-  from gpaw.tools import h2gpts
+  from gpaw.utilities import h2gpts
   GPAW(gpts=h2gpts(0.2, atoms.get_cell(), idiv=16))
 
 Parallelizing over very small domains can also undesirably limit the
@@ -39,108 +42,79 @@ number of multigrid levels even if the total number of grid points
 is divisible by a high power of 2."""
 
 
-class PoissonSolver:
-    def __init__(self, nn=3, relax='J', eps=2e-10, maxiter=1000,
-                 remove_moment=None, use_charge_center=False):
-        self.relax = relax
-        self.nn = nn
-        self.eps = eps
-        self.charged_periodic_correction = None
-        self.maxiter = maxiter
-        self.remove_moment = remove_moment
-        self.use_charge_center = use_charge_center
+def create_poisson_solver(name='fd', **kwargs):
+    if isinstance(name, _PoissonSolver):
+        return name
+    elif isinstance(name, dict):
+        kwargs.update(name)
+        return create_poisson_solver(**kwargs)
+    elif name == 'fft':
+        return FFTPoissonSolver(**kwargs)
+    elif name == 'fdtd':
+        from gpaw.fdtd.poisson_fdtd import FDTDPoissonSolver
+        return FDTDPoissonSolver(**kwargs)
+    elif name == 'fd':
+        return PoissonSolver(**kwargs)
+    elif name == 'ExtraVacuumPoissonSolver':
+        from gpaw.poisson_extravacuum import ExtraVacuumPoissonSolver
+        return ExtraVacuumPoissonSolver(**kwargs)
+    else:
+        raise ValueError('Unknown poisson solver: %s' % name)
 
-        # Relaxation method
-        if relax == 'GS':
-            # Gauss-Seidel
-            self.relax_method = 1
-        elif relax == 'J':
-            # Jacobi
-            self.relax_method = 2
-        else:
-            raise NotImplementedError('Relaxation method %s' % relax)
 
-        self.description = None
+def PoissonSolver(dipolelayer=None, **kwargs):
+    if dipolelayer is not None:
+        return DipoleCorrection(PoissonSolver(**kwargs), dipolelayer)
+    return FDPoissonSolver(**kwargs)
 
-    def get_stencil(self):
-        return self.nn
+
+class _PoissonSolver(object):
+    """Abstract PoissonSolver class
+
+       This class defines an interface and a common ancestor
+       for various PoissonSolver implementations (including wrappers)."""
+    def __init__(self):
+        object.__init__(self)
 
     def set_grid_descriptor(self, gd):
-        # Should probably be renamed initialize
-        self.gd = gd
-        scale = -0.25 / pi
+        raise NotImplementedError()
 
-        if self.nn == 'M':
-            if not gd.orthogonal:
-                raise RuntimeError('Cannot use Mehrstellen stencil with '
-                                   'non orthogonal cell.')
+    def solve(self):
+        raise NotImplementedError()
 
-            self.operators = [LaplaceA(gd, -scale)]
-            self.B = LaplaceB(gd)
-        else:
-            self.operators = [Laplace(gd, scale, self.nn)]
-            self.B = None
-
-        self.interpolators = []
-        self.restrictors = []
-
-        level = 0
-        self.presmooths = [2]
-        self.postsmooths = [1]
-
-        # Weights for the relaxation,
-        # only used if 'J' (Jacobi) is chosen as method
-        self.weights = [2.0 / 3.0]
-
-        while level < 8:
-            try:
-                gd2 = gd.coarsen()
-            except ValueError:
-                break
-            self.operators.append(Laplace(gd2, scale, 1))
-            self.interpolators.append(Transformer(gd2, gd))
-            self.restrictors.append(Transformer(gd, gd2))
-            self.presmooths.append(4)
-            self.postsmooths.append(4)
-            self.weights.append(1.0)
-            level += 1
-            gd = gd2
-
-        self.levels = level
-
-        if self.operators[-1].gd.N_c.max() > 36:
-            # Try to warn exactly once no matter how one uses the solver.
-            if gd.comm.parent is None:
-                warn = (gd.comm.rank == 0)
-            else:
-                warn = (gd.comm.parent.rank == 0)
-
-            if warn:
-                warntxt = '\n'.join([POISSON_GRID_WARNING, '',
-                                     self.get_description()])
-            else:
-                warntxt = ('Poisson warning from domain rank %d'
-                           % self.gd.comm.rank)
-
-            # Warn from all ranks to avoid deadlocks.
-            warnings.warn(warntxt, stacklevel=2)
+    def todict(self):
+        raise NotImplementedError(self.__class__.__name__)
 
     def get_description(self):
-        name = {1: 'Gauss-Seidel', 2: 'Jacobi'}[self.relax_method]
-        coarsest_grid = self.operators[-1].gd.N_c
-        coarsest_grid_string = ' x '.join([str(N) for N in coarsest_grid])
-        assert self.levels + 1 == len(self.operators)
-        lines = ['%s solver with %d multi-grid levels'
-                 % (name, self.levels + 1),
-                 '    Coarsest grid: %s points' % coarsest_grid_string]
-        if coarsest_grid.max() > 24:
-            # This friendly warning has lower threshold than the big long
-            # one that we print when things are really bad.
-            lines.extend(['    Warning: Coarse grid has more than 24 points.',
-                          '             More multi-grid levels recommended.'])
-        lines.extend(['    Stencil: %s' % self.operators[0].description,
-                      '    Tolerance: %e' % self.eps,
-                      '    Max iterations: %d' % self.maxiter])
+        return self.__class__.__name__
+
+    def estimate_memory(self, mem):
+        raise NotImplementedError()
+
+
+class BasePoissonSolver(_PoissonSolver):
+    def __init__(self, eps=None, remove_moment=None, use_charge_center=False):
+        self.gd = None
+        self.remove_moment = remove_moment
+        self.use_charge_center = use_charge_center
+        self.eps = eps
+
+    def todict(self):
+        d = {'name': 'basepoisson'}
+        if self.eps is not None:
+            d['eps'] = self.eps
+        if self.remove_moment:
+            d['remove_moment'] = self.remove_moment
+        if self.use_charge_center:
+            d['use_charge_center'] = self.use_charge_center
+        return d
+
+    def get_description(self):
+        # The idea is that the subclass writes a header and main parameters,
+        # then adds the below string.
+        lines = []
+        if self.eps is not None:
+            lines.append('    Tolerance: %e' % self.eps),
         if self.remove_moment is not None:
             lines.append('    Remove moments up to L=%d' % self.remove_moment)
         if self.use_charge_center:
@@ -148,37 +122,11 @@ class PoissonSolver:
                          'majority charge')
         return '\n'.join(lines)
 
-    def initialize(self, load_gauss=False):
-        # Should probably be renamed allocate
-        gd = self.gd
-        self.rhos = [gd.empty()]
-        self.phis = [None]
-        self.residuals = [gd.empty()]
-        for level in range(self.levels):
-            gd2 = gd.coarsen()
-            self.phis.append(gd2.empty())
-            self.rhos.append(gd2.empty())
-            self.residuals.append(gd2.empty())
-            gd = gd2
-        assert len(self.phis) == len(self.rhos)
-        level += 1
-        assert level == self.levels
-
-        self.step = 0.66666666 / self.operators[0].get_diagonal_element()
-        self.presmooths[level] = 8
-        self.postsmooths[level] = 8
-
-        if load_gauss:
-            self.load_gauss()
-
-    def load_gauss(self, center=None):
-        if not hasattr(self, 'rho_gauss') or center is not None:
-            gauss = Gaussian(self.gd, center=center)
-            self.rho_gauss = gauss.get_gauss(0)
-            self.phi_gauss = gauss.get_gauss_pot(0)
-
     def solve(self, phi, rho, charge=None, eps=None, maxcharge=1e-6,
               zero_initial_phi=False):
+        self._init()
+        assert np.all(phi.shape == self.gd.n_c)
+        assert np.all(rho.shape == self.gd.n_c)
 
         if eps is None:
             eps = self.eps
@@ -241,15 +189,15 @@ class PoissonSolver:
                 rho_sign = rho * charge_sign
                 rho_sign[np.where(rho_sign < 0)] = 0
                 absolute_charge = self.gd.integrate(rho_sign)
-                center = - self.gd.calculate_dipole_moment(rho_sign) \
-                        / absolute_charge
+                center = - (self.gd.calculate_dipole_moment(rho_sign) /
+                            absolute_charge)
                 border_offset = np.inner(self.gd.h_cv, np.array((7, 7, 7)))
                 borders = np.inner(self.gd.h_cv, self.gd.N_c)
                 borders -= border_offset
                 if np.any(center > borders) or np.any(center < border_offset):
-                    raise RuntimeError(
-                            'Poisson solver: center of charge outside' + \
-                            ' borders - please increase box')
+                    raise RuntimeError('Poisson solver: '
+                                       'center of charge outside borders '
+                                       '- please increase box')
                     center[np.where(center > borders)] = borders
                 self.load_gauss(center=center)
             else:
@@ -278,7 +226,163 @@ class PoissonSolver:
                    ' boundary conditions')
             raise NotImplementedError(msg)
 
+    def load_gauss(self, center=None):
+        if not hasattr(self, 'rho_gauss') or center is not None:
+            gauss = Gaussian(self.gd, center=center)
+            self.rho_gauss = gauss.get_gauss(0)
+            self.phi_gauss = gauss.get_gauss_pot(0)
+
+
+class FDPoissonSolver(BasePoissonSolver):
+    def __init__(self, nn=3, relax='J', eps=2e-10, maxiter=1000,
+                 remove_moment=None, use_charge_center=False):
+        super(FDPoissonSolver, self).__init__(
+            eps=eps,
+            remove_moment=remove_moment,
+            use_charge_center=use_charge_center)
+        self.relax = relax
+        self.nn = nn
+        self.charged_periodic_correction = None
+        self.maxiter = maxiter
+
+        # Relaxation method
+        if relax == 'GS':
+            # Gauss-Seidel
+            self.relax_method = 1
+        elif relax == 'J':
+            # Jacobi
+            self.relax_method = 2
+        else:
+            raise NotImplementedError('Relaxation method %s' % relax)
+
+        self.description = None
+        self._initialized = False
+
+    def todict(self):
+        d = super(FDPoissonSolver, self).todict()
+        d.update({'name': 'fd', 'nn': self.nn, 'relax': self.relax})
+        return d
+
+    def get_stencil(self):
+        return self.nn
+
+    def create_laplace(self, gd, scale=1.0, n=1, dtype=float):
+        """Instantiate and return a Laplace operator
+
+        Allows subclasses to change the Laplace operator
+        """
+        return Laplace(gd, scale, n, dtype)
+
+    def set_grid_descriptor(self, gd):
+        # Should probably be renamed initialize
+        self.gd = gd
+        scale = -0.25 / pi
+
+        if self.nn == 'M':
+            if not gd.orthogonal:
+                raise RuntimeError('Cannot use Mehrstellen stencil with '
+                                   'non orthogonal cell.')
+
+            self.operators = [LaplaceA(gd, -scale)]
+            self.B = LaplaceB(gd)
+        else:
+            self.operators = [self.create_laplace(gd, scale, self.nn)]
+            self.B = None
+
+        self.interpolators = []
+        self.restrictors = []
+
+        level = 0
+        self.presmooths = [2]
+        self.postsmooths = [1]
+
+        # Weights for the relaxation,
+        # only used if 'J' (Jacobi) is chosen as method
+        self.weights = [2.0 / 3.0]
+
+        while level < 8:
+            try:
+                gd2 = gd.coarsen()
+            except ValueError:
+                break
+            self.operators.append(self.create_laplace(gd2, scale, 1))
+            self.interpolators.append(Transformer(gd2, gd))
+            self.restrictors.append(Transformer(gd, gd2))
+            self.presmooths.append(4)
+            self.postsmooths.append(4)
+            self.weights.append(1.0)
+            level += 1
+            gd = gd2
+
+        self.levels = level
+
+        if self.operators[-1].gd.N_c.max() > 36:
+            # Try to warn exactly once no matter how one uses the solver.
+            if gd.comm.parent is None:
+                warn = (gd.comm.rank == 0)
+            else:
+                warn = (gd.comm.parent.rank == 0)
+
+            if warn:
+                warntxt = '\n'.join([POISSON_GRID_WARNING, '',
+                                     self.get_description()])
+            else:
+                warntxt = ('Poisson warning from domain rank %d'
+                           % self.gd.comm.rank)
+
+            # Warn from all ranks to avoid deadlocks.
+            warnings.warn(warntxt, stacklevel=2)
+
+        self._initialized = False
+        # The Gaussians depend on the grid as well so we have to 'unload' them
+        if hasattr(self, 'rho_gauss'):
+            del self.rho_gauss
+            del self.phi_gauss
+
+
+    def get_description(self):
+        name = {1: 'Gauss-Seidel', 2: 'Jacobi'}[self.relax_method]
+        coarsest_grid = self.operators[-1].gd.N_c
+        coarsest_grid_string = ' x '.join([str(N) for N in coarsest_grid])
+        assert self.levels + 1 == len(self.operators)
+        lines = ['%s solver with %d multi-grid levels'
+                 % (name, self.levels + 1),
+                 '    Coarsest grid: %s points' % coarsest_grid_string]
+        if coarsest_grid.max() > 24:
+            # This friendly warning has lower threshold than the big long
+            # one that we print when things are really bad.
+            lines.extend(['    Warning: Coarse grid has more than 24 points.',
+                          '             More multi-grid levels recommended.'])
+        lines.extend(['    Stencil: %s' % self.operators[0].description,
+                      '    Max iterations: %d' % self.maxiter])
+        lines.append(super(FDPoissonSolver, self).get_description())
+        return '\n'.join(lines)
+
+    def _init(self):
+        if self._initialized:
+            return
+        # Should probably be renamed allocate
+        gd = self.gd
+        self.rhos = [gd.empty()]
+        self.phis = [None]
+        self.residuals = [gd.empty()]
+        for level in range(self.levels):
+            gd2 = gd.coarsen()
+            self.phis.append(gd2.empty())
+            self.rhos.append(gd2.empty())
+            self.residuals.append(gd2.empty())
+            gd = gd2
+        assert len(self.phis) == len(self.rhos)
+        level += 1
+        assert level == self.levels
+
+        self.step = 0.66666666 / self.operators[0].get_diagonal_element()
+        self.presmooths[level] = 8
+        self.postsmooths[level] = 8
+        self._initialized = True
+
     def solve_neutral(self, phi, rho, eps=2e-10):
+        self._init()
         self.phis[0] = phi
 
         if self.B is None:
@@ -303,33 +407,9 @@ class PoissonSolver:
 
         return niter
 
-    def iterate(self, step, level=0):
-        residual = self.residuals[level]
-        niter = 0
-        while True:
-            niter += 1
-            if level > 0 and niter == 1:
-                residual[:] = -self.rhos[level]
-            else:
-                self.operators[level].apply(self.phis[level], residual)
-                residual -= self.rhos[level]
-            error = self.gd.comm.sum(np.vdot(residual, residual))
-            if niter == 1 and level < self.levels:
-                self.restrictors[level].apply(residual, self.rhos[level + 1])
-                self.phis[level + 1][:] = 0.0
-                self.iterate(4.0 * step, level + 1)
-                self.interpolators[level].apply(self.phis[level + 1], residual)
-                self.phis[level] -= residual
-                continue
-            residual *= step
-            self.phis[level] -= residual
-            if niter == 2:
-                break
-
-        return error
-
     def iterate2(self, step, level=0):
         """Smooths the solution in every multigrid level"""
+        self._init()
 
         residual = self.residuals[level]
 
@@ -359,6 +439,10 @@ class PoissonSolver:
             residual -= self.rhos[level]
             error = self.gd.comm.sum(np.dot(residual.ravel(),
                                             residual.ravel())) * self.gd.dv
+
+            # How about this instead:
+            # error = self.gd.comm.max(abs(residual).max())
+
             return error
 
     def estimate_memory(self, mem):
@@ -374,12 +458,12 @@ class PoissonSolver:
         mem.subnode('rho, phi, residual [%d levels]' % self.levels, nbytes)
 
     def __repr__(self):
-        template = 'PoissonSolver(relax=\'%s\', nn=%s, eps=%e)'
+        template = 'FDPoissonSolver(relax=\'%s\', nn=%s, eps=%e)'
         representation = template % (self.relax, repr(self.nn), self.eps)
         return representation
 
 
-class NoInteractionPoissonSolver:
+class NoInteractionPoissonSolver(_PoissonSolver):
     relax_method = 0
     nn = 1
 
@@ -395,46 +479,82 @@ class NoInteractionPoissonSolver:
     def set_grid_descriptor(self, gd):
         pass
 
-    def initialize(self):
+    def todict(self):
+        return {'name': 'nointeraction'}
+
+    def estimate_memory(self, mem):
         pass
 
 
-class FFTPoissonSolver(PoissonSolver):
+class FFTPoissonSolver(BasePoissonSolver):
     """FFT Poisson solver for general unit cells."""
 
     relax_method = 0
     nn = 999
 
-    def __init__(self, eps=2e-10):
-        self.charged_periodic_correction = None
-        self.remove_moment = None
-        self.eps = eps
+    def __init__(self):
+        super(FFTPoissonSolver, self).__init__()
+        self._initialized = False
 
     def get_description(self):
-        return 'FFT'
+        return 'Parallel FFT'
+
+    def todict(self):
+        return {'name': 'fft'}
 
     def set_grid_descriptor(self, gd):
+        # We will probably want to use this on non-periodic grids too...
         assert gd.pbc_c.all()
         self.gd = gd
 
-    def initialize(self):
-        if self.gd.comm.rank == 0:
-            self.k2_Q, self.N3 = construct_reciprocal(self.gd)
+        self.grids = [gd]
+        for c in range(3):
+            N_c = gd.N_c.copy()
+            N_c[c] = 1  # Will be serial in that direction
+            parsize_c = decompose_domain(N_c, gd.comm.size)
+            self.grids.append(gd.new_descriptor(parsize_c=parsize_c))
+
+    def _init(self):
+        if self._initialized:
+            return
+
+        gd = self.grids[-1]
+        k2_Q, N3 = construct_reciprocal(gd, distributed=True)
+        self.poisson_factor_Q = 4.0 * np.pi / k2_Q
+        self._initialized = True
 
     def solve_neutral(self, phi_g, rho_g, eps=None):
-        if self.gd.comm.size == 1:
-            phi_g[:] = ifftn(fftn(rho_g) * 4.0 * pi / self.k2_Q).real
-        else:
-            rho_g = self.gd.collect(rho_g)
-            if self.gd.comm.rank == 0:
-                globalphi_g = ifftn(fftn(rho_g) * 4.0 * pi / self.k2_Q).real
-            else:
-                globalphi_g = None
-            self.gd.distribute(globalphi_g, phi_g)
+        self._init()
+        # Will be a bit more efficient if reduced dimension is always
+        # contiguous.  Probably more things can be improved...
+
+        gd1 = self.gd
+        work1_g = rho_g
+
+        for c in range(3):
+            gd2 = self.grids[c + 1]
+            work2_g = gd2.empty(dtype=work1_g.dtype)
+            grid2grid(gd1.comm, gd1, gd2, work1_g, work2_g)
+            work1_g = fftn(work2_g, axes=[c])
+            gd1 = gd2
+
+        work1_g *= self.poisson_factor_Q
+
+        for c in [2, 1, 0]:
+            gd2 = self.grids[c]
+            work2_g = ifftn(work1_g, axes=[c])
+            work1_g = gd2.empty(dtype=work2_g.dtype)
+            grid2grid(gd1.comm, gd1, gd2, work2_g, work1_g)
+            gd1 = gd2
+
+        phi_g[:] = work1_g.real
         return 1
 
+    def estimate_memory(self, mem):
+        mem.subnode('k squared', self.grids[-1].bytecount())
 
-class FixedBoundaryPoissonSolver(PoissonSolver):
+
+class FixedBoundaryPoissonSolver(FDPoissonSolver):
     """Solve the Poisson equation with FFT in two directions,
     and with central differential method in the third direction."""
 
@@ -505,8 +625,8 @@ class FixedBoundaryPoissonSolver(PoissonSolver):
         self.d1, self.d2, self.d3 = self.gd.N_c
         self.r_distribution = np.array_split(np.arange(self.d3),
                                              self.gd.comm.size)
-        self.comm_reshape = not (self.gd.parsize_c[0] == 1
-                                 and self.gd.parsize_c[1] == 1)
+        self.comm_reshape = not (self.gd.parsize_c[0] == 1 and
+                                 self.gd.parsize_c[1] == 1)
 
     def solve(self, phi_g, rho_g, charge=None):
         if charge is None:
