@@ -10,8 +10,9 @@ from ase.units import Hartree, Bohr
 
 import gpaw.mpi as mpi
 from gpaw.response.chi0 import Chi0
-from gpaw.response.kernel2 import calculate_Kxc, truncated_coulomb
+from gpaw.response.kernels import get_coulomb_kernel
 from gpaw.response.wstc import WignerSeitzTruncatedCoulomb
+from gpaw.response.fxc import get_xc_kernel
 
 
 class DielectricFunction:
@@ -21,10 +22,11 @@ class DielectricFunction:
                  nbands=None, eta=0.2, ftol=1e-6, threshold=1,
                  intraband=True, nblocks=1, world=mpi.world, txt=sys.stdout,
                  gate_voltage=None, truncation=None, disable_point_group=False,
-                 disable_time_reversal=False, use_more_memory=1,
-                 unsymmetrized=True):
+                 disable_time_reversal=False,
+                 integrationmode=None, pbc=None, rate=0.0,
+                 omegacutlower=None, omegacutupper=None, eshift=0.0):
         """Creates a DielectricFunction object.
-        
+
         calc: str
             The groundstate calculation file that the linear response
             calculation is based on.
@@ -72,8 +74,11 @@ class DielectricFunction:
             Shift Fermi level of ground state calculation by the
             specified amount.
         truncation: str
-            'wigner-seitz' for Wigner Seitz truncated Coulomb
-            '2D' for regular truncation in the z-direction
+            'wigner-seitz' for Wigner Seitz truncated Coulomb.
+            '2D, 1D or 0d for standard analytical truncation schemes.
+            Non-periodic directions are determined from k-point grid
+        eshift: float
+            Shift unoccupied bands
         """
 
         self.chi0 = Chi0(calc, frequencies, domega0=domega0,
@@ -84,14 +89,19 @@ class DielectricFunction:
                          txt=txt, gate_voltage=gate_voltage,
                          disable_point_group=disable_point_group,
                          disable_time_reversal=disable_time_reversal,
-                         use_more_memory=use_more_memory,
-                         unsymmetrized=unsymmetrized)
-        
+                         integrationmode=integrationmode,
+                         pbc=pbc, rate=rate, eshift=eshift)
+
         self.name = name
 
         self.omega_w = self.chi0.omega_w
+        if omegacutlower is not None:
+            inds_w = np.logical_and(self.omega_w > omegacutlower / Hartree,
+                                    self.omega_w < omegacutupper / Hartree)
+            self.omega_w = self.omega_w[inds_w]
+
         nw = len(self.omega_w)
-        
+
         world = self.chi0.world
         self.mynw = (nw + world.size - 1) // world.size
         self.w1 = min(self.mynw * world.rank, nw)
@@ -117,7 +127,7 @@ class DielectricFunction:
         self.chi0.timer.write(self.chi0.fd)
         if self.name:
             self.write(name, pd, chi0_wGG, chi0_wxvG, chi0_wvv)
-        
+
         return pd, chi0_wGG, chi0_wxvG, chi0_wvv
 
     def write(self, name, pd, chi0_wGG, chi0_wxvG, chi0_wvv):
@@ -131,7 +141,7 @@ class DielectricFunction:
                         fd, pickle.HIGHEST_PROTOCOL)
             for chi0_GG in chi0_wGG:
                 pickle.dump(chi0_GG, fd, pickle.HIGHEST_PROTOCOL)
-            
+
             tmp_wGG = np.empty((self.mynw, nG, nG), complex)
             w1 = self.mynw
             for rank in range(1, world.size):
@@ -148,19 +158,23 @@ class DielectricFunction:
         print('Reading from', name, file=self.chi0.fd)
         fd = open(name, 'rb')
         omega_w, pd, chi0_wGG, chi0_wxvG, chi0_wvv = pickle.load(fd)
-        assert np.allclose(omega_w, self.omega_w)
+        for omega in self.omega_w:
+            assert np.any(np.abs(omega - omega_w) < 1e-8)
 
+        wmin = np.argmin(np.abs(np.min(self.omega_w) - omega_w))
         world = self.chi0.world
-        
+
         nw = len(omega_w)
         nG = pd.ngmax
-        
+
         if chi0_wGG is not None:
             # Old file format:
-            chi0_wGG = chi0_wGG[self.w1:self.w2].copy()
+            chi0_wGG = chi0_wGG[wmin + self.w1:self.w2].copy()
         else:
             if world.rank == 0:
                 chi0_wGG = np.empty((self.mynw, nG, nG), complex)
+                for _ in range(wmin):
+                    pickle.load(fd)
                 for chi0_GG in chi0_wGG:
                     chi0_GG[:] = pickle.load(fd)
                 tmp_wGG = np.empty((self.mynw, nG, nG), complex)
@@ -174,8 +188,13 @@ class DielectricFunction:
             else:
                 chi0_wGG = np.empty((self.w2 - self.w1, nG, nG), complex)
                 world.receive(chi0_wGG, 0)
+
+        if chi0_wvv is not None:
+            chi0_wxvG = chi0_wxvG[wmin:wmin + nw]
+            chi0_wvv = chi0_wvv[wmin:wmin + nw]
+
         return pd, chi0_wGG, chi0_wxvG, chi0_wvv
-        
+
     def collect(self, a_w):
         world = self.chi0.world
         b_w = np.zeros(self.mynw, a_w.dtype)
@@ -189,14 +208,36 @@ class DielectricFunction:
         """Return frequencies that Chi is evaluated on"""
         return self.omega_w * Hartree
 
-    def get_chi(self, xc='RPA', q_c=[0, 0, 0], direction='x'):
-        """ Returns v^1/2 chi V^1/2"""
+    def get_chi(self, xc='RPA', q_c=[0, 0, 0], direction='x',
+                return_VchiV=True, q_v=None):
+        """ Returns v^1/2 chi v^1/2. The truncated Coulomb interaction is
+        then included as v^-1/2 v_t v^-1/2. This is in order to conform with
+        the head and wings of chi0, which is treated specially for q=0."""
+
         pd, chi0_wGG, chi0_wxvG, chi0_wvv = self.calculate_chi0(q_c)
-        G_G = pd.G2_qG[0]**0.5
-        nG = len(G_G)
+        N_c = self.chi0.calc.wfs.kd.N_c
+        Kbare_G = get_coulomb_kernel(pd,
+                                     N_c,
+                                     truncation=None,
+                                     q_v=q_v)
+        vsqr_G = Kbare_G**0.5
+        nG = len(vsqr_G)
+
+        if self.truncation is not None:
+            if self.truncation == 'wigner-seitz':
+                self.wstc = WignerSeitzTruncatedCoulomb(pd.gd.cell_cv, N_c)
+            else:
+                self.wstc = None
+            Ktrunc_G = get_coulomb_kernel(pd,
+                                          N_c,
+                                          truncation=self.truncation,
+                                          wstc=self.wstc,
+                                          q_v=q_v)
+            K_GG = np.diag(Ktrunc_G / Kbare_G)
+        else:
+            K_GG = np.eye(nG, dtype=complex)
 
         if pd.kd.gamma:
-            G_G[0] = 1.0
             if isinstance(direction, str):
                 d_v = {'x': [1, 0, 0],
                        'y': [0, 1, 0],
@@ -208,74 +249,77 @@ class DielectricFunction:
             chi0_wGG[:, 0] = np.dot(d_v, chi0_wxvG[W, 0])
             chi0_wGG[:, :, 0] = np.dot(d_v, chi0_wxvG[W, 1])
             chi0_wGG[:, 0, 0] = np.dot(d_v, np.dot(chi0_wvv[W], d_v).T)
-        
-        G_G /= (4 * pi)**0.5
-
-        if self.truncation == 'wigner-seitz':
-            kernel = WignerSeitzTruncatedCoulomb(pd.gd.cell_cv,
-                                                 self.chi0.calc.wfs.kd.N_c)
-            K_G = kernel.get_potential(pd)
-            K_G *= G_G**2
-            if pd.kd.gamma:
-                K_G[0] = 0.0
-        elif self.truncation == '2D':
-            K_G = truncated_coulomb(pd)
-            K_G *= G_G**2
-            if pd.kd.gamma:
-                K_G[0] = 0.0
-        else:
-            K_G = np.ones(nG)
-
-        K_GG = np.zeros((nG, nG), dtype=complex)
-        for i in range(nG):
-            K_GG[i, i] = K_G[i]
 
         if xc != 'RPA':
-            R_av = self.chi0.calc.atoms.positions / Bohr
-            nt_sG = self.chi0.calc.density.nt_sG
-            Kxc_sGG = calculate_Kxc(pd, nt_sG, R_av, self.chi0.calc.wfs.setups,
-                                    self.chi0.calc.density.D_asp,
-                                    functional=xc)
-            K_GG += Kxc_sGG[0] * G_G * G_G[:, np.newaxis]
+            Kxc_sGG = get_xc_kernel(pd,
+                                    self.chi0,
+                                    functional=xc,
+                                    chi0_wGG=chi0_wGG)
+            K_GG += Kxc_sGG[0] / vsqr_G / vsqr_G[:, np.newaxis]
+
         chi_wGG = []
         for chi0_GG in chi0_wGG:
-            chi0_GG[:] = chi0_GG / G_G / G_G[:, np.newaxis]
-            chi_wGG.append(np.dot(np.linalg.inv(np.eye(nG) -
-                                                np.dot(chi0_GG, K_GG)),
-                                  chi0_GG))
-        return chi0_wGG, np.array(chi_wGG)
-  
+            """v^1/2 chi0 V^1/2"""
+            chi0_GG[:] = chi0_GG * vsqr_G * vsqr_G[:, np.newaxis]
+            chi_GG = np.dot(np.linalg.inv(np.eye(nG) -
+                                          np.dot(chi0_GG, K_GG)),
+                            chi0_GG)
+            if not return_VchiV:
+                chi0_GG /= vsqr_G * vsqr_G[:, np.newaxis]
+                chi_GG /= vsqr_G * vsqr_G[:, np.newaxis]
+            chi_wGG.append(chi_GG)
+
+        if len(chi_wGG):
+            chi_wGG = np.array(chi_wGG)
+        else:
+            chi_wGG = np.zeros((0, nG, nG), complex)
+
+        return pd, chi0_wGG, chi_wGG
+
     def get_dielectric_matrix(self, xc='RPA', q_c=[0, 0, 0],
                               direction='x', symmetric=True,
-                              calculate_chi=False, q0=None,
+                              calculate_chi=False, q_v=None,
                               add_intraband=True):
         """Returns the symmetrized dielectric matrix.
-        
+
         ::
-        
+
             \tilde\epsilon_GG' = v^{-1/2}_G \epsilon_GG' v^{1/2}_G',
-            
+
         where::
-            
+
             epsilon_GG' = 1 - v_G * P_GG' and P_GG'
-            
+
         is the polarization.
-        
+
         ::
-            
+
             In RPA:   P = chi^0
             In TDDFT: P = (1 - chi^0 * f_xc)^{-1} chi^0
-        
+
+        in addition to RPA one can use the kernels, ALDA, rALDA, rAPBE,
+        Bootstrap and LRalpha (long-range kerne), where alpha is a user
+        specified parameter (for example xc='LR0.25')
+
         The head of the inverse symmetrized dielectric matrix is equal
         to the head of the inverse dielectric matrix (inverse dielectric
-        function)
-        """
+        function)"""
+
         pd, chi0_wGG, chi0_wxvG, chi0_wvv = self.calculate_chi0(q_c)
-        G_G = pd.G2_qG[0]**0.5
-        nG = len(G_G)
+
+        N_c = self.chi0.calc.wfs.kd.N_c
+        if self.truncation == 'wigner-seitz':
+            self.wstc = WignerSeitzTruncatedCoulomb(pd.gd.cell_cv, N_c)
+        else:
+            self.wstc = None
+        K_G = get_coulomb_kernel(pd,
+                                 N_c,
+                                 truncation=self.truncation,
+                                 wstc=self.wstc,
+                                 q_v=q_v)**0.5
+        nG = len(K_G)
 
         if pd.kd.gamma:
-            G_G[0] = 1.0
             if isinstance(direction, str):
                 d_v = {'x': [1, 0, 0],
                        'y': [0, 1, 0],
@@ -289,33 +333,17 @@ class DielectricFunction:
                 chi0_wGG[:, 0] = np.dot(d_v, chi0_wxvG[W, 0])
                 chi0_wGG[:, :, 0] = np.dot(d_v, chi0_wxvG[W, 1])
                 chi0_wGG[:, 0, 0] = np.dot(d_v, np.dot(chi0_wvv[W], d_v).T)
-            if q0 is not None:
+            if q_v is not None:
                 print('Restoring q dependence of head and wings of chi0')
-                chi0_wGG[:, 1:, 0] *= q0
-                chi0_wGG[:, 0, 1:] *= q0
-                chi0_wGG[:, 0, 0] *= q0**2
-                G_G[0] = q0
-                    
-        if self.truncation == 'wigner-seitz':
-            kernel = WignerSeitzTruncatedCoulomb(pd.gd.cell_cv,
-                                                 self.chi0.calc.wfs.kd.N_c)
-            K_G = kernel.get_potential(pd)**0.5
-            if pd.kd.gamma:
-                K_G[0] = 0.0
-        elif self.truncation == '2D':
-            K_G = truncated_coulomb(pd, q0=q0)
-            if pd.kd.gamma and q0 is None:
-                K_G[0] = 0.0
-        else:
-            K_G = (4 * pi)**0.5 / G_G
+                chi0_wGG[:, 1:, 0] *= np.dot(q_v, d_v)
+                chi0_wGG[:, 0, 1:] *= np.dot(q_v, d_v)
+                chi0_wGG[:, 0, 0] *= np.dot(q_v, d_v)**2
 
         if xc != 'RPA':
-            R_av = self.chi0.calc.atoms.positions / Bohr
-            nt_sG = self.chi0.calc.density.nt_sG
-            Kxc_sGG = calculate_Kxc(pd, nt_sG, R_av,
-                                    self.chi0.calc.wfs.setups,
-                                    self.chi0.calc.density.D_asp,
-                                    functional=xc)
+            Kxc_sGG = get_xc_kernel(pd,
+                                    self.chi0,
+                                    functional=xc,
+                                    chi0_wGG=chi0_wGG)
 
         if calculate_chi:
             chi_wGG = []
@@ -342,30 +370,36 @@ class DielectricFunction:
             chi0_GG[:] = e_GG
 
         # chi0_wGG is now the dielectric matrix
+        if calculate_chi:
+            if len(chi_wGG):
+                chi_wGG = np.array(chi_wGG)
+            else:
+                chi_wGG = np.zeros((0, nG, nG), complex)
+
         if not calculate_chi:
             return chi0_wGG
         else:
             # chi_wGG is the full density response function..
-            return pd, chi0_wGG, np.array(chi_wGG)
+            return pd, chi0_wGG, chi_wGG
 
-    def get_dielectric_function(self, xc='RPA', q_c=[0, 0, 0],
+    def get_dielectric_function(self, xc='RPA', q_c=[0, 0, 0], q_v=None,
                                 direction='x', filename='df.csv'):
         """Calculate the dielectric function.
 
         Returns dielectric function without and with local field correction:
         df_NLFC_w, df_LFC_w = DielectricFunction.get_dielectric_function()
         """
-        e_wGG = self.get_dielectric_matrix(xc, q_c, direction)
+        e_wGG = self.get_dielectric_matrix(xc, q_c, direction, q_v=q_v)
         df_NLFC_w = np.zeros(len(e_wGG), dtype=complex)
         df_LFC_w = np.zeros(len(e_wGG), dtype=complex)
 
         for w, e_GG in enumerate(e_wGG):
             df_NLFC_w[w] = e_GG[0, 0]
             df_LFC_w[w] = 1 / np.linalg.inv(e_GG)[0, 0]
-        
+
         df_NLFC_w = self.collect(df_NLFC_w)
         df_LFC_w = self.collect(df_LFC_w)
-        
+
         if filename is not None and mpi.rank == 0:
             with open(filename, 'w') as fd:
                 for omega, nlfc, lfc in zip(self.omega_w * Hartree,
@@ -374,17 +408,17 @@ class DielectricFunction:
                     print('%.6f, %.6f, %.6f, %.6f, %.6f' %
                           (omega, nlfc.real, nlfc.imag, lfc.real, lfc.imag),
                           file=fd)
-                
+
         return df_NLFC_w, df_LFC_w
 
-    def get_macroscopic_dielectric_constant(self, xc='RPA', direction='x'):
+    def get_macroscopic_dielectric_constant(self, xc='RPA', direction='x', q_v=None):
         """Calculate macroscopic dielectric constant.
-        
+
         Returns eM_NLFC and eM_LFC.
 
         Macroscopic dielectric constant is defined as the real part
         of dielectric function at w=0.
-        
+
         Parameters:
 
         eM_LFC: float
@@ -396,40 +430,43 @@ class DielectricFunction:
         fd = self.chi0.fd
         print('', file=fd)
         print('%s Macroscopic Dielectric Constant:' % xc, file=fd)
-       
+
         df_NLFC_w, df_LFC_w = self.get_dielectric_function(
             xc=xc,
             filename=None,
-            direction=direction)
+            direction=direction,
+            q_v=q_v)
         eps0 = np.real(df_NLFC_w[0])
         eps = np.real(df_LFC_w[0])
         print('  %s direction' % direction, file=fd)
         print('    Without local field: %f' % eps0, file=fd)
         print('    Include local field: %f' % eps, file=fd)
-            
+
         return eps0, eps
 
     def get_eels_spectrum(self, xc='RPA', q_c=[0, 0, 0],
                           direction='x', filename='eels.csv'):
         """Calculate EELS spectrum. By default, generate a file 'eels.csv'.
 
-        EELS spectrum is obtained from the imaginary part of the inverse
-        of dielectric function. Returns EELS spectrum without and with
-        local field corrections:
+        EELS spectrum is obtained from the imaginary part of the
+        density response function as, EELS(\omega) = - 4 * \pi / q^2 Im \chi.
+        Returns EELS spectrum without and with local field corrections:
 
         df_NLFC_w, df_LFC_w = DielectricFunction.get_eels_spectrum()
         """
 
-        # Calculate dielectric function
-        df_NLFC_w, df_LFC_w = self.get_dielectric_function(
-            xc=xc, q_c=q_c,
-            direction=direction,
-            filename=None)
-        Nw = df_NLFC_w.shape[0]
-        
-        # Calculate eels
-        eels_NLFC_w = -(1 / df_NLFC_w).imag
-        eels_LFC_w = -(1 / df_LFC_w).imag
+        # Calculate V^1/2 \chi V^1/2
+        pd, Vchi0_wGG, Vchi_wGG = self.get_chi(xc=xc, q_c=q_c,
+                                               direction=direction)
+        Nw = self.omega_w.shape[0]
+
+        # Calculate eels = -Im 4 \pi / q^2  \chi
+        eels_NLFC_w = -(1. / (1. - Vchi0_wGG[:, 0, 0])).imag
+        eels_LFC_w = - (Vchi_wGG[:, 0, 0]).imag
+
+        # Collect frequencies
+        eels_NLFC_w = self.collect(eels_NLFC_w)
+        eels_LFC_w = self.collect(eels_LFC_w)
 
         # Write to file
         if filename is not None and mpi.rank == 0:
@@ -442,8 +479,8 @@ class DielectricFunction:
             fd.close()
 
         return eels_NLFC_w, eels_LFC_w
-        
-    def get_polarizability(self, xc='RPA', direction='x',
+
+    def get_polarizability(self, xc='RPA', direction='x', q_c=[0, 0, 0],
                            filename='polarizability.csv', pbc=None):
         """Calculate the polarizability alpha.
         In 3D the imaginary part of the polarizability is related to the
@@ -470,23 +507,37 @@ class DielectricFunction:
             V = np.abs(np.linalg.det(cell_cv[~pbc_c][:, ~pbc_c]))
 
         if not self.truncation:
-            # Without truncation alpha is simply related to eps_M
-            df0_w, df_w = self.get_dielectric_function(xc=xc, q_c=[0, 0, 0],
+            """Standard expression for the polarizability"""
+            df0_w, df_w = self.get_dielectric_function(xc=xc,
+                                                       q_c=q_c,
                                                        filename=None,
                                                        direction=direction)
             alpha_w = V * (df_w - 1.0) / (4 * pi)
             alpha0_w = V * (df0_w - 1.0) / (4 * pi)
         else:
-            # With truncation we need to calculate \chit = v^0.5*chi*v^0.5
-            print('Using truncated Coulomb interaction',
-                  file=self.chi0.fd)
-            chi0_wGG, chi_wGG = self.get_chi(xc=xc, direction=direction)
+            """Since eps_M = 1.0 for a truncated Coulomb interaction, it does
+            not make sense to apply it here. Instead one should define the
+            polarizability by
+
+                alpha * eps_M^{-1} = -L / (4 * pi) * <v_ind>
+
+            where <v_ind> = 4 * pi * \chi / q^2 is the averaged induced
+            potential (relative to the strength of the  external potential).
+            With the bare Coulomb potential, this expression is equivalent to
+            the standard one. In a 2D system \chi should be calculated with a
+            truncated Coulomb potential and eps_M = 1.0"""
+
+            print('Using truncated Coulomb interaction', file=self.chi0.fd)
+
+            pd, chi0_wGG, chi_wGG = self.get_chi(xc=xc,
+                                                 q_c=q_c,
+                                                 direction=direction)
             alpha_w = -V * (chi_wGG[:, 0, 0]) / (4 * pi)
             alpha0_w = -V * (chi0_wGG[:, 0, 0]) / (4 * pi)
 
             alpha_w = self.collect(alpha_w)
             alpha0_w = self.collect(alpha0_w)
-        
+
         Nw = len(alpha_w)
         if filename is not None and mpi.rank == 0:
             fd = open(filename, 'w')
@@ -503,17 +554,17 @@ class DielectricFunction:
 
     def check_sum_rule(self, spectrum=None):
         """Check f-sum rule.
-        
+
         It takes the y of a spectrum as an entry and it check its integral.
-        
+
         spectrum: np.ndarray
             Input spectrum
         """
-        
+
         assert (self.omega_w[1:] - self.omega_w[:-1]).ptp() < 1e-10
-                
+
         fd = self.chi0.fd
-        
+
         if spectrum is None:
             raise ValueError('No spectrum input ')
         dw = self.chi0.omega_w[1] - self.chi0.omega_w[0]
@@ -531,7 +582,7 @@ class DielectricFunction:
     def get_eigenmodes(self, q_c=[0, 0, 0], w_max=None, name=None,
                        eigenvalue_only=False, direction='x',
                        checkphase=True):
-        
+
         """Plasmon eigenmodes as eigenvectors of the dielectric matrix."""
 
         assert self.chi0.world.size == 1
@@ -540,9 +591,9 @@ class DielectricFunction:
         e_wGG = self.get_dielectric_matrix(xc='RPA', q_c=q_c,
                                            direction=direction,
                                            symmetric=False)
-        
+
         kd = pd.kd
-        
+
         # Get real space grid for plasmon modes:
         r = pd.gd.get_grid_point_coordinates()
         w_w = self.omega_w * Hartree
@@ -550,10 +601,10 @@ class DielectricFunction:
             w_w = w_w[np.where(w_w < w_max)]
         Nw = len(w_w)
         nG = e_wGG.shape[1]
-             
+
         eig = np.zeros([Nw, nG], dtype=complex)
         eig_all = np.zeros([Nw, nG], dtype=complex)
-      
+
         # Find eigenvalues and eigenvectors:
         e_GG = e_wGG[0]
         eig_all[0], vec = np.linalg.eig(e_GG)
@@ -565,7 +616,7 @@ class DielectricFunction:
                          dtype=complex)
         n_ind = np.zeros([0, r.shape[1], r.shape[2], r.shape[3]],
                          dtype=complex)
-         
+
         # Loop to find the eigenvalues that crosses zero
         # from negative to positive values:
         for i in np.array(range(1, Nw)):
@@ -603,7 +654,7 @@ class DielectricFunction:
                 print('crossing found at w = %1.2f eV' % w0)
                 omega0 = np.append(omega0, w0)
                 eigen0 = np.append(eigen0, eig0)
-                
+
                 # Fourier Transform:
                 qG = pd.get_reciprocal_vectors(add_q=True)
                 coef_G = np.diagonal(np.inner(qG, qG)) / (4 * pi)
@@ -622,7 +673,7 @@ class DielectricFunction:
                     n_temp *= np.exp(1j * pi * phase)
                 v_ind = np.append(v_ind, v_temp[np.newaxis, :], axis=0)
                 n_ind = np.append(n_ind, n_temp[np.newaxis, :], axis=0)
-        
+
         kd = self.chi0.calc.wfs.kd
         if name is None and self.name:
             name = (self.name + '%+d%+d%+d-eigenmodes.pckl' %
@@ -645,19 +696,19 @@ class DielectricFunction:
                          v_ind, n_ind), open(name, 'wb'),
                         pickle.HIGHEST_PROTOCOL)
             return r * Bohr, w_w, eig, omega0, eigen0, v_ind, n_ind
-    
+
     def get_spatial_eels(self, q_c=[0, 0, 0], direction='x',
-                         w_max=None, filename='eels'):
+                         w_max=None, filename='eels', r=None, perpdir=None):
         """Spatially resolved loss spectrum.
-        
+
         The spatially resolved loss spectrum is calculated as the inverse
         fourier transform of ``VChiV = (eps^{-1}-I)V``::
-            
+
             EELS(w,r) = - Im [sum_{G,G'} e^{iGr} Vchi_{GG'}(w) V_G'e^{-iG'r}]
                           \delta(w-G\dot v_e )
-        
+
         Input parameters:
-            
+
         direction: 'x', 'y', or 'z'
             The direction for scanning acroos the structure
             (perpendicular to the electron beam) .
@@ -665,55 +716,72 @@ class DielectricFunction:
             maximum frequency
         filename: str
             name of output
-            
+
         Returns: real space grid, frequency points, EELS(w,r)
         """
 
         assert self.chi0.world.size == 1
+
         pd, chi0_wGG, chi0_wxvG, chi0_wvv = self.calculate_chi0(q_c)
         e_wGG = self.get_dielectric_matrix(xc='RPA', q_c=q_c,
                                            symmetric=False)
-        r = pd.gd.get_grid_point_coordinates()
-        ix = r.shape[1] / 2
-        iy = r.shape[2] / 2
-        iz = r.shape[3] / 2
-        if direction == 'x':
-            r = r[:, :, iy, iz]
-            perpdir = [1, 2]
-        if direction == 'y':
-            r = r[:, ix, :, iz]
-            perpdir = [0, 2]
-        if direction == 'z':
-            r = r[:, ix, iy, :]
-            perpdir = [0, 1]
+
+        if r is None:
+            r = pd.gd.get_grid_point_coordinates()
+            ix = r.shape[1] // 2 * 0
+            iy = r.shape[2] // 2 * 0
+            iz = r.shape[3] // 2
+            if direction == 'x':
+                r = r[:, :, iy, iz]
+                perpdir = [1, 2]
+            if direction == 'y':
+                r = r[:, ix, :, iz]
+                perpdir = [0, 2]
+            if direction == 'z':
+                r = r[:, ix, iy, :]
+                perpdir = [0, 1]
 
         nG = e_wGG.shape[1]
         Gvec = pd.G_Qv[pd.Q_qG[0]]
         Glist = []
+
         # Only use G-vectors that are zero along electron beam
         # due to \delta(w-G\dot v_e )
+        q_v = pd.K_qv[0]
         for iG in range(nG):
-            if Gvec[iG, perpdir[0]] == 0 and Gvec[iG, perpdir[1]] == 0:
+            if perpdir is not None:
+                if Gvec[iG, perpdir[0]] == 0 and Gvec[iG, perpdir[1]] == 0:
+                    Glist.append(iG)
+            elif not np.abs(np.dot(q_v, Gvec[iG])) < \
+                 np.linalg.norm(q_v) * np.linalg.norm(Gvec[iG]):
                 Glist.append(iG)
         qG = Gvec[Glist] + pd.K_qv
+
         w_w = self.omega_w * Hartree
         if w_max:
             w_w = w_w[np.where(w_w < w_max)]
         Nw = len(w_w)
         qGr = np.inner(qG, r.T).T
         phase = np.exp(1j * qGr)
-        
         V_G = (4 * pi) / np.diagonal(np.inner(qG, qG))
         phase2 = np.exp(-1j * qGr) * V_G
         E_wrr = np.zeros([Nw, r.shape[1], r.shape[1]])
         E_wr = np.zeros([Nw, r.shape[1]])
+        Eavg_w = np.zeros([Nw], complex)
+        Ec_wr = np.zeros([Nw, r.shape[1]], complex)
         for i in range(Nw):
-            Vchi_GG = (np.linalg.inv(e_wGG[i, Glist, :][:, Glist]) -
-                       np.eye(len(Glist)))
+            Vchi_GG = (np.linalg.inv(e_wGG[i]) -
+                       np.eye(nG))[Glist, :][:, Glist]
+
+            qG_G = np.sum(qG**2, axis=1)**0.5
+            Eavg_w[i] = np.trace(Vchi_GG * np.diag(V_G * qG_G))
+
             # Fourier transform:
             E_wrr[i] = -np.imag(np.dot(np.dot(phase, Vchi_GG), phase2.T))
             E_wr[i] = np.diagonal(E_wrr[i])
+            Ec_wr[i] = np.diagonal(np.dot(np.dot(phase, Vchi_GG *
+                                                 np.diag(qG_G)), phase2.T))
         pickle.dump((r * Bohr, w_w, E_wr), open('%s.pickle' % filename, 'wb'),
                     pickle.HIGHEST_PROTOCOL)
-                    
-        return r * Bohr, w_w, E_wr
+
+        return r * Bohr, w_w, E_wr, Ec_wr, Eavg_w
