@@ -1,25 +1,28 @@
-from __future__ import print_function
+from __future__ import print_function, division
 
-import sys
 import functools
-
+import numbers
+import sys
 from math import pi
 
 import numpy as np
+from scipy.spatial import Delaunay, cKDTree
+
 from ase.units import Hartree
-from ase.utils import devnull
+from ase.utils import convert_string_to_fd
 from ase.utils.timing import timer, Timer
 
 import gpaw.mpi as mpi
 from gpaw import GPAW
 from gpaw.fd_operators import Gradient
+from gpaw.kpt_descriptor import KPointDescriptor
 from gpaw.occupations import FermiDirac
 from gpaw.response.math_func import (two_phi_planewave_integrals,
                                      two_phi_nabla_planewave_integrals)
 from gpaw.utilities.blas import gemm
 from gpaw.utilities.progressbar import ProgressBar
 from gpaw.wavefunctions.pw import PWLFC
-import gpaw.io.tar as io
+from gpaw.bztools import get_reduced_bz, unique_rows
 
 
 class KPoint:
@@ -53,7 +56,7 @@ class KPointPair:
     def get_k1(self):
         """ Return KPoint object 1."""
         return self.kpt1
-    
+
     def get_k2(self):
         """ Return KPoint object 2."""
         return self.kpt2
@@ -64,18 +67,22 @@ class KPointPair:
 
     def get_transition_energies(self, n_n, m_m):
         """Return the energy difference for specified bands."""
+        n_n = np.array(n_n)
+        m_m = np.array(m_m)
         kpt1 = self.kpt1
         kpt2 = self.kpt2
-        deps_nm = (kpt1.eps_n[n_n][:, np.newaxis] -
-                   kpt2.eps_n[m_m])
+        deps_nm = (kpt1.eps_n[n_n - self.kpt1.n1][:, np.newaxis] -
+                   kpt2.eps_n[m_m - self.kpt2.n1])
         return deps_nm
 
     def get_occupation_differences(self, n_n, m_m):
         """Get difference in occupation factor between specified bands."""
+        n_n = np.array(n_n)
+        m_m = np.array(m_m)
         kpt1 = self.kpt1
         kpt2 = self.kpt2
-        df_nm = (kpt1.f_n[n_n][:, np.newaxis] -
-                 kpt2.f_n[m_m])
+        df_nm = (kpt1.f_n[n_n - self.kpt1.n1][:, np.newaxis] -
+                 kpt2.f_n[m_m - self.kpt2.n1])
         return df_nm
 
 
@@ -87,7 +94,7 @@ class PWSymmetryAnalyzer:
                  disable_time_reversal=False,
                  timer=None):
         """Creates a PWSymmetryAnalyzer object.
-        
+
         Determines which of the symmetries of the atomic structure
         that is compatible with the reciprocal lattice. Contains the
         necessary functions for mapping quantities between kpoints,
@@ -116,7 +123,7 @@ class PWSymmetryAnalyzer:
         assert disable_non_symmorphic, \
             print('You are not allowed to use non symmorphic syms, sorry. ',
                   file=self.fd)
-        
+
         # Settings
         self.disable_point_group = disable_point_group
         self.disable_time_reversal = disable_time_reversal
@@ -131,7 +138,7 @@ class PWSymmetryAnalyzer:
         self.disable_symmetries = (self.disable_point_group and
                                    self.disable_time_reversal and
                                    self.disable_non_symmorphic)
-        
+
         # Number of symmetries
         U_scc = kd.symmetry.op_scc
         self.nU = len(U_scc)
@@ -142,6 +149,8 @@ class PWSymmetryAnalyzer:
         # Which timer to use
         self.timer = timer or Timer()
 
+        self.KDTree = cKDTree(np.mod(np.mod(kd.bzk_kc, 1).round(6), 1))
+
         # Initialize
         self.initialize()
 
@@ -176,6 +185,9 @@ class PWSymmetryAnalyzer:
         print(self.infostring, file=self.fd)
         self.print_symmetries()
 
+    def find_kpoint(self, k_c):
+        return self.KDTree.query(np.mod(np.mod(k_c, 1).round(6), 1))[1]
+
     def print_symmetries(self):
         """Handsome print function for symmetry operations."""
 
@@ -204,7 +216,6 @@ class PWSymmetryAnalyzer:
         K_gK = self.group_kpoints()
         ng = len(K_gK)
         self.infostring += '{0} groups of equivalent kpoints. '.format(ng)
-        
         percent = (1. - (ng + 0.) / self.kd.nbzkpts) * 100
         self.infostring += '{0}% reduction. '.format(percent)
 
@@ -236,18 +247,18 @@ class PWSymmetryAnalyzer:
         conserveq_s = np.zeros(nsym, bool)
 
         newq_sc = np.dot(U_scc, q_c)
-        
+
         # Direct symmetries
         dshift_sc = (newq_sc - q_c[np.newaxis]).round().astype(int)
         inds_s = np.argwhere((newq_sc == q_c[np.newaxis] + dshift_sc).all(1))
         conserveq_s[inds_s] = True
-        
+
         shift_sc[:nU] = dshift_sc
 
         # Time reversal
         trshift_sc = (-newq_sc - q_c[np.newaxis]).round().astype(int)
-        trinds_s = np.argwhere((-newq_sc == q_c[np.newaxis]
-                                + trshift_sc).all(1)) + nU
+        trinds_s = np.argwhere((-newq_sc == q_c[np.newaxis] +
+                                trshift_sc).all(1)) + nU
         conserveq_s[trinds_s] = True
         shift_sc[nU:nsym] = trshift_sc
 
@@ -256,24 +267,24 @@ class PWSymmetryAnalyzer:
 
         # Filter out disabled symmetries
         if self.disable_point_group:
-            s_s = [s for s in s_s if self.is_not_point_group(s)]
+            s_s = list(filter(self.is_not_point_group, s_s))
 
         if self.disable_time_reversal:
-            s_s = [s for s in s_s if self.is_not_time_reversal(s)]
+            s_s = list(filter(self.is_not_time_reversal, s_s))
 
         if self.disable_non_symmorphic:
-            s_s = [s for s in s_s if self.is_not_non_symmorphic(s)]
+            s_s = list(filter(self.is_not_non_symmorphic, s_s))
 
-        stmp_s = []
-        for s in s_s:
-            if self.kd.bz2bz_ks[0, s] == -1:
-                assert (self.kd.bz2bz_ks[:, s] == -1).all()
-            else:
-                stmp_s.append(s)
-        
-        s_s = stmp_s
+#        stmp_s = []
+#        for s in s_s:
+#            if self.kd.bz2bz_ks[0, s] == -1:
+#                assert (self.kd.bz2bz_ks[:, s] == -1).all()
+#            else:
+#                stmp_s.append(s)
 
-        self.infostring += 'Found {0} allowed symmetries. '.format(len(s_s))
+#        s_s = stmp_s
+
+        self.infostring += 'Found {} allowed symmetries. '.format(len(s_s))
         self.s_s = s_s
         self.shift_sc = shift_sc
 
@@ -281,7 +292,7 @@ class PWSymmetryAnalyzer:
         U_scc = self.kd.symmetry.op_scc
         nU = self.nU
         return (U_scc[s % nU] == np.eye(3)).all()
-    
+
     def is_not_time_reversal(self, s):
         nU = self.nU
         return not bool(s // nU)
@@ -306,15 +317,89 @@ class PWSymmetryAnalyzer:
         sbz2sbz_ks = bz2bz_ks[K_k][:, s_s]  # Reduced number of symmetries
         # Avoid -1 (see documentation in gpaw.symmetry)
         sbz2sbz_ks[sbz2sbz_ks == -1] = nk
-        
+
         smallestk_k = np.sort(sbz2sbz_ks)[:, 0]
         k2g_g = np.unique(smallestk_k, return_index=True)[1]
-        
+
         K_gs = sbz2sbz_ks[k2g_g]
         K_gk = [np.unique(K_s[K_s != nk]) for K_s in K_gs]
 
         return K_gk
 
+    def get_BZ(self):
+        # Get the little group of q
+        U_scc = []
+        for s in self.s_s:
+            U_cc, sign, _, _, _ = self.get_symmetry_operator(s)
+            U_scc.append(sign * U_cc)
+        U_scc = np.array(U_scc)
+
+        # Determine the irreducible BZ
+        bzk_kc, ibzk_kc = get_reduced_bz(self.pd.gd.cell_cv,
+                                         U_scc,
+                                         False)
+
+        return bzk_kc
+
+    def get_reduced_kd(self, pbc_c=np.ones(3, bool)):
+        # Get the little group of q
+        U_scc = []
+        for s in self.s_s:
+            U_cc, sign, _, _, _ = self.get_symmetry_operator(s)
+            U_scc.append(sign * U_cc)
+        U_scc = np.array(U_scc)
+
+        # Determine the irreducible BZ
+        bzk_kc, ibzk_kc = get_reduced_bz(self.pd.gd.cell_cv,
+                                         U_scc,
+                                         False,
+                                         pbc_c=pbc_c)
+
+        n = 3
+        N_xc = np.indices((n, n, n)).reshape((3, n**3)).T - n // 2
+
+        # Find the irreducible kpoints
+        tess = Delaunay(ibzk_kc)
+        ik_kc = []
+        for N_c in N_xc:
+            k_kc = self.kd.bzk_kc + N_c
+            k_kc = k_kc[tess.find_simplex(k_kc) >= 0]
+            if not len(ik_kc) and len(k_kc):
+                ik_kc = unique_rows(k_kc)
+            elif len(k_kc):
+                ik_kc = unique_rows(np.append(k_kc, ik_kc, axis=0))
+
+        return KPointDescriptor(ik_kc)
+
+    def unfold_kpoints(self, points_pv, tol=1e-8, mod=None):
+        points_pc = np.dot(points_pv, self.pd.gd.cell_cv.T) / (2 * np.pi)
+
+        # Get the little group of q
+        U_scc = []
+        for s in self.s_s:
+            U_cc, sign, _, _, _ = self.get_symmetry_operator(s)
+            U_scc.append(sign * U_cc)
+        U_scc = np.array(U_scc)
+
+        points = np.concatenate(np.dot(points_pc, U_scc.transpose(0, 2, 1)))
+        points = unique_rows(points, tol=tol, mod=mod)
+        points = np.dot(points, self.pd.gd.icell_cv) * (2 * np.pi)
+        return points
+
+    def get_kpoint_weight(self, k_c):
+        K = self.find_kpoint(k_c)
+        iK = self.kd.bz2ibz_k[K]
+        K_k = self.unfold_ibz_kpoint(iK)
+        K_gK = self.group_kpoints(K_k)
+
+        for K_k in K_gK:
+            if K in K_k:
+                if self.kd.refine_info is not None:
+                    weight = sum(self.kd.refine_info.weight_k[K_k])
+                    return weight
+                else:
+                    return len(K_k)
+
     def get_kpoint_mapping(self, K1, K2):
         """Get index of symmetry for mapping between K1 and K2"""
         s_s = self.s_s
@@ -333,11 +418,11 @@ class PWSymmetryAnalyzer:
         kd = self.kd
         k1_c = kd.bzk_kc[K1]
         k2_c = kd.bzk_kc[K2]
-        
+
         shift_c = np.dot(U_cc, k1_c) - k2_c * sign
         assert np.allclose(shift_c.round(), shift_c)
         shift_c = shift_c.round().astype(int)
-        
+
         return shift_c
 
     @timer('map_G')
@@ -358,21 +443,57 @@ class PWSymmetryAnalyzer:
 
     def symmetrize_wGG(self, A_wGG):
         """Symmetrize an array in GG'."""
-        tmp_wGG = np.zeros_like(A_wGG)
+
+        for A_GG in A_wGG:
+            tmp_GG = np.zeros_like(A_GG)
+
+            for s in self.s_s:
+                G_G, sign, _ = self.G_sG[s]
+                if sign == 1:
+                    tmp_GG += A_GG[G_G, :][:, G_G]
+                if sign == -1:
+                    tmp_GG += A_GG[G_G, :][:, G_G].T
+
+            A_GG[:] = tmp_GG / self.how_many_symmetries()
+
+    def symmetrize_wxx(self, A_wxx, optical_limit=False):
+        """Symmetrize an array in xx'."""
+        tmp_wxx = np.zeros_like(A_wxx)
+
+        A_cv = self.pd.gd.cell_cv
+        iA_cv = self.pd.gd.icell_cv
 
         if self.use_time_reversal:
-            AT_wGG = np.transpose(A_wGG, (0, 2, 1))
+            AT_wxx = np.transpose(A_wxx, (0, 2, 1))
 
         for s in self.s_s:
-            G_G, sign, _ = self.G_sG[s]
+            G_G, sign, shift_c = self.G_sG[s]
+            if optical_limit:
+                G_G = np.array(G_G) + 2
+                G_G = np.insert(G_G, 0, [0, 1])
+                U_cc, _, TR, shift_c, ft_c = self.get_symmetry_operator(s)
+                M_vv = np.dot(np.dot(A_cv.T, U_cc.T), iA_cv)
+
             if sign == 1:
-                tmp_wGG += A_wGG[:, G_G, :][:, :, G_G]
-            if sign == -1:
-                tmp_wGG += AT_wGG[:, G_G, :][:, :, G_G]
+                tmp = A_wxx[:, G_G, :][:, :, G_G]
+                if optical_limit:
+                    tmp[:, 0:3, :] = np.transpose(np.dot(M_vv.T,
+                                                         tmp[:, 0:3, :]),
+                                                  (1, 0, 2))
+                    tmp[:, :, 0:3] = np.dot(tmp[..., 0:3], M_vv)
+                tmp_wxx += tmp
+            elif sign == -1:
+                tmp = AT_wxx[:, G_G, :][:, :, G_G]
+                if optical_limit:
+                    tmp[:, 0:3, :] = np.transpose(np.dot(M_vv.T,
+                                                         tmp[:, 0:3, :]),
+                                                  (1, 0, 2)) * sign
+                    tmp[:, :, 0:3] = np.dot(tmp[:, :, 0:3], M_vv) * sign
+                tmp_wxx += tmp
 
         # Inplace overwriting
-        A_wGG[:] = tmp_wGG
-        
+        A_wxx[:] = tmp_wxx / self.how_many_symmetries()
+
     def symmetrize_wxvG(self, A_wxvG):
         """Symmetrize chi0_wxvG"""
         A_cv = self.pd.gd.cell_cv
@@ -392,9 +513,9 @@ class PWSymmetryAnalyzer:
             elif sign == -1:
                 tmp = sign * np.dot(M_vv.T, AT_wxvG[..., G_G])
             tmp_wxvG += np.transpose(tmp, (1, 2, 0, 3))
-            
+
         # Overwrite the input
-        A_wxvG[:] = tmp_wxvG
+        A_wxvG[:] = tmp_wxvG / self.how_many_symmetries()
 
     def symmetrize_wvv(self, A_wvv):
         """Symmetrize chi_wvv."""
@@ -415,7 +536,7 @@ class PWSymmetryAnalyzer:
             tmp_wvv += np.transpose(tmp, (1, 0, 2))
 
         # Overwrite the input
-        A_wvv[:] = tmp_wvv
+        A_wvv[:] = tmp_wvv / self.how_many_symmetries()
 
     @timer('map_v')
     def map_v(self, K1, K2, a_Mv):
@@ -447,7 +568,7 @@ class PWSymmetryAnalyzer:
         """Return symmetry operator s."""
         U_scc = self.kd.symmetry.op_scc
         ft_sc = self.kd.symmetry.op_scc
-        
+
         reds = s % self.nU
         if self.timereversal(s):
             TR = lambda x: x.conj()
@@ -455,7 +576,7 @@ class PWSymmetryAnalyzer:
         else:
             sign = 1
             TR = lambda x: x
-        
+
         return U_scc[reds], sign, TR, self.shift_sc[s], ft_sc[reds]
 
     @timer('map_G_vectors')
@@ -514,86 +635,91 @@ class PairDensity:
     def __init__(self, calc, ecut=50,
                  ftol=1e-6, threshold=1,
                  real_space_derivatives=False,
-                 world=mpi.world, txt=sys.stdout, timer=None, nblocks=1,
-                 gate_voltage=None):
+                 world=mpi.world, txt='-', timer=None,
+                 nblocks=1, gate_voltage=None):
+
         if ecut is not None:
             ecut /= Hartree
 
         if gate_voltage is not None:
-            gate_voltage /= Hartree
+            gate_voltage = gate_voltage / Hartree
 
         self.ecut = ecut
         self.ftol = ftol
         self.threshold = threshold
         self.real_space_derivatives = real_space_derivatives
-        self.world = world
         self.gate_voltage = gate_voltage
 
+        self.timer = timer or Timer()
+
+        self.world = world
+
         if nblocks == 1:
-            self.blockcomm = self.world.new_communicator([world.rank])
+            self.blockcomm = world.new_communicator([world.rank])
             self.kncomm = world
         else:
             assert world.size % nblocks == 0, world.size
             rank1 = world.rank // nblocks * nblocks
             rank2 = rank1 + nblocks
             self.blockcomm = self.world.new_communicator(range(rank1, rank2))
-            ranks = np.arange(world.rank % nblocks, world.size, nblocks)
+            ranks = range(world.rank % nblocks, world.size, nblocks)
             self.kncomm = self.world.new_communicator(ranks)
 
-        if world.rank != 0:
-            txt = devnull
-        elif isinstance(txt, str):
-            txt = open(txt, 'w')
-        self.fd = txt
-
-        self.timer = timer or Timer()
+        self.fd = convert_string_to_fd(txt, world)
 
         with self.timer('Read ground state'):
             if isinstance(calc, str):
                 print('Reading ground state calculation:\n  %s' % calc,
                       file=self.fd)
-                if not calc.split('.')[-1] == 'gpw':
-                    calc = calc + '.gpw'
-                self.reader = io.Reader(calc, comm=mpi.serial_comm)
-                calc = GPAW(calc, txt=None, communicator=mpi.serial_comm,
-                            read_projections=False)
+                calc = GPAW(calc, txt=None, communicator=mpi.serial_comm)
             else:
-                self.reader = None
                 assert calc.wfs.world.size == 1
 
         assert calc.wfs.kd.symmetry.symmorphic
         self.calc = calc
 
+        self.fermi_level = self.calc.occupations.get_fermi_level()
+
         if gate_voltage is not None:
             self.add_gate_voltage(gate_voltage)
+        else:
+            self.add_gate_voltage(gate_voltage=0)
 
-        self.spos_ac = calc.atoms.get_scaled_positions()
+        self.spos_ac = calc.spos_ac
 
         self.nocc1 = None  # number of completely filled bands
         self.nocc2 = None  # number of non-empty bands
         self.count_occupied_bands()
 
-        self.vol = abs(np.linalg.det(calc.wfs.gd.cell_cv))
-
         self.ut_sKnvR = None  # gradient of wave functions for optical limit
 
+        self.vol = abs(np.linalg.det(calc.wfs.gd.cell_cv))
+
+        kd = self.calc.wfs.kd
+        self.KDTree = cKDTree(np.mod(np.mod(kd.bzk_kc, 1).round(6), 1))
         print('Number of blocks:', nblocks, file=self.fd)
 
+    def find_kpoint(self, k_c):
+        return self.KDTree.query(np.mod(np.mod(k_c, 1).round(6), 1))[1]
+
     def add_gate_voltage(self, gate_voltage=0):
         """Shifts the Fermi-level by e * Vg. By definition e = 1."""
         assert isinstance(self.calc.occupations, FermiDirac)
         print('Shifting Fermi-level by %.2f eV' % (gate_voltage * Hartree),
               file=self.fd)
-
+        self.fermi_level += gate_voltage
         for kpt in self.calc.wfs.kpt_u:
-            kpt.f_n = (self.shift_occupations(kpt.eps_n, gate_voltage)
-                       * kpt.weight)
+            kpt.f_n = (self.shift_occupations(kpt.eps_n, gate_voltage) *
+                       kpt.weight)
 
     def shift_occupations(self, eps_n, gate_voltage):
         """Shift fermilevel."""
-        fermi = self.calc.occupations.get_fermi_level() + gate_voltage
+        fermi = self.fermi_level
         width = self.calc.occupations.width
-        tmp = (eps_n - fermi) / width
+        if width < 1e-9:
+            return (eps_n < fermi).astype(float)
+        else:
+            tmp = (eps_n - fermi) / width
         f_n = np.zeros_like(eps_n)
         f_n[tmp <= 100] = 1 / (1 + np.exp(tmp[tmp <= 100]))
         f_n[tmp > 100] = 0.0
@@ -645,7 +771,7 @@ class PairDensity:
                     self.mysKn1n2.append((s, K, n1 + band1, n2 + band1))
                 i += nbands
 
-        print('BZ k-points:', self.calc.wfs.kd.description, file=self.fd)
+        print('BZ k-points:', self.calc.wfs.kd, file=self.fd)
         print('Distributing spins, k-points and bands (%d x %d x %d)' %
               (ns, nk, nbands),
               'over %d process%s' %
@@ -654,7 +780,7 @@ class PairDensity:
         print('Number of blocks:', self.blockcomm.size, file=self.fd)
 
     @timer('Get a k-point')
-    def get_k_point(self, s, K, n1, n2, block=False):
+    def get_k_point(self, s, k_c, n1, n2, load_wfs=True, block=False):
         """Return wave functions for a specific k-point and spin.
 
         s: int
@@ -666,6 +792,17 @@ class PairDensity:
         """
 
         wfs = self.calc.wfs
+        kd = wfs.kd
+
+        # Parse kpoint: is k_c an index or a vector
+        if not isinstance(k_c, numbers.Integral):
+            K = self.find_kpoint(k_c)
+            shift0_c = (kd.bzk_kc[K] - k_c).round().astype(int)
+        else:
+            # Fall back to index
+            K = k_c
+            shift0_c = np.array([0, 0, 0])
+            k_c = None
 
         if block:
             nblocks = self.blockcomm.size
@@ -679,7 +816,9 @@ class PairDensity:
         nb = min(na + blocksize, n2)
 
         U_cc, T, a_a, U_aii, shift_c, time_reversal = \
-            self.construct_symmetry_operators(K)
+            self.construct_symmetry_operators(K, k_c=k_c)
+
+        shift_c += -shift0_c
         ik = wfs.kd.bz2ibz_k[K]
         kpt = wfs.kpt_u[s * wfs.kd.nibzkpts + ik]
 
@@ -688,35 +827,24 @@ class PairDensity:
         eps_n = kpt.eps_n[n1:n2]
         f_n = kpt.f_n[n1:n2] / kpt.weight
 
-        psit_nG = kpt.psit_nG
-        ut_nR = wfs.gd.empty(nb - na, wfs.dtype)
-        for n in range(na, nb):
-            ut_nR[n - na] = T(wfs.pd.ifft(psit_nG[n], ik))
+        if not load_wfs:
+            return KPoint(s, K, n1, n2, blocksize, na, nb,
+                          None, eps_n, f_n, None, shift_c)
 
-        P_ani = []
-        if self.reader is None:
+        with self.timer('load wfs'):
+            psit_nG = kpt.psit_nG
+            ut_nR = wfs.gd.empty(nb - na, wfs.dtype)
+            for n in range(na, nb):
+                ut_nR[n - na] = T(wfs.pd.ifft(psit_nG[n], ik))
+
+        with self.timer('Load projections'):
+            P_ani = []
             for b, U_ii in zip(a_a, U_aii):
                 P_ni = np.dot(kpt.P_ani[b][na:nb], U_ii)
                 if time_reversal:
                     P_ni = P_ni.conj()
                 P_ani.append(P_ni)
-        else:
-            II_a = []
-            I1 = 0
-            for U_ii in U_aii:
-                I2 = I1 + len(U_ii)
-                II_a.append((I1, I2))
-                I1 = I2
-                
-            P_ani = []
-            P_nI = self.reader.get('Projections', kpt.s, kpt.k)
-            for b, U_ii in zip(a_a, U_aii):
-                I1, I2 = II_a[b]
-                P_ni = np.dot(P_nI[na:nb, I1:I2], U_ii)
-                if time_reversal:
-                    P_ni = P_ni.conj()
-                P_ani.append(P_ni)
-        
+
         return KPoint(s, K, n1, n2, blocksize, na, nb,
                       ut_nR, eps_n, f_n, P_ani, shift_c)
 
@@ -724,7 +852,7 @@ class PairDensity:
                                 PWSA=None, disable_optical_limit=False,
                                 unsymmetrized=False, use_more_memory=1):
         """Generator for returning pair densities.
-        
+
         Returns the pair densities between the occupied and
         the states in range(m1, m2).
 
@@ -786,7 +914,7 @@ class PairDensity:
                     weight = np.sqrt(PWSA.how_many_symmetries() / len(K_k))
 
                 # Use kpt2 to compute intraband transitions
-                # These conditions are sufficent to make sure
+                # These conditions are sufficient to make sure
                 # that it still works in parallel
                 if kpt1.n1 == 0 and self.blockcomm.rank == 0 and \
                    optical_limit and intraband:
@@ -850,7 +978,7 @@ class PairDensity:
                     n0_mG = n0_nmG.reshape((-1, nG))
                     if optical_limit:
                         n0_mv = n0_nmv.reshape((-1, 3))
-                    
+
                     if unsymmetrized:
                         if optical_limit:
                             yield (None, df_m, deps_m,
@@ -888,17 +1016,28 @@ class PairDensity:
                         yield (None, df_M, deps_M, n_MG, n_Mv, None)
                     else:
                         yield (None, df_M, deps_M, n_MG, None, None)
-                        
+
         pb.finish()
 
     @timer('Get kpoint pair')
-    def get_kpoint_pair(self, pd, s, K, n1, n2, m1, m2):
-        wfs = self.calc.wfs
+    def get_kpoint_pair(self, pd, s, Kork_c, n1, n2, m1, m2,
+                        load_wfs=True, block=False):
+        # wfs = self.calc.wfs
+        # bzk_kc = wfs.kd.bzk_kc
+
+        if isinstance(Kork_c, int):
+            # If k_c is an integer then it refers to
+            # the index of the kpoint in the BZ
+            k_c = self.calc.wfs.kd.bzk_kc[Kork_c]
+        else:
+            k_c = Kork_c
+
         q_c = pd.kd.bzk_kc[0]
         with self.timer('get k-points'):
-            kpt1 = self.get_k_point(s, K, n1, n2)
-            K2 = wfs.kd.find_k_plus_q(q_c, [kpt1.K])[0]
-            kpt2 = self.get_k_point(s, K2, m1, m2, block=True)
+            kpt1 = self.get_k_point(s, k_c, n1, n2, load_wfs=load_wfs)
+            # K2 = wfs.kd.find_k_plus_q(q_c, [kpt1.K])[0]
+            kpt2 = self.get_k_point(s, k_c + q_c, m1, m2,
+                                    load_wfs=load_wfs, block=block)
 
         with self.timer('fft indices'):
             Q_G = self.get_fft_indices(kpt1.K, kpt2.K, q_c, pd,
@@ -909,53 +1048,51 @@ class PairDensity:
     @timer('get_pair_density')
     def get_pair_density(self, pd, kptpair, n_n, m_m,
                          optical_limit=False, intraband=False,
-                         Q_aGii=None):
+                         Q_aGii=None, block=False, direction=2,
+                         extend_head=True):
         """Get pair density for a kpoint pair."""
-        if optical_limit:
-            assert np.allclose(pd.kd.bzk_kc[0], 0.0)
+        ol = optical_limit = np.allclose(pd.kd.bzk_kc[0], 0.0)
+        eh = extend_head
+        cpd = self.calculate_pair_densities  # General pair densities
+        opd = self.optical_pair_density  # Interband pair densities / q
 
         if Q_aGii is None:
             Q_aGii = self.initialize_paw_corrections(pd)
-            
+
         kpt1 = kptpair.kpt1
         kpt2 = kptpair.kpt2
         Q_G = kptpair.Q_G  # Fourier components of kpoint pair
+        nG = len(Q_G)
 
-        n_nmG = pd.empty((len(n_n), len(m_m)))
-        if optical_limit:
-            n_nmv = np.zeros((len(n_n), len(m_m), 3), pd.dtype)
+        if extend_head:
+            n_nmG = np.zeros((len(n_n), len(m_m), nG + 2 * ol), pd.dtype)
         else:
-            n_nmv = None
+            n_nmG = np.zeros((len(n_n), len(m_m), nG), pd.dtype)
 
         for j, n in enumerate(n_n):
             Q_G = kptpair.Q_G
             with self.timer('conj'):
-                ut1cc_R = kpt1.ut_nR[n].conj()
+                ut1cc_R = kpt1.ut_nR[n - kpt1.na].conj()
             with self.timer('paw'):
-                C1_aGi = [np.dot(Q_Gii, P1_ni[n].conj())
+                C1_aGi = [np.dot(Q_Gii, P1_ni[n - kpt1.na].conj())
                           for Q_Gii, P1_ni in zip(Q_aGii, kpt1.P_ani)]
-                n_nmG[j] = self.calculate_pair_densities(ut1cc_R,
-                                                         C1_aGi, kpt2,
-                                                         pd, Q_G)
+                n_nmG[j, :, 2 * ol * eh:] = cpd(ut1cc_R, C1_aGi, kpt2, pd, Q_G,
+                                                block=block)
             if optical_limit:
-                n_nmv[j] = self.optical_pair_density(n, m_m, kpt1, kpt2)
-
-        if optical_limit:
-            n_nmG[..., 0] = n_nmv[..., 0]
-
-        if intraband:
-            vel_mv = self.intraband_pair_density(kpt2)
-        else:
-            vel_mv = None
-
-        return n_nmG, n_nmv, vel_mv
+                if extend_head:
+                    n_nmG[j, :, 0:3] = opd(n, m_m, kpt1, kpt2,
+                                           block=block)
+                else:
+                    n_nmG[j, :, 0] = opd(n, m_m, kpt1, kpt2,
+                                         block=block)[:, direction]
+        return n_nmG
 
     @timer('get_pair_momentum')
     def get_pair_momentum(self, pd, kptpair, n_n, m_m, Q_avGii=None):
         """Calculate matrix elements of the momentum operator.
 
         Calculates::
-                                               
+
           n_{nm\mathrm{k}}\int_{\Omega_{\mathrm{cell}}}\mathrm{d}\mathbf{r}
           \psi_{n\mathrm{k}}^*(\mathbf{r})
           e^{-i\,(\mathrm{q} + \mathrm{G})\cdot\mathbf{r}}
@@ -1013,7 +1150,7 @@ class PairDensity:
                 utvcc_R = -ut_nvR[n, v].conj()
                 Cv1_aGi = [np.dot(P1_ni[n].conj(), Q_vGii[v])
                            for Q_vGii, P1_ni in zip(Q_avGii, kpt1.P_ani)]
-                
+
                 nv_mG = self.calculate_pair_densities(utvcc_R,
                                                       Cv1_aGi, kpt2,
                                                       pd, Q_G)
@@ -1026,7 +1163,8 @@ class PairDensity:
         return n_nmvG
 
     @timer('Calculate pair-densities')
-    def calculate_pair_densities(self, ut1cc_R, C1_aGi, kpt2, pd, Q_G):
+    def calculate_pair_densities(self, ut1cc_R, C1_aGi, kpt2, pd, Q_G,
+                                 block=True):
         """Calculate FFT of pair-densities and add PAW corrections.
 
         ut1cc_R: 3-d complex ndarray
@@ -1056,7 +1194,7 @@ class PairDensity:
             for C1_Gi, P2_mi in zip(C1_aGi, kpt2.P_ani):
                 gemm(1.0, C1_Gi, P2_mi, 1.0, n_mG[:myblocksize], 't')
 
-        if self.blockcomm.size == 1:
+        if not block or self.blockcomm.size == 1:
             return n_mG
         else:
             n_MG = pd.empty(kpt2.blocksize * self.blockcomm.size)
@@ -1064,7 +1202,7 @@ class PairDensity:
             return n_MG[:kpt2.n2 - kpt2.n1]
 
     @timer('Optical limit')
-    def optical_pair_velocity(self, n, m_m, kpt1, kpt2):
+    def optical_pair_velocity(self, n, m_m, kpt1, kpt2, block=False):
         if self.ut_sKnvR is None or kpt1.K not in self.ut_sKnvR[kpt1.s]:
             self.ut_sKnvR = self.calculate_derivatives(kpt1)
 
@@ -1073,25 +1211,26 @@ class PairDensity:
         k_c = kd.bzk_kc[kpt1.K] + kpt1.shift_c
         k_v = 2 * np.pi * np.dot(k_c, np.linalg.inv(gd.cell_cv).T)
 
-        ut_vR = self.ut_sKnvR[kpt1.s][kpt1.K][n]
+        ut_vR = self.ut_sKnvR[kpt1.s][kpt1.K][n - kpt1.n1]
         atomdata_a = self.calc.wfs.setups
-        C_avi = [np.dot(atomdata.nabla_iiv.T, P_ni[n])
+        C_avi = [np.dot(atomdata.nabla_iiv.T, P_ni[n - kpt1.na])
                  for atomdata, P_ni in zip(atomdata_a, kpt1.P_ani)]
 
         blockbands = kpt2.nb - kpt2.na
         n0_mv = np.empty((kpt2.blocksize, 3), dtype=complex)
         nt_m = np.empty(kpt2.blocksize, dtype=complex)
-        n0_mv[:blockbands] = -self.calc.wfs.gd.integrate(ut_vR, kpt2.ut_nR).T
-        nt_m[:blockbands] = self.calc.wfs.gd.integrate(kpt1.ut_nR[n],
+        n0_mv[:blockbands] = -self.calc.wfs.gd.integrate(ut_vR,
+                                                         kpt2.ut_nR).T
+        nt_m[:blockbands] = self.calc.wfs.gd.integrate(kpt1.ut_nR[n - kpt1.na],
                                                        kpt2.ut_nR)
 
-        n0_mv[:blockbands] += (1j * nt_m[:blockbands, np.newaxis]
-                               * k_v[np.newaxis, :])
+        n0_mv[:blockbands] += (1j * nt_m[:blockbands, np.newaxis] *
+                               k_v[np.newaxis, :])
 
         for C_vi, P_mi in zip(C_avi, kpt2.P_ani):
             gemm(1.0, C_vi, P_mi, 1.0, n0_mv[:blockbands], 'c')
 
-        if self.blockcomm.size > 1:
+        if block and self.blockcomm.size > 1:
             n0_Mv = np.empty((kpt2.blocksize * self.blockcomm.size, 3),
                              dtype=complex)
             self.blockcomm.all_gather(n0_mv, n0_Mv)
@@ -1099,14 +1238,15 @@ class PairDensity:
 
         return -1j * n0_mv
 
-    def optical_pair_density(self, n, m_m, kpt1, kpt2):
+    def optical_pair_density(self, n, m_m, kpt1, kpt2,
+                             block=False):
         # Relative threshold for perturbation theory
         threshold = self.threshold
 
-        eps1 = kpt1.eps_n[n]
-        deps_m = (eps1 - kpt2.eps_n)[m_m]
-        
-        n0_mv = self.optical_pair_velocity(n, m_m, kpt1, kpt2)
+        eps1 = kpt1.eps_n[n - kpt1.n1]
+        deps_m = (eps1 - kpt2.eps_n)[m_m - kpt2.n1]
+        n0_mv = self.optical_pair_velocity(n, m_m, kpt1, kpt2,
+                                           block=block)
 
         deps_m = deps_m.copy()
         deps_m[deps_m == 0.0] = np.inf
@@ -1114,7 +1254,6 @@ class PairDensity:
         smallness_mv = np.abs(-1e-3 * n0_mv / deps_m[:, np.newaxis])
         inds_mv = (np.logical_and(np.inf > smallness_mv,
                                   smallness_mv > threshold))
-
         n0_mv *= - 1 / deps_m[:, np.newaxis]
         n0_mv[inds_mv] = 0
 
@@ -1122,7 +1261,7 @@ class PairDensity:
 
     @timer('Intraband')
     def intraband_pair_density(self, kpt, n_n=None,
-                               only_partially_occupied=True):
+                               only_partially_occupied=False):
         """Calculate intraband matrix elements of nabla"""
         # Bands and check for block parallelization
         na, nb, n1 = kpt.na, kpt.nb, kpt.n1
@@ -1130,7 +1269,8 @@ class PairDensity:
         if n_n is None:
             n_n = np.arange(na, nb)
         assert np.max(n_n) < nb, print('This is too many bands')
-        
+        assert np.min(n_n) >= na, print('This is too few bands')
+
         # Load kpoints
         kd = self.calc.wfs.kd
         gd = self.calc.wfs.gd
@@ -1141,13 +1281,16 @@ class PairDensity:
 
         # No carriers when T=0
         width = self.calc.occupations.width
-        if width == 0.0:
-            return None
 
         # Only works with Fermi-Dirac distribution
         assert isinstance(self.calc.occupations, FermiDirac)
-        dfde_n = (- 1. / width * (f_n - f_n**2.0))  # Analytical derivative
-        partocc_n = np.abs(dfde_n) > 1e-5  # Is part. occupied?
+        if width > 1e-15:
+            dfde_n = -1 / width * (f_n - f_n**2.0)  # Analytical derivative
+            partocc_n = np.abs(dfde_n) > 1e-5  # Is part. occupied?
+        else:
+            # Just include all bands to be sure
+            partocc_n = np.ones(len(f_n), dtype=bool)
+
         if only_partially_occupied and not partocc_n.any():
             return None
 
@@ -1200,13 +1343,13 @@ class PairDensity:
                     gemm(1.0, C_vi, P_ni[ind_n - na], 1.0, nabla0_nv, 'c')
 
                 vel_nnv[n] = -1j * nabla0_nv
-            
+
             for iv in range(3):
                 vel, _ = np.linalg.eig(vel_nnv[..., iv])
                 vel_nv[ind_n - na, iv] = vel  # Use eigenvalues
 
         return vel_nv[n_n - na]
-    
+
     def get_fft_indices(self, K1, K2, q_c, pd, shift0_c):
         """Get indices for G-vectors inside cutoff sphere."""
         kd = self.calc.wfs.kd
@@ -1219,7 +1362,7 @@ class PairDensity:
             N_G = np.ravel_multi_index(n_cG, pd.gd.N_c, 'wrap')
         return N_G
 
-    def construct_symmetry_operators(self, K):
+    def construct_symmetry_operators(self, K, k_c=None):
         """Construct symmetry operators for wave function and PAW projections.
 
         We want to transform a k-point in the irreducible part of the BZ to
@@ -1246,12 +1389,24 @@ class PairDensity:
         U_cc = kd.symmetry.op_scc[s]
         time_reversal = kd.time_reversal_k[K]
         ik = kd.bz2ibz_k[K]
-        k_c = kd.bzk_kc[K]
+        if k_c is None:
+            k_c = kd.bzk_kc[K]
         ik_c = kd.ibzk_kc[ik]
 
         sign = 1 - 2 * time_reversal
         shift_c = np.dot(U_cc, ik_c) - k_c * sign
-        assert np.allclose(shift_c.round(), shift_c)
+
+        try:
+            assert np.allclose(shift_c.round(), shift_c)
+        except AssertionError:
+            print('shift_c ' + str(shift_c), file=self.fd)
+            print('k_c ' + str(k_c), file=self.fd)
+            print('kd.bzk_kc[K] ' + str(kd.bzk_kc[K]), file=self.fd)
+            print('ik_c ' + str(ik_c), file=self.fd)
+            print('U_cc ' + str(U_cc), file=self.fd)
+            print('sign ' + str(sign), file=self.fd)
+            raise AssertionError
+
         shift_c = shift_c.round().astype(int)
 
         if (U_cc == np.eye(3)).all():
@@ -1281,7 +1436,6 @@ class PairDensity:
 
     @timer('Initialize PAW corrections')
     def initialize_paw_corrections(self, pd, soft=False):
-        print('Initializing PAW Corrections', file=self.fd)
         wfs = self.calc.wfs
         q_v = pd.K_qv[0]
         optical_limit = np.allclose(q_v, 0)
