@@ -1,5 +1,4 @@
-from gpaw.poisson import PoissonSolver
-from gpaw.transformers import Transformer
+from gpaw.poisson import FDPoissonSolver
 from gpaw.fd_operators import Laplace, Gradient
 from gpaw.wfd_operators import WeightedFDOperator
 from gpaw.utilities.gauss import Gaussian
@@ -8,7 +7,7 @@ import warnings
 import numpy as np
 
 
-class SolvationPoissonSolver(PoissonSolver):
+class SolvationPoissonSolver(FDPoissonSolver):
     """Base class for Poisson solvers with spatially varying dielectric.
 
     The Poisson equation
@@ -17,14 +16,17 @@ class SolvationPoissonSolver(PoissonSolver):
     """
 
     def __init__(self, nn=3, relax='J', eps=2e-10, maxiter=1000,
-                 remove_moment=None, use_charge_center=True):
+                 remove_moment=None, use_charge_center=False):
         if remove_moment is not None:
             raise NotImplementedError(
                 'Removing arbitrary multipole moments '
-                'is not implemented for SolvationPoissonSolver!'
-            )
-        PoissonSolver.__init__(self, nn, relax, eps, maxiter, remove_moment,
-                               use_charge_center=use_charge_center)
+                'is not implemented for SolvationPoissonSolver!')
+        if nn == 'M':
+            raise NotImplementedError(
+                'Mehrstellen stencil is not implemented '
+                'for SolvationPoissonSolver!')
+        FDPoissonSolver.__init__(self, nn, relax, eps, maxiter, remove_moment,
+                                 use_charge_center=use_charge_center)
 
     def set_dielectric(self, dielectric):
         """Set the dielectric.
@@ -67,88 +69,50 @@ class WeightedFDPoissonSolver(SolvationPoissonSolver):
     J. Chem. Phys. 131, 174108 (2009).
     """
 
+    def create_laplace(self, gd, scale=1.0, n=1, dtype=float):
+        operators = [Laplace(gd, scale, n, dtype)]
+        operators += [Gradient(gd, j, scale, n, dtype) for j in (0, 1, 2)]
+        return WeightedFDOperator(operators)
+
     def solve(self, phi, rho, charge=None, eps=None,
               maxcharge=1e-6,
               zero_initial_phi=False):
+        self._init()
         if self.gd.pbc_c.all():
             actual_charge = self.gd.integrate(rho)
             if abs(actual_charge) > maxcharge:
                 raise NotImplementedError(
-                    'charged periodic systems are not implemented'
-                )
+                    'charged periodic systems are not implemented')
         self.restrict_op_weights()
-        ret = PoissonSolver.solve(self, phi, rho, charge, eps, maxcharge,
-                                  zero_initial_phi)
+        ret = FDPoissonSolver.solve(self, phi, rho, charge, eps, maxcharge,
+                                    zero_initial_phi)
         return ret
 
     def restrict_op_weights(self):
         """Restric operator weights to coarse grids."""
+        self._init()
         weights = [self.dielectric.eps_gradeps] + self.op_coarse_weights
         for i, res in enumerate(self.restrictors):
             for j in range(4):
                 res.apply(weights[i][j], weights[i + 1][j])
         self.step = 0.66666666 / self.operators[0].get_diagonal_element()
 
-    def set_grid_descriptor(self, gd):
-        self.gd = gd
-        self.gds = [gd]
-        self.dv = gd.dv
-        gd = self.gd
-        self.B = None
-        self.interpolators = []
-        self.restrictors = []
-        self.operators = []
-        level = 0
-        self.presmooths = [2]
-        self.postsmooths = [1]
-        self.weights = [2. / 3.]
-        while level < 8:
-            try:
-                gd2 = gd.coarsen()
-            except ValueError:
-                break
-            self.gds.append(gd2)
-            self.interpolators.append(Transformer(gd2, gd))
-            self.restrictors.append(Transformer(gd, gd2))
-            self.presmooths.append(4)
-            self.postsmooths.append(4)
-            self.weights.append(1.0)
-            level += 1
-            gd = gd2
-        self.levels = level
-
     def get_description(self):
-        if len(self.operators) == 0:
-            return 'uninitialized WeightedFDPoissonSolver'
-        else:
-            description = SolvationPoissonSolver.get_description(self)
-            return description.replace(
-                'solver with',
-                'weighted FD solver with dielectric and'
-            )
+        description = SolvationPoissonSolver.get_description(self)
+        return description.replace(
+            'solver with',
+            'weighted FD solver with dielectric and')
 
-    def initialize(self, load_gauss=False):
-        self.presmooths[self.levels] = 8
-        self.postsmooths[self.levels] = 8
-        self.phis = [None] + [gd.zeros() for gd in self.gds[1:]]
-        self.residuals = [gd.zeros() for gd in self.gds]
-        self.rhos = [gd.zeros() for gd in self.gds]
-        self.op_coarse_weights = [
-            [g.empty() for g in (gd, ) * 4] for gd in self.gds[1:]
-        ]
-        scale = -0.25 / np.pi
-        for i, gd in enumerate(self.gds):
-            if i == 0:
-                nn = self.nn
-                weights = self.dielectric.eps_gradeps
-            else:
-                nn = 1
-                weights = self.op_coarse_weights[i - 1]
-            operators = [Laplace(gd, scale, nn)] + \
-                        [Gradient(gd, j, scale, nn) for j in (0, 1, 2)]
-            self.operators.append(WeightedFDOperator(weights, operators))
-        if load_gauss:
-            self.load_gauss()
+    def _init(self):
+        if self._initialized:
+            return
+        self.operators[0].set_weights(self.dielectric.eps_gradeps)
+        self.op_coarse_weights = []
+        for operator in self.operators[1:]:
+            weights = [gd.empty() for gd in (operator.gd, ) * 4]
+            self.op_coarse_weights.append(weights)
+            operator.set_weights(weights)
+        return SolvationPoissonSolver._init(self)
 
 
 class PolarizationPoissonSolver(SolvationPoissonSolver):
@@ -163,18 +127,14 @@ class PolarizationPoissonSolver(SolvationPoissonSolver):
     """
 
     def __init__(self, nn=3, relax='J', eps=2e-10, maxiter=1000,
-                 remove_moment=None, use_charge_center=True):
+                 remove_moment=None, use_charge_center=False):
         polarization_warning = UserWarning(
-            (
-                'PolarizationPoissonSolver is not accurate enough'
-                ' and therefore not recommended for production code!'
-            )
-        )
+            'PolarizationPoissonSolver is not accurate enough'
+            ' and therefore not recommended for production code!')
         warnings.warn(polarization_warning)
         SolvationPoissonSolver.__init__(
             self, nn, relax, eps, maxiter, remove_moment,
-            use_charge_center=use_charge_center
-        )
+            use_charge_center=use_charge_center)
         self.phi_tilde = None
 
     def get_description(self):
@@ -184,19 +144,18 @@ class PolarizationPoissonSolver(SolvationPoissonSolver):
             description = SolvationPoissonSolver.get_description(self)
             return description.replace(
                 'solver with',
-                'polarization solver with dielectric and'
-            )
+                'polarization solver with dielectric and')
 
     def solve(self, phi, rho, charge=None, eps=None,
               maxcharge=1e-6,
               zero_initial_phi=False):
+        self._init()
         if self.phi_tilde is None:
             self.phi_tilde = self.gd.zeros()
         phi_tilde = self.phi_tilde
-        niter_tilde = PoissonSolver.solve(
+        niter_tilde = FDPoissonSolver.solve(
             self, phi_tilde, rho, None, self.eps,
-            maxcharge, False
-        )
+            maxcharge, False)
 
         epsr, dx_epsr, dy_epsr, dz_epsr = self.dielectric.eps_gradeps
         dx_phi_tilde = self.gd.empty()
@@ -209,21 +168,18 @@ class PolarizationPoissonSolver(SolvationPoissonSolver):
         scalar_product = (
             dx_epsr * dx_phi_tilde +
             dy_epsr * dy_phi_tilde +
-            dz_epsr * dz_phi_tilde
-        )
+            dz_epsr * dz_phi_tilde)
 
         rho_and_pol = (
-            rho / epsr + scalar_product / (4. * np.pi * epsr ** 2)
-        )
+            rho / epsr + scalar_product / (4. * np.pi * epsr ** 2))
 
-        niter = PoissonSolver.solve(
+        niter = FDPoissonSolver.solve(
             self, phi, rho_and_pol, None, eps,
-            maxcharge, zero_initial_phi
-        )
+            maxcharge, zero_initial_phi)
         return niter_tilde + niter
 
     def load_gauss(self, center=None):
-        return PoissonSolver.load_gauss(self, center=center)
+        return FDPoissonSolver.load_gauss(self, center=center)
 
 
 class ADM12PoissonSolver(SolvationPoissonSolver):
@@ -243,24 +199,20 @@ class ADM12PoissonSolver(SolvationPoissonSolver):
     """
 
     def __init__(self, nn=3, relax='J', eps=2e-10, maxiter=1000,
-                 remove_moment=None, eta=.6, use_charge_center=True):
+                 remove_moment=None, eta=.6, use_charge_center=False):
         """Constructor for ADM12PoissonSolver.
 
         Additional arguments not present in SolvationPoissonSolver:
         eta -- linear mixing parameter
         """
         adm12_warning = UserWarning(
-            (
-                'ADM12PoissonSolver is not tested thoroughly'
-                ' and therefore not recommended for production code!'
-            )
-        )
+            'ADM12PoissonSolver is not tested thoroughly'
+            ' and therefore not recommended for production code!')
         warnings.warn(adm12_warning)
         self.eta = eta
         SolvationPoissonSolver.__init__(
             self, nn, relax, eps, maxiter, remove_moment,
-            use_charge_center=use_charge_center
-        )
+            use_charge_center=use_charge_center)
 
     def set_grid_descriptor(self, gd):
         SolvationPoissonSolver.set_grid_descriptor(self, gd)
@@ -275,32 +227,34 @@ class ADM12PoissonSolver(SolvationPoissonSolver):
             description = SolvationPoissonSolver.get_description(self)
             return description.replace(
                 'solver with',
-                'ADM12 solver with dielectric and'
-            )
+                'ADM12 solver with dielectric and')
 
-    def initialize(self, load_gauss=False):
+    def _init(self):
+        if self._initialized:
+            return
         self.rho_iter = self.gd.zeros()
         self.d_phi = self.gd.empty()
-        return SolvationPoissonSolver.initialize(self, load_gauss)
+        return SolvationPoissonSolver._init(self)
 
     def solve(self, phi, rho, charge=None, eps=None,
               maxcharge=1e-6,
               zero_initial_phi=False):
+        self._init()
         if self.gd.pbc_c.all():
             actual_charge = self.gd.integrate(rho)
             if abs(actual_charge) > maxcharge:
                 raise NotImplementedError(
-                    'charged periodic systems are not implemented'
-                )
-        return PoissonSolver.solve(
-            self, phi, rho, charge, eps, maxcharge, zero_initial_phi
-        )
+                    'charged periodic systems are not implemented')
+        return FDPoissonSolver.solve(
+            self, phi, rho, charge, eps, maxcharge, zero_initial_phi)
 
     def solve_neutral(self, phi, rho, eps=2e-10):
+        self._init()
         self.rho = rho
         return SolvationPoissonSolver.solve_neutral(self, phi, rho, eps)
 
     def iterate2(self, step, level=0):
+        self._init()
         if level == 0:
             epsr, dx_epsr, dy_epsr, dz_epsr = self.dielectric.eps_gradeps
             self.gradx.apply(self.phis[0], self.d_phi)
