@@ -5,28 +5,26 @@ operators."""
 
 import numpy as np
 
-from gpaw.external_potential import ExternalPotential
 from gpaw.utilities import unpack
 from gpaw.fd_operators import Laplace, Gradient
 from gpaw.overlap import Overlap
 from gpaw.wavefunctions.fd import FDWaveFunctions
-from gpaw.tddft.abc import *
 
 
 class TimeDependentHamiltonian:
     """Time-dependent Hamiltonian, H(t)
-    
+
     This class contains information required to apply time-dependent
     Hamiltonian to a wavefunction.
     """
-    
-    def __init__(self, wfs, atoms, hamiltonian, td_potential):
+
+    def __init__(self, wfs, spos_ac, hamiltonian, td_potential):
         """Create the TimeDependentHamiltonian-object.
-        
+
         The time-dependent potential object must (be None or) have a member
         function strength(self,time), which provides the strength of the
         time-dependent external potential to x-direction at the given time.
-        
+
         Parameters
         ----------
         wfs: FDWaveFunctions
@@ -41,13 +39,14 @@ class TimeDependentHamiltonian:
         self.hamiltonian = hamiltonian
         self.td_potential = td_potential
         self.time = self.old_time = 0
-        
+
         # internal smooth potential
         self.vt_sG = hamiltonian.gd.zeros(hamiltonian.nspins)
 
         # Increase the accuracy of Poisson solver
-        if self.hamiltonian.poisson.eps > 1e-12:
-            self.hamiltonian.poisson.eps = 1e-12
+        poisson = self.hamiltonian.poisson
+        if hasattr(poisson, 'eps') and poisson.eps > 1e-12:
+            poisson.eps = 1e-12
 
         # external potential
         #if hamiltonian.vext_g is None:
@@ -58,12 +57,12 @@ class TimeDependentHamiltonian:
 
         self.P = None
 
-        self.spos_ac = atoms.get_scaled_positions() % 1.0
+        self.spos_ac = spos_ac
         self.absorbing_boundary = None
-        
+
     def update(self, density, time):
         """Updates the time-dependent Hamiltonian.
-    
+
         Parameters
         ----------
         density: Density
@@ -76,11 +75,11 @@ class TimeDependentHamiltonian:
 
         self.old_time = self.time = time
         self.hamiltonian.update(density)
-        
+
     def half_update(self, density, time):
         """Updates the time-dependent Hamiltonian, in such a way, that a
         half of the old Hamiltonian is kept and the other half is updated.
-        
+
         Parameters
         ----------
         density: Density
@@ -90,7 +89,7 @@ class TimeDependentHamiltonian:
             the current time
 
         """
-        
+
         self.old_time = self.time
         self.time = time
 
@@ -111,7 +110,7 @@ class TimeDependentHamiltonian:
 
     def half_apply_local_potential(self, psit_nG, Htpsit_nG, s):
         """Apply the half-difference Hamiltonian operator to a set of vectors.
-        
+
         Parameters:
 
         psit_nG: ndarray
@@ -120,7 +119,7 @@ class TimeDependentHamiltonian:
             resulting H applied to psit_nG vectors.
         s: int
             spin index of k-point object defined in kpoint.py.
-        
+
         """
         # Does exactly the same as Hamiltonian.apply_local_potential
         # but uses the difference between vt_sG at time t and t+dt.
@@ -134,7 +133,7 @@ class TimeDependentHamiltonian:
     def half_apply(self, kpt, psit, hpsit, calculate_P_ani=True):
         """Applies the half-difference of the time-dependent Hamiltonian
         to the wavefunction psit of the k-point kpt.
-        
+
         Parameters
         ----------
         kpt: Kpoint
@@ -180,7 +179,7 @@ class TimeDependentHamiltonian:
     def apply(self, kpt, psit, hpsit, calculate_P_ani=True):
         """Applies the time-dependent Hamiltonian to the wavefunction psit of
         the k-point kpt.
-        
+
         Parameters
         ----------
         kpt: Kpoint
@@ -235,19 +234,19 @@ class TimeDependentHamiltonian:
         if self.td_potential is not None:
             #TODO on shaky ground here...
             strength = self.td_potential.strength
-            ExternalPotential().add_linear_field(self.wfs, self.spos_ac,
-                                                 psit, hpsit,
-                                                 0.5 * strength(self.time) +
-                                                 0.5 * strength(self.old_time),
-                                                 kpt)
+            add_linear_field(self.wfs, self.spos_ac,
+                             psit, hpsit,
+                             0.5 * strength(self.time) +
+                             0.5 * strength(self.old_time),
+                             kpt)
+
 
-            
     def set_absorbing_boundary(self, absorbing_boundary):
         """ Sets up the absorbing boundary.
             Parameters:
             absorbing_boundary: absorbing boundary object of any kind.
         """
-        
+
         self.absorbing_boundary = absorbing_boundary
         self.absorbing_boundary.set_up(self.hamiltonian.gd)
         if self.absorbing_boundary.type == 'PML':
@@ -278,8 +277,8 @@ class TimeDependentHamiltonian:
         shape = psit_nG.shape[:-3]
         P_axi = wfs.pt.dict(shape)
         wfs.pt.integrate(psit_nG, P_axi, kpt.q)
-        
-            
+
+
         #Coefficients for calculating P \psi_n
         # P = -i sum_a v_a P^a, P^a = T^{\dagger} \nabla_{R_a} T
         w_ani = wfs.pt.dict(wfs.bd.mynbands, zero=True)
@@ -302,7 +301,7 @@ class TimeDependentHamiltonian:
                 pt_ni = np.dot(dpt_ni, dO_ii)
                 #pt_aniv[a] = np.dot(Dpt_ni, dO_ii)
                 w_ani[a] += (dphi_ni + pt_ni) * v_atom[a,c]
-                        
+
             w_ani[a] *= complex(0,1)
             #dO_ani[a] *= complex(0,1)
 
@@ -313,28 +312,28 @@ class TimeDependentHamiltonian:
 # AbsorptionKickHamiltonian
 class AbsorptionKickHamiltonian:
     """Absorption kick Hamiltonian, p.r
-    
+
     This class contains information required to apply absorption kick
     Hamiltonian to a wavefunction.
     """
-    
-    def __init__(self, wfs, atoms, strength=[0.0, 0.0, 1e-3]):
+
+    def __init__(self, wfs, spos_ac, strength=[0.0, 0.0, 1e-3]):
         """Create the AbsorptionKickHamiltonian-object.
 
         Parameters
         ----------
         wfs: FDWaveFunctions
             time-independent grid-based wavefunctions
-        atoms: Atoms
-            list of atoms
+        spos_ac: ndarray
+            scaled positions
         strength: float[3]
             strength of the delta field to different directions
 
         """
 
         self.wfs = wfs
-        self.spos_ac = atoms.get_scaled_positions() % 1.0
-        
+        self.spos_ac = spos_ac
+
         # magnitude
         magnitude = np.sqrt(strength[0]*strength[0]
                              + strength[1]*strength[1]
@@ -348,11 +347,11 @@ class AbsorptionKickHamiltonian:
 
         # hamiltonian
         self.abs_hamiltonian = np.array([self.dp[0], self.dp[1], self.dp[2]])
-        
+
 
     def update(self, density, time):
         """Dummy function = does nothing. Required to have correct interface.
-        
+
         Parameters
         ----------
         density: Density or None
@@ -362,10 +361,10 @@ class AbsorptionKickHamiltonian:
 
         """
         pass
-        
+
     def half_update(self, density, time):
         """Dummy function = does nothing. Required to have correct interface.
-        
+
         Parameters
         ----------
         density: Density or None
@@ -375,11 +374,11 @@ class AbsorptionKickHamiltonian:
 
         """
         pass
-        
+
     def apply(self, kpt, psit, hpsit, calculate_P_ani=True):
         """Applies the absorption kick Hamiltonian to the wavefunction psit of
         the k-point kpt.
-        
+
         Parameters
         ----------
         kpt: Kpoint
@@ -398,33 +397,33 @@ class AbsorptionKickHamiltonian:
         hpsit[:] = 0.0
 
         #TODO on shaky ground here...
-        ExternalPotential().add_linear_field(self.wfs, self.spos_ac,
-                                             psit, hpsit,
-                                             self.abs_hamiltonian, kpt)
+        add_linear_field(self.wfs, self.spos_ac,
+                         psit, hpsit,
+                         self.abs_hamiltonian, kpt)
 
 
 # Overlap
 class TimeDependentOverlap(Overlap):
     """Time-dependent overlap operator S(t)
-    
+
     This class contains information required to apply time-dependent
     overlap operator to a set of wavefunctions.
     """
-    
-    def __init__(self, ksl, timer):
+
+    def __init__(self, timer):
         """Creates the TimeDependentOverlap-object.
-        
+
         Parameters
         ----------
         XXX TODO
 
         """
-        Overlap.__init__(self, ksl, timer)
+        Overlap.__init__(self, timer)
 
     def update_k_point_projections(self, wfs, kpt, psit=None):
         """Updates the projector function overlap integrals
         with the wavefunctions of a given k-point.
-        
+
         Parameters
         ----------
         wfs: TimeDependentWaveFunctions
@@ -443,7 +442,7 @@ class TimeDependentOverlap(Overlap):
 
     def update(self, wfs):
         """Updates the time-dependent overlap operator.
-        
+
         Parameters
         ----------
         wfs: TimeDependentWaveFunctions
@@ -452,7 +451,7 @@ class TimeDependentOverlap(Overlap):
         """
         for kpt in wfs.kpt_u:
             self.update_k_point_projections(wfs, kpt)
-    
+
     def half_update(self, wfs):
         """Updates the time-dependent overlap operator, in such a way,
         that a half of the old overlap operator is kept and the other half
@@ -478,7 +477,7 @@ class TimeDependentOverlap(Overlap):
 
         # !!! FIX ME !!! update overlap operator/projectors/...
         pass
-    
+
     #def apply(self, psit, spsit, wfs, kpt, calculate_P_ani=True):
     #    """Apply the time-dependent overlap operator to the wavefunction
     #    psit of the k-point kpt.
@@ -505,7 +504,7 @@ class TimeDependentOverlap(Overlap):
     def apply_inverse(self, a_nG, b_nG, wfs, kpt, calculate_P_ani=True, use_cg=True):
         """Apply the approximative time-dependent inverse overlap operator
         to the wavefunction psit of the k-point kpt.
-    
+
         Parameters
         ----------
         a_nG: List of coarse grids
@@ -523,7 +522,7 @@ class TimeDependentOverlap(Overlap):
             When False, existing P_uni are used
         use_cg: bool
             When True, use conjugate gradient method to solve for inverse.
-    
+
         """
         if not use_cg:
             self.timer.start('Apply approximate inverse overlap')
@@ -532,7 +531,7 @@ class TimeDependentOverlap(Overlap):
             return
 
         self.timer.start('Apply exact inverse overlap')
-        from gpaw.utilities.blas import dotu, axpy, dotc
+        from gpaw.utilities.blas import dotu, axpy
         #from gpaw.tddft.cscg import multi_zdotu, multi_scale, multi_zaxpy
         #initialization
           # Multivector dot product, a^T b, where ^T is transpose
@@ -552,7 +551,6 @@ class TimeDependentOverlap(Overlap):
         nvec = len(a_nG)
         r = wfs.gd.zeros(nvec, dtype=wfs.dtype)
         z  = wfs.gd.zeros((nvec,), dtype=wfs.dtype)
-        sx = wfs.gd.zeros(nvec, dtype=wfs.dtype)
         p = wfs.gd.zeros(nvec, dtype=wfs.dtype)
         q = wfs.gd.zeros(nvec, dtype=wfs.dtype)
         alpha = np.zeros((nvec,), dtype=wfs.dtype)
@@ -589,15 +587,15 @@ class TimeDependentOverlap(Overlap):
             multi_scale(beta, p, nvec)
             p += z
 
-            self.apply(p,q,wfs,kpt, calculate_P_ani)
+            self.apply(p, q, wfs, kpt, calculate_P_ani)
 
             multi_zdotu(alpha, p, q, nvec)
-            alpha = rho/alpha
+            alpha = rho / alpha
 
             multi_zaxpy(alpha, p, x, nvec)
             multi_zaxpy(-alpha, q, r, nvec)
 
-            multi_zdotu(normr2, r,r, nvec)
+            multi_zdotu(normr2, r, r, nvec)
 
             #rhoc = rho.copy()
             rho_prev[:] = rho.copy()
@@ -612,21 +610,25 @@ class TimeDependentOverlap(Overlap):
 
 
 class TimeDependentWaveFunctions(FDWaveFunctions):
-    def __init__(self, stencil, diagksl, orthoksl, initksl, gd, nvalence, setups,
-                 bd, world, kd, kptband_comm, timer=None):
-        FDWaveFunctions.__init__(self, stencil, diagksl, orthoksl, initksl,
-                                 gd, nvalence, setups, bd, complex, world,
-                                 kd, kptband_comm, timer)
+    def __init__(self, stencil, parallel, initksl, gd, nvalence, collinear,
+                 setups, bd, dtype, world, kd, kptband_comm, timer):
+        assert dtype == complex
+        FDWaveFunctions.__init__(self, stencil, parallel, initksl,
+                                 gd, nvalence,
+                                 setups, bd, dtype, world,
+                                 kd, kptband_comm,
+                                 collinear=collinear, timer=timer)
+        self.overlap = self.make_overlap()
 
     def make_overlap(self):
-        return TimeDependentOverlap(self.orthoksl, self.timer)
+        return TimeDependentOverlap(self.timer)
 
     def calculate_forces(self, hamiltonian, F_av):
         """ Calculate wavefunction forces with optional corrections for
             Ehrenfest dynamics
         """
 
-            
+
         #If td_correction is not none, we replace the overlap part of the
         #force, sum_n f_n eps_n < psit_n | dO / dR_a | psit_n>, with
         #sum_n f_n <psit_n | H S^-1 D^a + c.c. | psit_n >, with D^a
@@ -672,7 +674,7 @@ class TimeDependentWaveFunctions(FDWaveFunctions):
                 F_vii_sinvh_dpt = -np.dot(np.dot(FdH1_niv.transpose(), G_ni), dO_ii)
                 F_vii_sinvh_dphi = -np.dot(nabla_iiv.transpose(2,0,1), np.dot(fP_ni.conj().transpose(), G_ni))
                 F_vii += F_vii_sinvh_dpt + F_vii_sinvh_dphi
-                                    
+
                 #F_av_dO[a] += 2 * F_vii_dO.real.trace(0,1,2)
                 #F_av_dH[a] += 2 * F_vii_dH.real.trace(0,1,2)
                 F_av[a] += 2 * F_vii.real.trace(0, 1, 2)
@@ -729,14 +731,14 @@ class DummyDensity:
 # Density
 class TimeDependentDensity(DummyDensity):
     """Time-dependent density rho(t)
-    
+
     This class contains information required to get the time-dependent
     density.
     """
-    
+
     def __init__(self, paw):
         """Creates the TimeDependentDensity-object.
-        
+
         Parameters
         ----------
         paw: PAW
@@ -747,7 +749,7 @@ class TimeDependentDensity(DummyDensity):
 
     def update(self):
         """Updates the time-dependent density.
-        
+
         Parameters
         ----------
         None
@@ -756,13 +758,103 @@ class TimeDependentDensity(DummyDensity):
         #for kpt in self.wfs.kpt_u:
         #    self.wfs.pt.integrate(kpt.psit_nG, kpt.P_ani)
         self.density.update(self.wfs)
-       
+
     def get_density(self):
         """Returns the current density.
-        
+
         Parameters
         ----------
         None
 
         """
         return self.density
+
+
+def add_linear_field(wfs, spos_ac, a_nG, b_nG, strength, kpt):
+    """Adds (does NOT apply) linear field.
+
+    ::
+
+        f(x,y,z) = str_x * x + str_y * y + str_z * z to wavefunctions.
+
+    Parameters:
+
+    a_nG:
+        the wavefunctions
+    b_nG:
+        the result
+    strength: float[3]
+        strength of the linear field
+    kpt: KPoint
+        K-point
+    """
+
+    gd = wfs.gd
+
+    # apply local part of x to smooth wavefunctions psit_n
+    for i in range(gd.n_c[0]):
+        x = (i + gd.beg_c[0]) * gd.h_cv[0, 0]
+        b_nG[:, i, :, :] += (strength[0] * x) * a_nG[:, i, :, :]
+
+    # FIXME: combine y and z to one vectorized operation,
+    # i.e., make yz-array and take its product with a_nG
+
+    # apply local part of y to smooth wavefunctions psit_n
+    for i in range(gd.n_c[1]):
+        y = (i + gd.beg_c[1]) * gd.h_cv[1, 1]
+        b_nG[:, :, i, :] += (strength[1] * y) * a_nG[:, :, i, :]
+
+    # apply local part of z to smooth wavefunctions psit_n
+    for i in range(gd.n_c[2]):
+        z = (i + gd.beg_c[2]) * gd.h_cv[2, 2]
+        b_nG[:, :, :, i] += (strength[2] * z) * a_nG[:, :, :, i]
+
+    # apply the non-local part for each nucleus
+
+    # number of wavefunctions, psit_nG
+    n = len(a_nG)
+    P_ani = wfs.pt.dict(n)
+    wfs.pt.integrate(a_nG, P_ani, kpt.q)
+
+    coef_ani = {}
+    for a, P_ni in P_ani.items():
+        c0 = np.dot(spos_ac[a] * gd.cell_cv.diagonal(), strength)
+        cxyz = strength
+        # calculate coefficient
+        # ---------------------
+        #
+        # coeffs_ni =
+        #   P_nj * c0 * 1_ij
+        #   + P_nj * cx * x_ij
+        #
+        # where (see spherical_harmonics.py)
+        #
+        #   1_ij = sqrt(4pi) Delta_0ij
+        #   y_ij = sqrt(4pi/3) Delta_1ij
+        #   z_ij = sqrt(4pi/3) Delta_2ij
+        #   x_ij = sqrt(4pi/3) Delta_3ij
+        # ...
+
+        Delta_iiL = wfs.setups[a].Delta_iiL
+
+        #   1_ij = sqrt(4pi) Delta_0ij
+        #   y_ij = sqrt(4pi/3) Delta_1ij
+        #   z_ij = sqrt(4pi/3) Delta_2ij
+        #   x_ij = sqrt(4pi/3) Delta_3ij
+        oneij = np.sqrt(4 * np.pi) \
+            * np.dot(P_ni, Delta_iiL[:, :, 0])
+        yij = np.sqrt(4 * np.pi / 3) \
+            * np.dot(P_ni, Delta_iiL[:, :, 1])
+        zij = np.sqrt(4 * np.pi / 3) \
+            * np.dot(P_ni, Delta_iiL[:, :, 2])
+        xij = np.sqrt(4 * np.pi / 3) \
+            * np.dot(P_ni, Delta_iiL[:, :, 3])
+
+        # coefficients
+        # coefs_ni = sum_j ( <phi_i| f(x,y,z) | phi_j>
+        #                    - <phit_i| f(x,y,z) | phit_j> ) P_nj
+        coef_ani[a] = (c0 * oneij +
+                       cxyz[0] * xij + cxyz[1] * yij + cxyz[2] * zij)
+
+    # add partial wave pt_nG to psit_nG with proper coefficient
+    wfs.pt.add(b_nG, coef_ani, kpt.q)
