@@ -11,7 +11,6 @@ from gpaw.io.fmf import FMF
 from gpaw.utilities.blas import gemmdot
 
 
-
 def print_projectors(setup):
     """Print information on the projectors of input nucleus object.
 
@@ -24,7 +23,7 @@ def print_projectors(setup):
     else:
         n_j = setup.n_j
         l_j = setup.l_j
-    
+
     angular = [['1'],
                ['y', 'z', 'x'],
                ['xy', 'yz', '3z^2-r^2', 'xz', 'x^2-y^2'],
@@ -41,7 +40,7 @@ def print_projectors(setup):
             print('%2s %s %s_%s' % (i, n, 'spdf'[l], angular[l][m]))
             i += 1
 
-            
+
 def number_of_projectors(setup):
     """Returns the number of the bound state projectors.
 
@@ -54,7 +53,7 @@ def number_of_projectors(setup):
     else:
         n_j = setup.n_j
         l_j = setup.l_j
-    
+
     i = 0
     for n, l in zip(n_j, l_j):
         for m in range(2 * l + 1):
@@ -62,7 +61,7 @@ def number_of_projectors(setup):
                 i += 1
     return i
 
-    
+
 def get_angular_projectors(setup, angular, type='bound'):
     """Determine the projector indices which have specified angula
     quantum number.
@@ -76,7 +75,6 @@ def get_angular_projectors(setup, angular, type='bound'):
         nj = len([n for n in setup.n_j if n >= 0])
     else:
         nj = len(setup.n_j)
-            
 
     # Choose the relevant projectors
     projectors = []
@@ -90,7 +88,7 @@ def get_angular_projectors(setup, angular, type='bound'):
 
     return projectors
 
-    
+
 def delta(x, x0, width, mode='Gauss'):
     """Return a gaussian of given width centered at x0."""
     if mode == 'Gauss':
@@ -100,7 +98,7 @@ def delta(x, x0, width, mode='Gauss'):
         return (2 / pi / width) / ((np.clip(((x - x0) / (width / 2))**2,
                                             -100.0, 100.0)) + 1)
 
-        
+
 def fold(energies, weights, npts, width, mode='Gauss'):
     """Take a list of energies and weights, and sum a delta function
     for each."""
@@ -112,7 +110,7 @@ def fold(energies, weights, npts, width, mode='Gauss'):
         dos_e += w * delta(e, e0, width, mode=mode)
     return e, dos_e
 
-    
+
 def raw_orbital_LDOS(paw, a, spin, angular='spdf', nbands=None):
     """Return a list of eigenvalues, and their weight on the specified atom.
 
@@ -133,31 +131,32 @@ def raw_orbital_LDOS(paw, a, spin, angular='spdf', nbands=None):
         nb = wfs.bd.nbands
     else:
         nb = nbands
-        assert nb <= wfs.bd.nbands, ("nbands higher than available number" +
-                                     "of bands")
+        assert nb <= wfs.bd.nbands, ('nbands higher than available number' +
+                                     'of bands')
 
     if a < 0:
         # Allow list-style negative indices; we'll need the positive a for the
         # dictionary lookup later
         a = len(wfs.setups) + a
 
+    I1 = sum(setup.ni for setup in wfs.setups[:a])
     setup = wfs.setups[a]
+    I2 = I1 + setup.ni
+
     energies = np.empty(nb * nk)
     weights_xi = np.empty((nb * nk, setup.ni))
     x = 0
     for k, w in enumerate(w_k):
-        eps = wfs.collect_eigenvalues(k=k, s=spin)[:nb]
-        #print(wfs.world.rank, type(eps))
-        if eps is not None:
-            energies[x:x + nb] = eps
-        u = spin * nk + k
-        P_ani = wfs.kpt_u[u].P_ani
-        if a in P_ani:
-            weights_xi[x:x + nb, :] = w * np.absolute(P_ani[a][:nb, :])**2
+        eps_n = wfs.collect_eigenvalues(k=k, s=spin)
+        if len(eps_n) > 0:
+            energies[x:x + nb] = eps_n[:nb]
+        P_nI = wfs.collect_projections(k, spin)
+        if P_nI is not None:
+            weights_xi[x:x + nb, :] = w * abs(P_nI[:nb, I1:I2])**2
         x += nb
 
     wfs.world.broadcast(energies, 0)
-    wfs.world.broadcast(weights_xi, wfs.atom_partition.rank_a[a])
+    wfs.world.broadcast(weights_xi, 0)
 
     if angular is None:
         return energies, weights_xi
@@ -169,10 +168,53 @@ def raw_orbital_LDOS(paw, a, spin, angular='spdf', nbands=None):
                                  indices=projectors, axis=1), axis=1)
         return energies, weights
 
-        
+
+def raw_spinorbit_orbital_LDOS(paw, a, spin, angular='spdf'):
+    """Like former, but with spinorbit coupling."""
+
+    from gpaw.spinorbit import get_spinorbit_eigenvalues
+    from gpaw.wannier90 import get_spinorbit_projections
+
+    e_mk, v_knm = get_spinorbit_eigenvalues(paw, return_wfs=True)
+    e_mk /= Hartree
+    ns = paw.wfs.nspins
+    w_k = paw.wfs.kd.weight_k
+    nk = len(w_k)
+    nb = len(e_mk)
+
+    if a < 0:
+        # Allow list-style negative indices; we'll need the positive a for the
+        # dictionary lookup later
+        a = len(paw.wfs.setups) + a
+
+    setup = paw.wfs.setups[a]
+    energies = np.empty(nb * nk)
+    weights_xi = np.empty((nb * nk, setup.ni))
+    x = 0
+    for k, w in enumerate(w_k):
+        energies[x:x + nb] = e_mk[:, k]
+        P_ami = get_spinorbit_projections(paw, k, v_knm[k])
+        if ns == 2:
+            weights_xi[x:x + nb, :] = w * np.absolute(P_ami[a][:, spin::2])**2
+        else:
+            weights_xi[x:x + nb, :] = w * np.absolute(P_ami[a][:, 0::2])**2 / 2
+            weights_xi[x:x + nb, :] += w * np.absolute(P_ami[a][:, 1::2])**2  / 2
+        x += nb
+
+    if angular is None:
+        return energies, weights_xi
+    elif isinstance(angular, int):
+        return energies, weights_xi[:, angular]
+    else:
+        projectors = get_angular_projectors(setup, angular, type='bound')
+        weights = np.sum(np.take(weights_xi,
+                                 indices=projectors, axis=1), axis=1)
+        return energies, weights
+
+
 def all_electron_LDOS(paw, mol, spin, lc=None, wf_k=None, P_aui=None):
     """Returns a list of eigenvalues, and their weights on a given molecule
-    
+
        wf_k should be a list of pseudo_wavefunctons of a Kohn-Sham state,
        corresponding to the different kpoints. It should be accompanied by a
        list of projector overlaps: P_aui=[[kpt.P_ani[a][n] for kpt in
@@ -189,19 +231,19 @@ def all_electron_LDOS(paw, mol, spin, lc=None, wf_k=None, P_aui=None):
     w_k = paw.wfs.kd.weight_k
     nk = len(w_k)
     nb = paw.wfs.bd.nbands
-    
+
     P_kn = np.zeros((nk, nb), np.complex)
     if wf_k is None:
         if lc is None:
-            lc = [[1,0,0,0] for a in mol]
+            lc = [[1, 0, 0, 0] for a in mol]
         for k, kpt in enumerate(paw.wfs.kpt_u[spin * nk:(spin + 1) * nk]):
             N = 0
             for atom, w_a in zip(mol, lc):
-                i=0
+                i = 0
                 for w_o in w_a:
                     P_kn[k] += w_o * kpt.P_ani[atom][:, i]
                     N += abs(w_o)**2
-                    i +=1
+                    i += 1
         P_kn /= sqrt(N)
 
     else:
@@ -214,33 +256,35 @@ def all_electron_LDOS(paw, mol, spin, lc=None, wf_k=None, P_aui=None):
                     p_i = kpt.P_ani[a][n]
                     for i in range(len(p_i)):
                         for j in range(len(p_i)):
-                            P_kn[k][n] += (P_aui[b][spin*nk + k][i] *
+                            P_kn[k][n] += (P_aui[b][spin * nk + k][i] *
                                            atom.dO_ii[i][j] * p_i[j])
-            print('# k', k, ' Sum_m |<m|n>|^2 =',  sum(abs(P_kn[k])**2))
-          
+            print('# k', k, ' Sum_m |<m|n>|^2 =', sum(abs(P_kn[k])**2))
+
     energies = np.empty(nb * nk)
     weights = np.empty(nb * nk)
     x = 0
     for k, w in enumerate(w_k):
-        energies[x:x + nb] = paw.wfs.collect_eigenvalues(k=k, s=spin)
-        weights[x:x + nb] = w * abs(P_kn[k])**2
+        eps_n = paw.wfs.collect_eigenvalues(k=k, s=spin)
+        if len(eps_n) > 0:
+            energies[x:x + nb] = eps_n
+            weights[x:x + nb] = w * abs(P_kn[k])**2
         x += nb
-        
+
     return energies, weights
 
-    
+
 def get_all_electron_IPR(paw):
     density = paw.density
     wfs = paw.wfs
     n_G = wfs.gd.empty()
     n_g = density.finegd.empty()
     print()
-    print("inverse participation function")
-    print("-"*35)
-    print("%5s %5s %10s %10s" % ("k","band","eps","ipr"))
-    print("-"*35)
+    print('inverse participation function')
+    print('-' * 35)
+    print('%5s %5s %10s %10s' % ('k', 'band', 'eps', 'ipr'))
+    print('-' * 35)
     for k, kpt in enumerate(paw.wfs.kpt_u):
-        for n, (eps, psit_G)  in enumerate(zip(kpt.eps_n, kpt.psit_nG)):
+        for n, (eps, psit_G) in enumerate(zip(kpt.eps_n, kpt.psit_nG)):
             n_G[:] = 0.0
             wfs.add_orbital_density(n_G, kpt, n)
             density.interpolator.apply(n_G, n_g)
@@ -255,24 +299,30 @@ def get_all_electron_IPR(paw):
                 # Get D_sp for atom a
                 D_sp = np.array(wfs.get_orbital_density_matrix(a, kpt, n))
 
-                # density a function of L and partial wave radial pair density coefficient
+                # density a function of L and partial wave radial pair
+                # density coefficient
                 D_sLq = gemmdot(D_sp, xccorr.B_Lqp, trans='t')
-                
+
                 # Create pseudo/ae density iterators for integration
                 n_iter = xccorr.expand_density(D_sLq, xccorr.n_qg, None)
                 nt_iter = xccorr.expand_density(D_sLq, xccorr.nt_qg, None)
 
-                # Take the spherical average of smooth and ae radial xc potentials
+                # Take the spherical average of smooth and ae radial
+                # xc potentials
                 for n_sg, nt_sg, integrator in zip(n_iter,
-                                                    nt_iter,
-                                                    xccorr.get_integrator(None)):
-                    ipr += integrator.weight * np.sum((n_sg[0]**2-nt_sg[0]**2) * xccorr.rgd.dv_g)
-                    norm += integrator.weight * np.sum((n_sg[0]-nt_sg[0]) * xccorr.rgd.dv_g)
+                                                   nt_iter,
+                                                   xccorr.get_integrator(
+                                                       None)):
+                    ipr += integrator.weight * np.sum((n_sg[0]**2 -
+                                                       nt_sg[0]**2) *
+                                                      xccorr.rgd.dv_g)
+                    norm += integrator.weight * np.sum((n_sg[0] - nt_sg[0]) *
+                                                       xccorr.rgd.dv_g)
+
+            print('%5i %5i %10.5f %10.5f' % (k, n, eps, ipr / norm**2))
+    print('-' * 35)
+
 
-            print("%5i %5i %10.5f %10.5f" % (k, n, eps, ipr/norm**2))
-    print("-"*35)
-            
-                            
 def raw_wignerseitz_LDOS(paw, a, spin):
     """Return a list of eigenvalues, and their weight on the specified atom"""
     wfs = paw.wfs
@@ -294,9 +344,9 @@ def raw_wignerseitz_LDOS(paw, a, spin):
             P_i = wfs.kpt_u[u].P_ani[a][n]
             P_p = pack(np.outer(P_i, P_i))
             Delta_p = sqrt(4 * pi) * wfs.setups[a].Delta_pL[:, 0]
-            weights[x + n] = w * (gd.integrate(abs(
-                np.where(atom_index == a, psit_G, 0.0))**2)
-                                  + np.dot(Delta_p, P_p))
+            weights[x + n] = w * (
+                gd.integrate(abs(np.where(atom_index == a, psit_G, 0.0))**2) +
+                np.dot(Delta_p, P_p))
         x += nb
     return energies, weights
 
@@ -321,7 +371,7 @@ class RawLDOS:
         spd = np.zeros((wfs.nspins, nibzkpts, wfs.bd.nbands, 3))
 
         if hasattr(atom, '__iter__'):
-            # atom is a list of atom indicies 
+            # atom is a list of atom indices
             for a in atom:
                 spd += self.get(a)
             return spd
@@ -340,7 +390,7 @@ class RawLDOS:
     def by_element(self):
         """Return a dict with elements as keys and LDOS as values."""
         elemi = {}
-        for i,a in enumerate(self.paw.atoms):
+        for i, a in enumerate(self.paw.atoms):
             symbol = a.symbol
             if symbol in elemi:
                 elemi[symbol].append(i)
@@ -350,7 +400,7 @@ class RawLDOS:
             elemi[key] = self.get(elemi[key])
         return elemi
 
-    def to_file(self, 
+    def to_file(self,
                 ldbe,
                 filename=None,
                 width=None,
@@ -358,10 +408,11 @@ class RawLDOS:
                 bound=False):
         """Write the LDOS to a file.
 
-        If a width is given, the LDOS will be Gaussian folded and shifted to set 
-        Fermi energy to 0 eV. The latter can be avoided by setting shift=False. 
+        If a width is given, the LDOS will be Gaussian folded and shifted to
+        set Fermi energy to 0 eV. The latter can be avoided by setting
+        shift=False.
 
-        If you use fixmagmom=true, you will get two fermi-levels, one for each 
+        If you use fixmagmom=true, you will get two fermi-levels, one for each
         spin-setting. Normaly these will shifted individually to 0 eV. If you
         want to shift them as pair to the higher energy use bound=True.
         """
@@ -373,13 +424,13 @@ class RawLDOS:
             for key in ldbe:
                 for l in 'spd':
                     data.append(key + '(' + l + ')-weight')
-                if len(key) == 1: 
+                if len(key) == 1:
                     key = ' ' + key
-                s +=  ' ' + key + ':s     p        d       '
+                s += ' ' + key + ':s     p        d       '
             return s
 
         wfs = self.paw.wfs
-        
+
         if width is None:
             # unfolded ldos
             fmf = FMF(['Raw LDOS obtained from projector weights'])
@@ -405,8 +456,8 @@ class RawLDOS:
                     w = wfs.kd.weight_k[k]
                     for n in range(wfs.bd.nbands):
                         sum = 0.0
-                        print('%10.5f %6.4f %2d %5d' % (e_n[n], f_n[n], 
-                                                              s, k), end=' ', file=f) 
+                        print('%10.5f %6.4f %2d %5d' % (e_n[n], f_n[n],
+                                                        s, k), end=' ', file=f)
                         print('%6d %8.4f' % (n, w), end=' ', file=f)
                         for key in ldbe:
                             spd = ldbe[key][s, k, n]
@@ -421,8 +472,9 @@ class RawLDOS:
 
             gauss = Gauss(width)
             print(fmf.field('folding',
-                                  ['function: Gauss',
-                                   'width: ' + str(width) + ' [eV]']), end=' ', file=f)
+                            ['function: Gauss',
+                             'width: ' + str(width) + ' [eV]']),
+                  end=' ', file=f)
 
             data = ['energy: energy [eV]',
                     'spin index: s',
@@ -450,7 +502,7 @@ class RawLDOS:
                     efermi = self.paw.get_fermi_level()
             except:
                 # set Fermi level half way between HOMO and LUMO
-                hl = self.paw.occupations.get_homo_lumo(wfs)
+                hl = wfs.get_homo_lumo()
                 efermi = (hl[0] + hl[1]) * Hartree / 2
 
             eshift = 0.0
@@ -460,7 +512,7 @@ class RawLDOS:
 
             # set de to sample 4 points in the width
             de = width / 4
-            
+
             string = '# e[eV]     s  '
             string += append_weight_strings(ldbe, data)
 
@@ -476,16 +528,16 @@ class RawLDOS:
                 print('# Gauss folded, width=%g [eV]' % width, file=f)
                 if shift:
                     print('# shifted to Fermi energy = 0', file=f)
-                    print('# Fermi energy was', end=' ', file=f) 
+                    print('# Fermi energy was', end=' ', file=f)
                 else:
                     print('# Fermi energy', end=' ', file=f)
                 print(efermi, 'eV', file=f)
                 print(string, file=f)
 
                 # loop over energies
-                emax=emax+.5*de
-                e=emin
-                while e<emax:
+                emax = emax + 0.5 * de
+                e = emin
+                while e < emax:
                     val = {}
                     for key in ldbe:
                         val[key] = np.zeros((3))
@@ -498,18 +550,17 @@ class RawLDOS:
                             for key in ldbe:
                                 val[key] += w_i * ldbe[key][s, k, n]
 
-                    print('%10.5f %2d' % (e + eshift, s), end=' ', file=f) 
+                    print('%10.5f %2d' % (e + eshift, s), end=' ', file=f)
                     for key in val:
                         spd = val[key]
                         for l in range(3):
                             print('%8.4f' % spd[l], end=' ', file=f)
                     print(file=f)
                     e += de
-                            
 
         f.close()
 
-    def by_element_to_file(self, 
+    def by_element_to_file(self,
                            filename='ldos_by_element.dat',
                            width=None,
                            shift=True,
@@ -533,7 +584,7 @@ class LCAODOS:
     def get_orbital_pdos(self, M, ravel=True):
         """Get projected DOS from LCAO basis function M."""
         return self.get_subspace_pdos([M], ravel=ravel)
-    
+
     def get_atomic_subspace_pdos(self, a, ravel=True):
         """Get projected subspace DOS from LCAO basis on atom(s) a.
 
@@ -557,7 +608,7 @@ class LCAODOS:
         bd = wfs.bd
         kd = wfs.kd
         gd = wfs.gd
-        
+
         for kpt in wfs.kpt_u:
             assert not np.isnan(kpt.eps_n).any()
 
@@ -576,7 +627,7 @@ class LCAODOS:
                 bd.comm.broadcast(S_MM, 0)
             else:
                 S_MM = kpt.S_MM
-            
+
             if gd.comm.rank == 0:
                 S_mm = S_MM[Mvalues, :][:, Mvalues].copy()
                 iS_mm = np.linalg.inv(S_mm).copy()
@@ -585,7 +636,7 @@ class LCAODOS:
                 w_n = (np.conj(CS_nm) * CSiS_nm).sum(axis=1) * kpt.weight
                 w_skn[kpt.s, kpt.k, bd.beg:bd.end:bd.step] = w_n.real
                 eps_skn[kpt.s, kpt.k, bd.beg:bd.end:bd.step] = kpt.eps_n
-        
+
         for arr in [eps_skn, w_skn]:
             gd.comm.broadcast(arr, 0)
             bd.comm.sum(arr)
