@@ -1,9 +1,7 @@
-
 import numpy as np
 
-from gpaw import debug, parsize_domain, parsize_bands
+from gpaw import debug
 from gpaw import mpi
-from gpaw.utilities.blas import axpy
 
 """
 def dscf_find_lumo(paw,band):
@@ -136,11 +134,10 @@ def dscf_kpoint_overlaps(paw, phasemod=True, broadcast=True):
             psit0_nG = eirk_G[np.newaxis,...]*kpt0.psit_nG
 
             P0_ani = paw.wfs.pt.dict(bd.mynbands)
-            spos_ac = atoms.get_scaled_positions() % 1.0
             for a, P0_ni in P0_ani.items():
                 # Expanding the exponential exp(ikr)=exp(ikR)*exp(ik(r-R))
                 # and neglecting the changed P_ani integral exp(ik(r-R))~1
-                P0_ni[:] = np.exp(2j*np.pi*np.sum(spos_ac[a]*(k_c-k0_c), axis=0)) * kpt0.P_ani[a]
+                P0_ni[:] = np.exp(2j*np.pi*np.sum(paw.spos_ac[a]*(k_c-k0_c), axis=0)) * kpt0.P_ani[a]
 
             ## NB: No exp(ikr) approximate here, but has a parallelization bug
             #kpt0_rank, myu0 = kd.get_rank_and_index(kpt0.s, kpt0.k)
@@ -476,8 +473,7 @@ def dscf_load_band(filename, paw, molecule=None):
 
     # Find domain ranks for each atom
     atoms = paw.get_atoms()
-    spos_ac = atoms.get_scaled_positions() % 1.0
-    rank_a = gd.get_ranks_from_positions(spos_ac)
+    rank_a = gd.get_ranks_from_positions(paw.spos_ac)
     #my_atom_indices = np.argwhere(rank_a == gd.comm.rank).ravel()
     #assert np.all(my_atom_indices == paw.wfs.pt.my_atom_indices)
     assert r.dimension('nproj') == sum([setup.ni for setup in paw.wfs.setups])
@@ -533,7 +529,7 @@ def dscf_collapse_orbitals(paw, nbands_max='occupied', f_tol=1e-4,
     assert nbands_max >= n0 + norbitals, 'Too few bands to include occupations.'
     ncut = nbands_max-norbitals
 
-    if debug: mpi_debug('nbands_max=%d' % nbands_max) 
+    if debug: mpi_debug('nbands_max=%d' % nbands_max)
 
     paw.wfs.initialize_wave_functions_from_restart_file() # hurts memmory
 
@@ -590,7 +586,7 @@ def dscf_collapse_orbitals(paw, nbands_max='occupied', f_tol=1e-4,
     # Replace occupations class with a fixed variant (gets the magmom right) XXX?!?
     fermilevel, magmom = paw.occupations.fermilevel, paw.occupations.magmom
     paw.occupations = FermiDirac(paw.occupations.width * Hartree, paw.occupations.fixmagmom)
-    paw.occupations.set_fermi_level(fermilevel)
+    paw.occupations.fermilevel = fermilevel
     paw.occupations.magmom = magmom
     del fermilevel, magmom
 
