@@ -6,17 +6,19 @@ evaluation of exact exchange.
 """
 
 import numpy as np
+from ase.utils import basestring
 
-from gpaw.xc import XC
-from gpaw.xc.kernel import XCNull
-from gpaw.xc.functional import XCFunctional
-from gpaw.poisson import PoissonSolver
-from gpaw.utilities import hartree, pack, pack2, unpack, unpack2, packed_index
-from gpaw.utilities.tools import symmetrize
 from gpaw.atom.configurations import core_states
-from gpaw.lfc import LFC
-from gpaw.utilities.blas import gemm
 from gpaw.gaunt import gaunt
+from gpaw.lfc import LFC
+from gpaw.poisson import PoissonSolver
+from gpaw.transformers import Transformer
+from gpaw.utilities import hartree, pack, pack2, unpack, unpack2, packed_index
+from gpaw.utilities.blas import gemm
+from gpaw.utilities.tools import symmetrize
+from gpaw.xc import XC
+from gpaw.xc.functional import XCFunctional
+from gpaw.xc.kernel import XCNull
 
 
 class HybridXCBase(XCFunctional):
@@ -56,21 +58,27 @@ class HybridXCBase(XCFunctional):
             hybrid = 0.25
             omega = 0.11
             xc = XC('HYB_GGA_XC_HSE06')
-            
-        if isinstance(xc, str):
+
+        if isinstance(xc, (basestring, dict)):
             xc = XC(xc)
 
         self.hybrid = float(hybrid)
         self.xc = xc
         self.omega = omega
-        self.type = xc.type
 
-        XCFunctional.__init__(self, name)
+        XCFunctional.__init__(self, name, xc.type)
+
+    def todict(self):
+        return {'type': 'hybrid',
+                'name': self.name,
+                'hybrid': self.hybrid,
+                'xc': self.xc.todict(),
+                'omega': self.omega}
 
     def get_setup_name(self):
         return 'PBE'
 
-        
+
 class HybridXC(HybridXCBase):
     def __init__(self, name, hybrid=None, xc=None,
                  finegrid=False, unocc=False):
@@ -84,12 +92,12 @@ class HybridXC(HybridXCBase):
         self.finegrid = finegrid
         self.unocc = unocc
         HybridXCBase.__init__(self, name, hybrid, xc)
-        
+
     def calculate_paw_correction(self, setup, D_sp, dEdD_sp=None,
                                  addcoredensity=True, a=None):
         return self.xc.calculate_paw_correction(setup, D_sp, dEdD_sp,
                                                 addcoredensity, a)
-    
+
     def initialize(self, density, hamiltonian, wfs, occupations):
         assert wfs.kd.gamma
         self.xc.initialize(density, hamiltonian, wfs, occupations)
@@ -101,29 +109,39 @@ class HybridXC(HybridXCBase):
         self.exx_s = np.zeros(self.nspins)
         self.ekin_s = np.zeros(self.nspins)
         self.nocc_s = np.empty(self.nspins, int)
-        
-        if self.finegrid:
-            self.poissonsolver = hamiltonian.poisson
-            self.ghat = density.ghat
-            self.interpolator = density.interpolator
-            self.restrictor = hamiltonian.restrictor
-        else:
-            self.poissonsolver = PoissonSolver(eps=1e-11)
-            self.poissonsolver.set_grid_descriptor(density.gd)
-            self.poissonsolver.initialize()
-            self.ghat = LFC(density.gd,
-                            [setup.ghat_l for setup in density.setups],
-                            integral=np.sqrt(4 * np.pi), forces=True)
+
         self.gd = density.gd
-        self.finegd = self.ghat.gd
+        self.redistributor = density.redistributor
+
+        # XXX How do we construct a copy of the Poisson solver of the
+        # Hamiltonian?  We don't know what class it is, etc., but gd
+        # may differ.
+        self.poissonsolver = PoissonSolver(eps=1e-11)
+        #self.poissonsolver = hamiltonian.poisson
+
+        if self.finegrid:
+            self.finegd = self.gd.refine()
+            # XXX Taking restrictor from Hamiltonian will not work in PW mode,
+            # will it?  I think this supports only real-space mode.
+            #self.restrictor = hamiltonian.restrictor
+            self.restrictor = Transformer(self.finegd, self.gd, 3)
+            self.interpolator = Transformer(self.gd, self.finegd, 3)
+        else:
+            self.finegd = self.gd
+
+        self.ghat = LFC(self.finegd,
+                        [setup.ghat_l for setup in density.setups],
+                        integral=np.sqrt(4 * np.pi), forces=True)
+        self.poissonsolver.set_grid_descriptor(self.finegd)
 
     def set_positions(self, spos_ac):
-        if not self.finegrid:
-            self.ghat.set_positions(spos_ac)
-    
+        self.ghat.set_positions(spos_ac)
+
     def calculate(self, gd, n_sg, v_sg=None, e_g=None):
         # Normal XC contribution:
         exc = self.xc.calculate(gd, n_sg, v_sg, e_g)
+        # Note that the quantities passed are on the density/Hamiltonian grids!
+        # They may be distributed differently from own quantities.
         self.ekin = self.kpt_comm.sum(self.ekin_s.sum())
         return exc + self.kpt_comm.sum(self.exx_s.sum())
 
@@ -135,10 +153,9 @@ class HybridXC(HybridXCBase):
                                             Htpsit_nG=None, dH_asp=None):
         if kpt.f_n is None:
             return
-        
+
         deg = 2 // self.nspins   # Spin degeneracy
         hybrid = self.hybrid
-        
         P_ani = kpt.P_ani
         setups = self.setups
 
@@ -165,6 +182,10 @@ class HybridXC(HybridXCBase):
         exx = 0.0
         ekin = 0.0
 
+        # XXXX nbands can be different numbers on different cpus!
+        # That means some will execute the loop and others not.
+        # And deadlocks with augment-grids.
+
         # Determine pseudo-exchange
         for n1 in range(nbands):
             psit1_G = psit_nG[n1]
@@ -175,10 +196,13 @@ class HybridXC(HybridXCBase):
 
                 # Double count factor:
                 dc = (1 + (n1 != n2)) * deg
-                
                 nt_G, rhot_g = self.calculate_pair_density(n1, n2, psit_nG,
                                                            P_ani)
                 vt_g[:] = 0.0
+                # XXXXX This will go wrong because we are solving the
+                # Poisson equation on the distribution of gd, not finegd
+                # Or maybe it's fixed now
+
                 self.poissonsolver.solve(vt_g, -rhot_g,
                                          charge=-float(n1 == n2),
                                          eps=1e-12,
@@ -231,12 +255,12 @@ class HybridXC(HybridXCBase):
                 ekin -= np.dot(kpt.f_n[:nbands], h_nn.diagonal())
 
                 dH_p = dH_asp[a][kpt.s]
-            
+
             # Get atomic density and Hamiltonian matrices
             D_p = self.density.D_asp[a][kpt.s]
             D_ii = unpack2(D_p)
             ni = len(D_ii)
-            
+
             # Add atomic corrections to the valence-valence exchange energy
             # --
             # >  D   C     D
@@ -254,7 +278,7 @@ class HybridXC(HybridXCBase):
                         dH_p[p12] -= 2 * hybrid / deg * A / ((i1 != i2) + 1)
                     ekin += 2 * hybrid / deg * D_ii[i1, i2] * A
                     exx -= hybrid / deg * D_ii[i1, i2] * A
-            
+
             # Add valence-core exchange energy
             # --
             # >  X   D
@@ -276,16 +300,16 @@ class HybridXC(HybridXCBase):
         if not hasattr(kpt, 'vxx_ani'):
             return
 
-        if self.gd.comm.rank > 0:
-            H_nn[:] = 0.0
-            
+        #if self.gd.comm.rank > 0:
+        #    H_nn[:] = 0.0
+
         nocc = self.nocc_s[kpt.s]
         nbands = len(kpt.vt_nG)
         for a, P_ni in kpt.P_ani.items():
             H_nn[:nbands, :nbands] += symmetrize(np.inner(P_ni[:nbands],
                                                           kpt.vxx_ani[a]))
-        self.gd.comm.sum(H_nn)
-        
+        #self.gd.comm.sum(H_nn)
+
         H_nn[:nocc, nocc:] = 0.0
         H_nn[nocc:, :nocc] = 0.0
 
@@ -297,7 +321,7 @@ class HybridXC(HybridXCBase):
             D_ii = np.outer(P1_i, P2_i.conj()).real
             D_p = pack(D_ii)
             Q_aL[a] = np.dot(D_p, self.setups[a].Delta_pL)
-            
+
         nt_G = psit_nG[n1] * psit_nG[n2]
 
         if self.finegd is self.gd:
@@ -317,7 +341,7 @@ class HybridXC(HybridXCBase):
             return
 
         nocc = self.nocc_s[kpt.s]
-        
+
         if calculate_change:
             for x, n in enumerate(n_x):
                 if n < nocc:
@@ -327,11 +351,12 @@ class HybridXC(HybridXCBase):
         else:
             for a, c_xi in c_axi.items():
                 c_xi[:nocc] += kpt.vxx_ani[a][:nocc]
-        
+
     def rotate(self, kpt, U_nn):
         if kpt.f_n is None:
             return
 
+        U_nn = U_nn.copy()
         nocc = self.nocc_s[kpt.s]
         if len(kpt.vt_nG) == nocc:
             U_nn = U_nn[:nocc, :nocc]
@@ -341,7 +366,7 @@ class HybridXC(HybridXCBase):
         for v_nii in kpt.vxx_anii.values():
             gemm(1.0, v_nii.copy(), U_nn, 0.0, v_nii)
 
-        
+
 def atomic_exact_exchange(atom, type='all'):
     """Returns the exact exchange energy of the atom defined by the
        instantiated AllElectron object 'atom'
@@ -367,7 +392,7 @@ def atomic_exact_exchange(atom, type='all'):
     # Arrays for storing the potential (times radius)
     vr = np.zeros(atom.N)
     vrl = np.zeros(atom.N)
-    
+
     # do actual calculation of exchange contribution
     Exx = 0.0
     for j1 in nstates:
@@ -437,7 +462,7 @@ def constructX(gen):
 
     # potential times radius
     vr = np.zeros(N)
-        
+
     # initialize X_ii matrix
     X_ii = np.zeros((Nvi, Nvi))
 
@@ -462,17 +487,17 @@ def constructX(gen):
             i2 = 0
             for jv2 in range(Njval):
                 lv2 = lv_j[jv2]
-                
+
                 # electron density 2
                 n2c = uv_j[jv2] * uc_j[jc] * dr
                 n2c[1:] /= r[1:]
-            
+
                 # sum expansion in angular momenta
                 for l in range(min(lv1, lv2) + lc + 1):
                     # Int density * potential * r^2 * dr:
                     hartree(l, n2c, r, vr)
                     nv = np.dot(n1c, vr)
-                    
+
                     # expansion coefficients
                     A_mm = X_ii[i1:i1 + 2 * lv1 + 1, i2:i2 + 2 * lv2 + 1]
                     for mc in range(2 * lc + 1):
